      subroutine arma(task,p,q,ndraw,nec,phi,vareps,theta,covar,dcovar,
     *   dphi,dvareps,dtheta,branchnum,time,cumulative,ipt,jpt)
 
c     Subroutine to create the covariance matrix of an ARMA or
c     Cumulative AR(1) process, and to unravel derivatives.
c     Special care is taken to exclude rows and columns for which
c     time()=NaN, i.e., outcomes in which the residual does not enter.
c
c     Note:  the covariance matrix covar is assigned, not accumulated.
c     When unraveling derivatives wrt covar, dcovar will be destroyed.
c
c     Constantijn Panis,  23 June 1997
c     Last change:  take care of CAR(1)
c     Last change:  also fix dphi if phi=0 in AR(1)
c     Last change:  process truly starts at time=1
 
      implicit none
      integer task,p,q,ndraw,nec,ipt,jpt,blocksize,drawpos,necpos,
     *   pos,n,n1,n2,i,j,realcontig
      double precision phi(p),vareps,theta(q),covar(ndraw,ndraw),
     *   dcovar(ndraw,ndraw),dphi(p),dvareps,dtheta(q),
     *   gamma0,ti,tj,lag,temp,temp1,temp2,innerp
      real branchnum(nec),time(nec)
      logical cumulative
 
      if(ndraw.eq.0)goto 999
      drawpos=1
      necpos=1
      if(task.eq.2)goto 200
 
c     Create covariance matrix:
      call zerodp(ndraw*ndraw,covar)
      if(p.eq.1.and.q.eq.0)gamma0=vareps/(1.d0-phi(1)*phi(1))
  110 if(necpos.gt.nec)goto 999
c     Determine the number of outcomes that are autocorrelated:
      blocksize=realcontig(branchnum(necpos),branchnum(necpos),
     *   nec-necpos+1)
      if(branchnum(necpos).eq.0.)then
c        ARMA residual does not enter this block:
         necpos=necpos+blocksize
         goto 110
      elseif(cumulative)then
         if(p.ne.1.or.q.ne.0)call bug(ipt,jpt)
c        Cumulative AR(1)
         n=necpos-drawpos
         do 120 i=drawpos,drawpos+blocksize-1
c           Note that the time variable is interpreted in absolute sense!
            ti=dble(time(n+i))
c           Alpha compiler does not allow negative numbers to dp power
c           If phi<0, t1 is integer-valued; else, ti may be anything.
c           May take absolute value; does not matter here.
c           covar(i,i)=gamma0*(1-phi(1)**(2.d0*ti))
            covar(i,i)=gamma0*(1-abs(phi(1))**(2.d0*ti))
            do 130 j=drawpos,i-1
c              Note that ti need not be larger than tj
               tj=dble(time(n+j))
c              Note that phi(1)=0 and ti=tj would give problems;
c              if ti=tj, the residuals are the same and the
c              covariance thus equal to the variance.
               if(tj.lt.ti)then
c                 Alpha compiler does not allow negative numbers to dp power
c                 temp = covar(j,j)*phi(1)**(ti-tj)
                  temp = abs(phi(1))**(ti-tj)
                  if(phi(1).lt.0.and.mod(int(ti-tj),2).eq.1)temp=-temp
                  temp = covar(j,j)*temp
               elseif(ti.eq.tj)then
                  temp = covar(j,j)
               else
c                 Alpha compiler does not allow negative numbers to dp power
c                 temp = covar(i,i)*phi(1)**(tj-ti)
                  temp = abs(phi(1))**(tj-ti)
                  if(phi(1).lt.0.and.mod(int(tj-ti),2).eq.1)temp=-temp
                  temp = covar(i,i)*temp
               endif
               covar(i,j) = temp
               covar(j,i) = temp
  130       continue
  120    continue
      elseif(p.eq.1.and.q.eq.0)then
c        AR(1) -- treat differently, as time variables need not be
c        integer-valued (provided phi(1)>0).
         n=necpos-drawpos
         do 140 i=drawpos,drawpos+blocksize-1
            covar(i,i)=gamma0
            do 150 j=drawpos,i-1
               lag=dble(abs(time(n+i)-time(n+j)))
c              Note that phi(1)=0 and lag=0 would give problems;
c              if lag=0, the residuals are the same and the
c              covariance thus equal to the variance, gamma0.
               if(lag.eq.0.d0)then
                  temp = gamma0
               else
c                 Alpha compiler does not allow negative numbers to dp power
c                 temp = gamma0*phi(1)**lag
                  temp = abs(phi(1))**lag
                  if(phi(1).lt.0.and.mod(int(lag),2).eq.1)temp=-temp
                  temp = gamma0*temp
               endif
               covar(i,j) = temp
               covar(j,i) = temp
  150       continue
  140    continue
      else
c        For each level (block) within which autocorrelation exists, place
c        covariances in the big covariance matrix, then move things
c        in the right place and restore block-diagonality.
         call fillarma(1,p,q,blocksize,phi,vareps,theta,
     *      covar(drawpos,drawpos),time(necpos),dcovar(drawpos,drawpos),
     *      dphi,dvareps,dtheta,ipt,jpt)
         if(blocksize.eq.ndraw)goto 999
         do 160 i=blocksize,1,-1
            pos = drawpos+ndraw*(drawpos-1)+blocksize*(i-1)
            n1=mod(pos-1,ndraw)+1
            n2=(pos-1)/ndraw+1
            call copy8r(covar(n1,n2),covar(drawpos,drawpos+i-1),
     *         blocksize)
c           Zero-out elements off the block-diagonal:
            if(drawpos.gt.1)call zerodp(drawpos-1,covar(1,drawpos+i-1))
            if(drawpos+blocksize.le.ndraw)call
     *         zerodp(ndraw-drawpos-blocksize+1,
     *         covar(drawpos+blocksize,drawpos+i-1))
  160    continue
      endif
      drawpos=drawpos+blocksize
      necpos=necpos+blocksize
      goto 110
 
c     Collapse derivatives.  Allow for dcovar being nonsymmetric,
c     but make use of the fact that covar is symmetric.
  200 continue
  210 if(necpos.gt.nec)then
         if(p.eq.1.and.q.eq.0)dvareps=dvareps/vareps
          goto 999
      endif
c     Determine the number of outcomes that are autocorrelated:
      blocksize=realcontig(branchnum(necpos),branchnum(necpos),
     *   nec-necpos+1)
      if(branchnum(necpos).eq.0.)then
c        ARMA residual does not enter this block:
         necpos=necpos+blocksize
         goto 210
      elseif(cumulative)then
         if(p.ne.1.or.q.ne.0)call bug(ipt,jpt)
c        Cumulative AR(1)
         n=necpos-drawpos
         if(phi(1).eq.0.d0)then
            do 220 i=drawpos,drawpos+blocksize-1
               dvareps=dvareps+dcovar(i,i)*covar(i,i)
               ti=dble(time(n+i))
               do 230 j=drawpos,i-1
c                 Note that ti need not be larger than tj
                  tj=dble(time(n+j))
                  if(ti.eq.tj)then
                     dvareps=dvareps+vareps*(dcovar(i,j)+dcovar(j,i))
                  elseif(abs(ti-tj).eq.1.d0)then
                     dphi(1)=dphi(1)+vareps*(dcovar(i,j)+dcovar(j,i))
                  endif
  230          continue
  220       continue
         else
            temp1=2.d0*phi(1)/(1.d0-phi(1)*phi(1))
            temp2=2.d0/(phi(1)*(1-phi(1)*phi(1)))
            do 240 i=drawpos,drawpos+blocksize-1
               dvareps=dvareps+innerp(dcovar(drawpos,i),
     *            covar(drawpos,i),blocksize)
               ti=dble(abs(time(n+i)-time(n+drawpos)))+1.d0
               dphi(1)=dphi(1)+dcovar(i,i)*
     *            ((2*ti/phi(1)+temp1)*covar(i,i)-ti*temp2*vareps)
               do 250 j=drawpos,i-1
c                 Note that ti need not be larger than tj
                  tj=dble(abs(time(n+j)-time(n+drawpos)))+1.d0
c                 Alpha compiler does not allow negative numbers to dp power
c                 temp=((ti+tj)/phi(1)+temp1)*covar(i,j)
c    *               -vareps*temp2*min(ti,tj)*phi(1)**abs(ti-tj)
                  temp=abs(phi(1))**abs(ti-tj)
                  if(phi(1).lt.0.d0.and.mod(int(abs(ti-tj)),2).eq.1)
     *               temp=-temp
                  temp=((ti+tj)/phi(1)+temp1)*covar(i,j)
     *               -vareps*temp2*min(ti,tj)*temp
                  dphi(1)=dphi(1)+temp*(dcovar(i,j)+dcovar(j,i))
  250          continue
  240       continue
        endif
      elseif(p.eq.1.and.q.eq.0)then
c        AR(1) -- treat differently, as time variables need not be
c        integer-valued (provided phi(1)>0).
c        Special case phi(1)=0 could be treated differently, but it
c        causes very few problems to do things more generally.
         temp1=2.d0*phi(1)/(1.d0-phi(1)*phi(1))
         n=necpos-drawpos
         do 260 i=drawpos,drawpos+blocksize-1
            dvareps=dvareps+innerp(dcovar(drawpos,i),covar(drawpos,i),
     *         blocksize)
            dphi(1)=dphi(1)+dcovar(i,i)*covar(i,i)*temp1
            do 270 j=drawpos,i-1
               lag=dble(abs(time(n+i)-time(n+j)))
               if(phi(1).eq.0.d0)then
                  if(lag.eq.1.d0)dphi(1)=dphi(1)+
     *               vareps*(dcovar(i,j)+dcovar(j,i))
               else
                  temp=(dcovar(i,j)+dcovar(j,i))*covar(i,j)
                  dphi(1)=dphi(1)+temp*(lag/phi(1)+temp1)
               endif
  270       continue
  260    continue
      else
c        For each block, move derivatives around such that they are
c        contiguous, as-if the matrix were not block-diagonal.  This
c        will destroy dcovar!  Then unravel derivatives.
         if(blocksize.ne.ndraw)then
            do 280 i=2,blocksize
               pos = drawpos+ndraw*(drawpos-1)+blocksize*(i-1)
               n1=mod(pos-1,ndraw)+1
               n2=(pos-1)/ndraw+1
               call copy8(dcovar(drawpos,drawpos+i-1),dcovar(n1,n2),
     *            blocksize)
  280       continue
         endif
         call fillarma(2,p,q,blocksize,phi,vareps,theta,
     *      covar(drawpos,drawpos),time(necpos),
     *      dcovar(drawpos,drawpos),dphi,dvareps,dtheta,ipt,jpt)
      endif
      drawpos=drawpos+blocksize
      necpos=necpos+blocksize
      goto 210
 
  999 return
      end
 
