      subroutine chol2corr(task,dim,chol,corr,dcorr,dchol)
 
c     Subroutine to convert a Cholesky decomposed covariance vector
c     into a vector of standard deviations and correlations, and to
c     unravel derivatives.
c
c     The vectors are stored as follows.  Diagonal elements are stored
c     first, followed by off-diagonal elements.  E.g., with dim=4:
c
c     ( sig1                   )     ( d11             )
c     ( rho21 sig2             )  or ( d21 d22         )
c     ( rho31 rho32 sig3       )     ( d31 d32 d33     )
c     ( rho41 rho42 rho43 sig4 )     ( d41 d42 d43 d44 )
c
c     are stored:
c     sig1 sig2 sig3 sig4 rho21 rho31 rho41 rho32 rho42 rho43
c      d11  d22  d33  d44   d21   d31   d41   d32   d42   d43
c
c     Input and output vectors may NOT share the same memory space.
c     If task=1, derivatives are not used;
c     if task=2, the original vector of standard deviations and
c     correlations is used.
c
c     If any one of the standard deviations is zero, correlations in
c     the corresponding row and column are theoretically indeterminate;
c     this routine does not change those correlations.
c
c     Constantijn Panis, 22 January 1994
 
      implicit none
      integer task,dim,i,j,k,ij,ik,sq2cmp
      double precision chol(dim*(dim+1)/2),corr(dim*(dim+1)/2),
     *   dchol(dim*(dim+1)/2),dcorr(dim*(dim+1)/2),cov,temp,
     *   dvarii,dcovik
 
      goto(100,200)task
 
  100 do 110 i=1,dim
c        Variance i,i is the innerproduct of row i with itself:
         temp=0.d0
         do 120 k=1,i
            temp=temp+chol(sq2cmp(i,k,dim))**2
  120    continue
         corr(i)=dsqrt(temp)
         do 130 j=1,i-1
            ij=sq2cmp(i,j,dim)
c           Covariance i,j is the innerproduct of rows i and j of the
c           Cholesky decomposed matrix:
            cov=0.d0
            do 140 k=1,j
               cov=cov+chol(sq2cmp(i,k,dim))*chol(sq2cmp(j,k,dim))
  140       continue
            temp=corr(i)*corr(j)
            if(temp.ne.0.d0)corr(ij)=cov/temp
  130    continue
  110 continue
      goto 999
 
c     Unravel derivatives wrt standard deviations and correlations into
c     wrt Cholesky decomposed parameters.  The derivative wrt Chol(i,j)
c     is twice the innerproduct of column j of the derivative wrt the
c     covariance matrix and column i of the Cholesky decomposition.
c     Neither matrix is available, but can be constructed.  In
c     particular, see subroutine cov2corr (task=2).
  200 do 210 i=1,dim
         do 220 j=1,i
            ij=sq2cmp(i,j,dim)
c           compute derivative wrt variance (i,i):
            dvarii=-dcorr(i)*corr(i)
            do 230 k=1,dim
               if(k.eq.i)goto 230
               ik=sq2cmp(i,k,dim)
               dvarii=dvarii+corr(ik)*dcorr(ik)
  230       continue
            temp=corr(i)*corr(i)
            if(temp.ne.0.d0)temp=1.d0/temp
            dvarii=-.5d0*temp*dvarii
            dchol(ij)=dvarii*chol(ij)
c           Upper triangle of Chol is zero, so innerproduct can start at j:
            do 240 k=j,dim
c              Compute derivative wrt covariance (k,i):
               if(k.eq.i)then
                  dcovik=dvarii
               else
                  temp=corr(k)*corr(i)
                  if(temp.ne.0.d0)temp=1.d0/temp
                  dcovik=temp*dcorr(sq2cmp(k,i,dim))
               endif
               dchol(ij)=dchol(ij)+dcovik*chol(sq2cmp(j,k,dim))
  240       continue
  220    continue
  210 continue
 
  999 return
      end
