      subroutine compcdm(task,intdist,intdim,intdimc,posi1,posi2,posi4,
     *   bigposi2,sumintdim,sumdim2,sumdim4,dimii,thisdraw,ndraw,
     *   distdim,dimused,ipt,jpt,status,sigii,dsigii,cdm,ddcdm,
     *   dcdmdsigii,ifdrv)
 
c     Subroutine to (task=1) compute Cholesky decomposed matrix for an
c     integrated distribution and (task=2) unravel derivatives.
c
c     Arguments:
c
c     task   [in]   1 = compute Cholesky decomposed matrix
c                   2 = unravel derivatives
c     intdist [in]  rank number of integrated distribution
c     intdim  [out] dimension of integrated distribution
c     intdimc [out] compressed dimension of integrated distribution
c     posi1   [in]  position of integrated distribution in
c                   one-dimensional non-expanded arrays of dimension
c                   sumintdim (such as dimused, intpntr intpnts, ...)
c     posi2   [in]  idem, two-dimensional arrays of dimension sumdim2
c                   (such as cdm, ddcdm)
c     posi4   [in]  idem, four-dimensional arrays of dimension sumdim4
c                   (only dcdmdsigii)
c     bigposi2 [in] position of integrated distribution and currently
c                   integrated draw in sigii, dsigii
c
c     Constantijn Panis, 20 March 2002
c     Last change: lots of arrays only pertain to integrated distributions
 
      implicit none
      integer task,intdist,intdim,intdimc,sumintdim,sumdim2,sumdim4,
     *   dimii,thisdraw(intdist),ndraw(intdist),distdim(intdist),
     *   dimused(sumintdim),ipt,jpt,status,j,bigdim,
     *   posi1,posi2,posi4,bigposi2
      double precision sigii(dimii),dsigii(dimii),
     *   cdm(sumdim2),ddcdm(sumdim2),dcdmdsigii(sumdim4)
      logical ifdrv
 
      bigdim=distdim(intdist)*ndraw(intdist)
      if(task.eq.2)goto 200
 
c     Move to position in sigii that contains covariance matrix of
c     current draw (differs across draws if there is conditioning on
c     continuous outcomes.)
c     Make sigii contiguous (if necessary):
      if(ndraw(intdist).gt.1)then
         do 120 j=2,intdim
            call copy8(sigii(bigposi2+(j-1)*bigdim),
     *         sigii(bigposi2+(j-1)*intdim),intdim)
  120    continue
      endif
c     Note that intdimc=0 costs some overhead, but causes no problems.
      if(intdimc.lt.intdim)call sqz_covar(1,intdim,intdimc,
     *   sigii(bigposi2),sigii(bigposi2),dsigii(bigposi2),
     *   dsigii(bigposi2),dimused(posi1))
      call compchol(1,sigii(bigposi2),cdm(posi2),intdimc,ifdrv,
     *   dcdmdsigii(posi4),ddcdm(posi2),dsigii(bigposi2),status)
      if(status.gt.0)goto 800
      goto 999
 
c     Unravel derivatives:
c     Note that dsigii is assigned here, not accumulated.
  200 call compchol(2,sigii(bigposi2),cdm(posi2),intdimc,ifdrv,
     *   dcdmdsigii(posi4),ddcdm(posi2),dsigii(bigposi2),status)
      if(intdimc.lt.intdim)call sqz_covar(2,intdim,intdimc,
     *   sigii(bigposi2),sigii(bigposi2),dsigii(bigposi2),
     *   dsigii(bigposi2),dimused(posi1))
      if(ndraw(intdist).gt.1)then
         do 220 j=intdim,2,-1
            call copy8r(dsigii(bigposi2+(j-1)*intdim),
     *         dsigii(bigposi2+(j-1)*bigdim),intdim)
c           Zero-out irrelevant elements to maintain block-diagonality
            call zerodp(intdim*(thisdraw(intdist)-1),dsigii(
     *         bigposi2+bigdim*(j-1)-(thisdraw(intdist)-1)*intdim))
            if(thisdraw(intdist).lt.ndraw(intdist))call zerodp(
     *         (ndraw(intdist)-thisdraw(intdist))*intdim,
     *         dsigii(bigposi2+bigdim*(j-1)+intdim))
  220    continue
c        First column: elements above dsigii(bigposi2) will always
c        already be zero.  Just deal with elements below block.
         if(thisdraw(intdist).lt.ndraw(intdist))call zerodp(
     *      (ndraw(intdist)-thisdraw(intdist))*intdim,
     *      dsigii(bigposi2+intdim))
      endif
      goto 999
 
  800 if(ipt.gt.0)write(8,801)
      if(jpt.gt.0)write(6,801)
  801 format(/,'The function evaluation failed because the covariance ',
     *   'matrix of an',/,'integrated distribution is not strictly ',
     *   'positive definite.  There are',/,'two possibilities:',//,
     *   '(1) A standard deviation became so small or a correlation ',
     *   'so close',/,4x,'to -1 or 1 that machine imprecision set ',
     *   'them to 0, -1, or 1',/,4x,'(respectively), i.e., the ',
     *   'resulting covariance matrix became',/,4x,'invalid.  This ',
     *   'indicates that your model may be underidentified.',/,4x,
     *   '(If a residual from the integrated distribution appears as',/,
     *   4x,'non-integrated residual in a continuous model, the ',
     *   'conditional',/,4x,'covariance matrix may not be full-rank, ',
     *   'with the same result.)',//,'(2) By default, standard ',
     *   'deviations are restricted to be strictly',/,4x,'positive ',
     *   'and correlations to be strictly between -1 and 1.  This',/,4x,
     *   'is a necessary, but not sufficient condition for a valid ',
     *   '(positive',/,4x,'definite) covariance matrix.  The search ',
     *   'path may have resulted in',/,4x,'an invalid covariance ',
     *   'matrix.  If the program continues searching,',/,4x,'there ',
     *   'is no problem.  Otherwise, try the "SEARCH=CHOLESKY;" option',
     *   /,4x,'as part of the distributions definition.')
  999 return
      end
