      subroutine compute_xb1(task,numterm,thisterm,nx1,nsb,beta,
     *   lambda,xbeta1,xbeta2,d1beta,d1lambda,d1xbeta,
     *   d2beta,d2lambda,d2xbeta,censor,x1)
 
c     Subroutine to compute a regressor set for hazard models,
c     and to unravel derivatives.
c
c     task = 1: compute interval-level XBeta
c            2: unravel
c            3: compute spell-level XBeta
c            4: unravel
c            5: assign parameter as interval-level regressor set
c            6: unravel
c            7: assign parameter as spell-level regressor set
c            8: unravel
c
c     Note: nsb is the number of time-varying intervals, whether used or
c     not.  Fewer may be used if there were time marks beyond the end of
c     the spell.  If there was no time mark corresponding to durvar1,
c     one interval is used twice.  When unravelling derivatives, all nsb
c     derivatives are used.  Make sure unused ones are zero.
c
c     Constantijn Panis, 16 October 2001
c     Last change: code back from nu to nsb.  Version 2.59 was wrong.
 
      implicit none
      integer task,censor,numterm,thisterm,nx1,nsb,i,j,num
      double precision beta(nx1),lambda,xbeta1(nsb),
     *   xbeta2(nsb,numterm),d1beta(nx1),d1lambda,d1xbeta(nsb),
     *   d2beta(nx1),d2lambda,d2xbeta(nsb),temp,dpvecsum,innerp84
c     should be x1(nx1,nsb) or x1(nx1) if interacted with slope or residual
      real x1(nx1,nsb)
 
c     Compute XBeta or unravel derivatives?
      if(task.eq.1)goto 100
      if(task.eq.2)goto 200
      if(task.eq.3)goto 300
      if(task.eq.4)goto 400
      if(task.eq.5)goto 500
      if(task.eq.6)goto 600
      if(task.eq.7)goto 700
      if(task.eq.8)goto 800
      call bug(1,1)
 
c     Compute and assign interval-level XBeta:
  100 do 110 i=1,nsb
         xbeta1(i) = innerp84(beta,x1(1,i),nx1)
  110 continue
      goto 999
 
c     Unravel derivatives from multiplied XB (accumulate, don't assign):
  200 if(numterm.eq.1)then
c        Simple: derivative is derivative wrt XB times X:
         do 210 i=1,nsb
            do 215 j=1,nx1
               d1beta(j) = d1beta(j) + d1xbeta(i)*dble(x1(j,i))
               if(censor.eq.0)d2beta(j) = d2beta(j) +
     *            d2xbeta(i)*dble(x1(j,i))
  215       continue
  210    continue
      else
c        Harder: xbeta consists of multiple terms:
         do 220 i=1,nsb
            temp = 1.d0
            do 230 num=1,numterm
               if(num.ne.thisterm)temp = temp*xbeta2(i,num)
  230       continue
            do 240 j=1,nx1
               d1beta(j) = d1beta(j) + temp*d1xbeta(i)*dble(x1(j,i))
               if(censor.eq.0)d2beta(j) = d2beta(j) +
     *            temp*d2xbeta(i)*dble(x1(j,i))
  240       continue
  220    continue
      endif
      goto 999
 
c     Compute and assign spell-level XBeta:
  300 xbeta1(1) = innerp84(beta,x1,nx1)
      goto 999
 
c     Unravel derivatives from multiplied XB (accumulate, don't assign):
  400 if(numterm.eq.1)then
c        Simple: derivative is derivative wrt XB times X:
         call add488(x1,d1beta,d1xbeta(1),nx1)
         if(censor.eq.0)call add488(x1,d2beta,d2xbeta(1),nx1)
      else
c        Harder: xbeta consists of multiple terms:
         temp = 1.d0
         do 410 num=1,numterm
            if(num.ne.thisterm)temp = temp*xbeta2(1,num)
  410    continue
         call add488(x1,d1beta,temp*d1xbeta(1),nx1)
         if(censor.eq.0)call add488(x1,d2beta,temp*d2xbeta(1),nx1)
      endif
      goto 999
 
c     Assign parameter to xbeta as if it were an interval-level regressor set:
  500 call put(lambda,xbeta1,nsb)
      goto 999
 
c     Unravel dxbeta wrt parameter.
  600 if(numterm.eq.1)then
c        The regressor set was a parameter only:
         d1lambda = d1lambda + dpvecsum(d1xbeta,nsb)
         if(censor.eq.0)d2lambda = d2lambda + dpvecsum(d2xbeta,nsb)
      else
c        Harder: parameter is part of interaction with numterm terms:
         do 610 i=1,nsb
            temp = 1.d0
            do 620 num=1,numterm
               if(num.ne.thisterm)temp = temp*xbeta2(i,num)
  620       continue
            d1lambda = d1lambda + temp*d1xbeta(i)
            if(censor.eq.0)d2lambda = d2lambda + temp*d2xbeta(i)
  610    continue
      endif
      goto 999
 
c     Assign parameter to xbeta as if it were an spell-level regressor set:
  700 xbeta1(1) = lambda
      goto 999
 
c     Unravel dxbeta wrt parameter.
  800 if(numterm.eq.1)then
c        The regressor set was a parameter only:
         d1lambda = d1lambda + d1xbeta(1)
         if(censor.eq.0)d2lambda = d2lambda + d2xbeta(1)
      else
c        Harder: parameter is part of interaction with numterm terms:
         temp = 1.d0
         do 810 num=1,numterm
            if(num.ne.thisterm)temp = temp*xbeta2(1,num)
  810    continue
         d1lambda = d1lambda + temp*d1xbeta(1)
         if(censor.eq.0)d2lambda = d2lambda + temp*d2xbeta(1)
      endif
      goto 999
 
  999 return
      end
