      subroutine define_rset(numrset,maxnumrset,nold,rsetbegin,rsetnum,
     *   rsetname,maxref,rsetref,nfull,xnodepos,maxxnode,xnode,
     *   oldname,line,il,ipt,jpt)
 
c     Subroutine to define a set of regressors or expressions.
c     All will be stored in xnode.  At the time a regressor set is
c     defined, the applicable data structure is not known yet.
c     Subroutine figure_rset will figure out the exact positions.
c
c     The syntax is:
c        DEFINE REGRESSOR SET [name];
c           [REFERENCE = [n...n];]
c           VAR = [varname...expr...];
c     where name is a variable name, and expr an expression mostly
c     involving variable names, including dummy and spline
c     transformations.
c
c     Constantijn Panis, 20 April 2000
c     Last change: Require a VAR= statement.
 
      implicit none
      integer numrset,maxnumrset,nold,rsetbegin(maxnumrset),
     *   rsetnum(maxnumrset),maxref,rsetref(maxref),nfull,
     *   xnodepos,maxxnode,ipt,jpt,i1,i2,il,nref,beginref,status
      real xnode(maxxnode)
      logical smtxt,dupchar12
      character oldname(nold)*8,line*128,rsetname(maxnumrset)*12
 
      numrset = numrset+1
      if(numrset.gt.maxnumrset)call bug(ipt,jpt)
      rsetbegin(numrset)=(nfull+1)*(maxxnode+1)+xnodepos+1
c     Locate begin point of reference numbers:
      call find_br(0,numrset,maxref,rsetref,beginref,ipt,jpt)
      nref=0
 
c     Is the regressor set named or referenced?
      call getword(4,line,il,i1,i2,status)
      if(status.gt.0)goto 800
      if(line(i1:i2).eq.';')then
         rsetname(numrset) = '(unnamed)'
      else
         rsetname(numrset) = line(i1:i2)
c        Check that this name has not been used yet.
         if(dupchar12(rsetname(numrset),rsetname,numrset-1))goto 810
         if(.not.smtxt(4,line,il,';'))goto 800
      endif
 
      if(smtxt(4,line,il,'REFerence='))then
         if(nref.gt.0)goto 800
c        REF = [n...n];
         call readrefs(maxref,beginref,rsetref,nref,line,il,ipt,jpt)
      endif
 
c     Check that the set has been named or referenced
      if(nref.eq.0.and.rsetname(numrset).eq.'(unnamed)')goto 800
 
      if(.not.smtxt(4,line,il,'VARiables='))goto 800
c     VAR = varname...expr...;
      call readexpr(2,0,0,0,rsetnum(numrset),nold,maxxnode,xnodepos,
     *   xnode,0,0,0,0,0,oldname,line,il,ipt,jpt,0,';')
      nfull=nfull+rsetnum(numrset)
 
      goto 999
 
  800 if(ipt.gt.0)then
         call caretline(8,line,il)
         write(8,801)
      endif
      if(jpt.gt.0)then
         call caretline(6,line,il)
         write(6,801)
      endif
  801 format('Syntax error encountered in definition of regressor ',
     *   'set.',/,'The syntax is:  DEFINE REGRESSOR SET [name];',/,
     *   16x,'[REF = [n...n];]',/,16x,'VAR = [varnames];',
     *   /,'where varnames may be variable names or expressions.',/,
     *   'Note that you must name or assign reference numbers to the',/,
     *   'regressor set.')
      stop
  810 if(ipt.gt.0)then
         call caretline(8,line,il)
         write(8,811)
      endif
      if(jpt.gt.0)then
         call caretline(6,line,il)
         write(6,811)
      endif
  811 format(/,'ERROR: you assigned duplicate names to regressor ',
     *   'set definitions.')
      stop
  999 return
      end
