      subroutine definite(name,a,n)
 
c     Quick-and-dirty subroutine to determine the rank of a square
c     symmetric matrix, its eigenvalues and and whether it is (semi-)
c     definite.
c
c     Results are printed to units 6 and 8.
c
c     The matrix a is not affected by this routine.
c
c     If print=.true. this function will print the singular values
c     to standard output.
c
c     The maximum dimension that can be handled is (maxn,maxn).
c     If this is exceeded, only an error message is printed.
c
c     Constantijn Panis,  24 October 1991
 
      implicit none
      integer n,maxn,i,nullity
      parameter(maxn=50)
      double precision a(n,n),a1(maxn*maxn),det,lowest,highest,
     *   eps,dpvecmin,dpvecmax,dpvecprod
      parameter(eps=1.d-12)
      character name*(*)
 
      if(n.gt.maxn)goto 800
 
      write(8,'(/,a,a)')'Matrix: ',name
      write(*,'(/,a,a)')'Matrix: ',name
 
      call copy8(a,a1,n*n)
      call eigen(1,a1,0.d0,n)
      nullity=0
      do 100 i=1,n
         if(dabs(a1(i)).lt.eps)then
            a1(i)=0.d0
            nullity=nullity+1
         endif
  100 continue
      det = dpvecprod(a1,n)
 
      write(8,101)(a1(i),i=1,n)
      write(6,101)(a1(i),i=1,n)
  101 format('Eigenvalues:',1p,100e12.4)
      write(8,*)'Rank =',n,' -',nullity,' =',n-nullity
      write(6,*)'Rank =',n,' -',nullity,' =',n-nullity
      write(8,*)'Determinant =',det
      write(6,*)'Determinant =',det
 
      lowest = dpvecmin(a1,n)
      highest = dpvecmax(a1,n)
      if(lowest*highest.lt.0.d0)then
c        One positive, one negative
         write(8,*)'Matrix is not definite.'
         write(*,*)'Matrix is not definite.'
      elseif(lowest*highest.eq.0.d0)then
c        At least one is zero
         if(lowest.lt.0.d0)then
            write(8,*)'Matrix is negative semi-definite.'
            write(*,*)'Matrix is negative semi-definite.'
         elseif(highest.gt.0.d0)then
            write(8,*)'Matrix is positive semi-definite.'
            write(*,*)'Matrix is positive semi-definite.'
         else
            write(8,*)'Matrix is (close to) the zero matrix.'
            write(*,*)'Matrix is (close to) the zero matrix.'
         endif
      else
c        Either both positive or both negative
         if(highest.gt.0.d0)then
            write(8,*)'Matrix is positive definite.'
            write(*,*)'Matrix is positive definite.'
         else
            write(8,*)'Matrix is negative definite.'
            write(*,*)'Matrix is negative definite.'
         endif
      endif
      goto 999
 
  800 write(*,801)
  801 format(/,'ERROR in subroutine definite: maxn is exceeded.')
      goto 999
  999 return
      end
