      subroutine diagcholesky(task,a,b,dim,db,da,status)
 
c     Subroutine to
c     Task = 1: compute the Cholesky decomposition B of a diagonal (!)
c               positive definite matrix A.
c     Task = 2: unravel derivatives wrt B into wrt A.
c
c     A     [in] diagonal positive definite matrix A(dim,dim).
c     B    [out] diagonal of B(dim,dim) contains the Cholesky
c                decomposition such that B'B = A.
c                Note: A may B may share the same memory space.
c     dB    [in] Derivatives wrt B.  (not used if task=1)
c     dA   [out] Derivatives wrt diagonal elements of A.  (not used if
c                task=1)  Note: off-diagonal derivatives of A are
c                not zero, but simply not computed by this routine.
c                If derivatives wrt off-diagonal elements of A are
c                required, use the general Cholesky decomposition
c                routine cholesky.
c     status [out] 0 if all OK; 1 if task=1 and some of the diagonal
c                elements of A are non-positive.  There is no check
c                that A is diagonal.
c
c     Constantijn Panis,  16 November 1991
 
      implicit none
      integer task,dim,status,i
      double precision a(dim,dim),b(dim,dim),db(dim,dim),da(dim,dim),s
 
      status=0
c     Compute Cholesky decomposition or unravel derivatives?
      goto(100,200)task
 
c     Compute Cholesky decomposition:
  100 do 110 i=1,dim
         s = a(i,i)
         if(s.le.0.d0)goto 800
         b(i,i) = dsqrt(s)
  110 continue
      goto 999
 
c     Unravel derivatives:
  200 call zerodp(dim*dim,da)
      do 210 i=1,dim
         da(i,i) = .5d0*db(i,i)/b(i,i)
  210 continue
      goto 999
 
  800 status=1
  999 return
      end
