      subroutine dnormk4i(lnprob,k,x,sigmainv,mu,ifdrv,ifsigma,dsigma,
     *   ifmu,dmu,xminmu,xmmsi,lndet)
 
c     Subroutine to compute the log of a k-dimensional normal density.
c     Arguments:
c     lnprob   [out]  logarithm of normal density
c     k        [in]   dimension of distribution
c     x        [in]   points in k-dimensional space
c     sigmainv [in]   inverse covariance matrix of stochast x.
c     mu       [in]   mean of stochast x
c     ifdrv    [in]   want derivatives?
c     dsigma   [out]  derivatives of lnprob wrt sigma
c     dmu      [out]  derivatives of lnprob wrt mu
c     xminmu   [scratch] will contain x-mu upon return
c     xmmsi    [scratch] will contain (x-mu)'*sigmainv upon return
c
c     This routine differs from dnormk4 in that dnormk4 expects sigma
c     upon input, and dnormk4i expects sigmainv.
c
c     Constantijn Panis, 3 December 2002
c     Last change: log-determinant
 
      implicit none
      integer k,i,j
      double precision twopi
      parameter(twopi=6.283185307179586476925287d0)
      double precision lnprob,sigmainv(k,k),mu(k),dsigma(k,k),
     *   dmu(k),lndet,xminmu(k),xmmsi(k),xmmsixmm
      real x(k)
      logical ifdrv,ifsigma,ifmu
 
c     compute x-mu:
      do 100 i=1,k
         xminmu(i) = dble(x(i))-mu(i)
  100 continue
 
c     compute (x-mu)'*sigmainv:
      call m2mult(100,1,k,k,xminmu,sigmainv,xmmsi,0)
 
c     compute the quadratic form (x-mu)'*sigmainv*(x-mu)
      call m2mult(000,1,k,1,xmmsi,xminmu,xmmsixmm,0)
 
c     compute the log-likelihood:
c        constant - .5*ln(determinant) -.5(x-mu)'*sigmainv*(x-mu),
c     where constant = -.5*k*ln(2*pi)
      lnprob = -.5d0*(k*dlog(twopi) + lndet + xmmsixmm)
 
c     compute derivatives (if desired):
      if(.not.ifdrv)goto 999
 
c     derivatives wrt mu happen to be xmmsi:
      if(ifmu)call copy8(xmmsi,dmu,k)
 
c     derivatives wrt sigma:
c     derivative of ln(det) wrt a parameter is
c     trace(sigmainv*derivative of sigma wrt that parameter);
c     if the parameter is element (i,j) of sigma, this collapses to
c     element (j,i) of sigmainv = element (i,j) of sigmainv.
c     derivative of xmmsixmm wrt a parameter is
c     -xminmu'*sigmainv*(deriv of sigma wrt parameter)*sigmainv*xminmu;
c     if the parameter is element (i,j) of sigma, this collapses to
c     -xmmsi(i)*xmmsi(j)
      if(.not.ifsigma)goto 999
      do 300 i=1,k
         do 310 j=1,i
            dsigma(i,j) = .5d0*(-sigmainv(i,j) + xmmsi(i)*xmmsi(j))
            dsigma(j,i) = dsigma(i,j)
  310    continue
  300 continue
 
  999 return
      end
