      subroutine figdata(sign,levelmax,nbdim,nb,xdim,vdim,maxxnode,
     *   struc,modeltype,mtype,mdllvl,outlvl,nsb,vdimmax,neint,maxmeno,
     *   meno,moel,maxnmapmoel,mapmoel,nmapmoel,numdist,sumdim,
     *   intpnts,distdim,maxdraws,draws,draw,ndraw,dimused,distyp,
     *   lamtyp,lamrow,sparsity,distpos,xpntr,nb2,fullxpntr,nfull,
     *   fulltype,maxref,parref,rsetref,resref,numpar,parpos,numrset,
     *   rsetbegin,rsetnum,numspl,beginspl,nodepos,numnodes,spltype,
     *   splref,maxmspl,nds,nngam,rs3,rs3max,rs4,rs4max,rs5,rs5max,
     *   resdone,splinedone,branch,namepntr,nxold,nold,originstats,
     *   rsetstats,x,v,v6,xnode,splineorigin,endmark,distvar,oldname,
     *   iflam,detail,check99999,splname,resname,nec,ie,ifint,iobs,id,
     *   ipt,jpt)
 
c     Subroutine to figure out exactly which model elements enter
c     an arbitrary model type.
c
c     Make sure to correctly pass in the following arguments:
c        draw(1,ie)
c        mdllvl(nummodel)
c        modeltype(nummodel)
c        nxold(1,ds)
c        namepntr(1,ds)
c        xpntr(2,ib2)        (but fullxpntr is just xpntr)
c        struc(ib2)
c
c     Integers rs keeps track of the number of columns in XB-type
c     scratch arrays that will be needed in prep?.  Total space
c     required is (rsbase,rs) or (rsbase,0:rs), where rsbase contains
c     the number of rows.  The zero-th column is used for the
c     accumulated XB.
c     Rs is _not_ initialized to zero here.  This needs to be done in
c     the calling routine.  The advantage is that rs is accumulated
c     properly even if XB space for regressor sets and residuals is
c     shared, such as in logit/probit and continuous models.
c
c     Modeltype indicates the type of model.  It may be 46 for Tobit models.
c     For Tobit models, it is important to know whether the outcome is
c     censored.  Mtype can help: it is the "basic" modeltype, e.g., 4 for
c     (any type of) probit and 6 for continuous outcomes.
c
c     Outlvl is the output level of regressor sets/splines only (termtype=-3).
c     For other types, the outlvl is assumed equal to mdllvl.
c     Outlvl is mdllvl+1 for hazard models with time-varying covariates.
c     All other models have outcomes at the lowest level, mdllvl.
c
c     Note: nsb is the number of subbranches, relevant for time-varying
c     variables (regressor sets, regressor splines) in hazard models.
c     Other types of models all call this routine with nsb=1.  If there
c     are timemarks beyond the upper event window, nsb may be more than the
c     relevant number of subbranches in the data.  The relevant number is
c     nu.  However, this routine prepares all nsb branches.  This cannot
c     be changed by calling this routine with nu.  Be careful to not use
c     all data in mapmodel.
c
c     Constantijn Panis,  18 December 2002
c     Last change: mtype introduced
 
      implicit none
      integer sign,levelmax,nbdim,nb(nbdim,2),neint,xdim,vdim,oldvdim,
     *   maxxnode,numdist,sumdim,intpnts(sumdim),struc,modeltype,
     *   mtype,mdllvl,outlvl,nsb,vdimmax,meno,maxmeno,moel(maxmeno),
     *   nmapmoel,maxnmapmoel,mapmoel(maxnmapmoel),distdim(numdist),
     *   maxdraws,draws(maxdraws,numdist),draw(numdist),ndraw(numdist),
     *   dimused(sumdim),distyp(numdist),lamtyp(numdist),
     *   lamrow(numdist),distpos(numdist),xpntr(2:levelmax),
     *   nb2,fullxpntr(2:levelmax,nb2),nfull,
     *   fulltype(nfull),maxref,parref(maxref),
     *   rs3,rs3max,rs4,rs4max,rs5,rs5max,resdone,splinedone,sparsity,
     *   rsetref(maxref),resref(maxref),numpar,parpos(numpar),
     *   numrset,rsetbegin(numrset),rsetnum(numrset),
     *   numspl,beginspl(numspl),nodepos(numspl),numnodes(numspl),
     *   maxmspl,nds,nngam
      integer spltype(numspl),splref(maxref),
     *   namepntr(levelmax),nxold(levelmax),
     *   nold,branch(levelmax),termtype,
     *   numterm,nec,ie,ifint(numdist),iobs,id,ipt,jpt,outlevel
      double precision originstats(5,numspl),rsetstats(5,nfull)
      real x(xdim),v(vdimmax),v6(*),xnode(maxxnode),
     *   splineorigin(maxmspl),endmark
      character distvar(numdist)*8,oldname(*)*8,
     *   splname(numspl)*12,resname(sumdim)*8
      logical iflam(numdist),detail,check99999
 
  100 if(nmapmoel+7.gt.maxnmapmoel)call
     *   maxexceed('maxnmapmoel',nmapmoel+7,maxnmapmoel,ipt,jpt)
      if(moel(meno).eq.-99)then
c        End of model
         nmapmoel=nmapmoel+1
         mapmoel(nmapmoel)=-99
         goto 999
      elseif(moel(meno).lt.0)then
c        What follows is a regressor set (-3), integrated residual (-4),
c        duration spline (-5), or a residual (-7);
c        the number of interactions terms is moel(+1).
         termtype=moel(meno)
         numterm=moel(meno+1)
         meno=meno+2
c        Should model elements be added or subtracted?
         if(sign.eq.-1)numterm=-numterm
c        Store old position in V in case this building block drops out.
         oldvdim=vdim
         mapmoel(nmapmoel+1)=termtype
         mapmoel(nmapmoel+2)=numterm
         nmapmoel=nmapmoel+2
c        numterm<0 for model elements that enter negatively
         numterm=abs(numterm)
c        numterm should not count residual or durspline
         if(termtype.eq.-4.or.termtype.eq.-5.or.termtype.eq.-7)
     *      numterm=numterm-1
         if(termtype.eq.-3)then
            if(rs3.eq.-1)call bug(ipt,jpt)
            rs3=rs3+numterm
            rs3max=max(rs3max,rs3)
            outlevel=outlvl
         elseif(termtype.eq.-4.or.termtype.eq.-7)then
            if(rs4.eq.-1)call bug(ipt,jpt)
            rs4=rs4+numterm
            rs4max=max(rs4max,rs4)
            outlevel=mdllvl
         elseif(termtype.eq.-5)then
            if(rs5.eq.-1)call bug(ipt,jpt)
            rs5=rs5+numterm
            rs5max=max(rs5max,rs5)
            outlevel=mdllvl
         else
            call bug(ipt,jpt)
         endif
      endif
      if(moel(meno).eq.2)then
c        Parameter
         call figure_par(levelmax,mdllvl,meno,maxmeno,moel,nmapmoel,
     *      mapmoel,maxxnode,nold,nxold,namepntr,maxref,parref,numpar,
     *      parpos,vdim,oldvdim,xpntr,branch,nbdim,nb,xdim,
     *      x,xnode,oldname,struc,iobs,id,ipt,jpt)
      elseif(moel(meno).eq.3)then
c        Regressor set
         call figure_rset(levelmax,mdllvl,outlevel,nfull,nold,nsb,
     *      nbdim,nb,meno,maxmeno,maxxnode,maxref,vdimmax,numrset,
     *      moel,rsetbegin,rsetnum,rsetref,namepntr,nxold,vdim,
     *      oldvdim,xpntr,nb2,fullxpntr,branch,nmapmoel,
     *      mapmoel,rsetstats,xdim,x,v,xnode,oldname,detail,iobs,id,
     *      struc,modeltype,ipt,jpt)
      elseif(moel(meno).eq.4.or.moel(meno).eq.7)then
c        Integrated or non-integrated residual
         call figure_res(levelmax,nbdim,nb,branch,mdllvl,nmapmoel,
     *      mapmoel,meno,maxmeno,moel,xpntr,namepntr,nxold,maxxnode,
     *      nold,numdist,sumdim,distdim,distyp,lamtyp,lamrow,
     *      distpos,neint,ifint,intpnts,dimused,nfull,fulltype,
     *      modeltype,mtype,maxdraws,draws,draw,ndraw,numterm,resdone,
     *      sparsity,maxref,resref,vdim,oldvdim,struc,iobs,id,ipt,jpt,
     *      xdim,x,nec,ie,v6,xnode,distvar,oldname,resname,iflam)
      elseif(moel(meno).eq.5)then
c        Duration spline
         if(modeltype.ne.1)call bug(ipt,jpt)
         call figure_dspl(levelmax,mdllvl,nbdim,nb,branch,numspl,
     *      maxref,nds,maxmspl,splref,nxold,namepntr,maxxnode,nold,
     *      vdim,oldvdim,numnodes,spltype,nngam,meno,maxmeno,moel,
     *      nmapmoel,mapmoel,originstats,xnode,splineorigin,xpntr,
     *      xdim,x,endmark,detail,check99999,splinedone,splname,
     *      oldname,struc,iobs,id,ipt,jpt)
      elseif(moel(meno).eq.8)then
c        Regressor spline
         call figure_rspl(levelmax,mdllvl,outlevel,nsb,nbdim,nb,branch,
     *      meno,maxmeno,maxref,vdimmax,moel,splref,numspl,beginspl,
     *      nodepos,numnodes,spltype,namepntr,nxold,maxxnode,nold,
     *      modeltype,vdim,oldvdim,xpntr,nb2,fullxpntr,xdim,nmapmoel,
     *      mapmoel,originstats,x,v,xnode,detail,check99999,splname,
     *      oldname,iobs,id,struc,ipt,jpt)
      elseif(moel(meno).eq.9)then
c        Variable
         call figure_var(nold,nbdim,nb,levelmax,branch,meno,maxmeno,
     *      moel,nxold,xpntr,vdim,vdimmax,xdim,nmapmoel,mapmoel,x,v,
     *      ipt,jpt)
      else
         call bug(ipt,jpt)
      endif
c     Check next model element:
      goto 100
 
  999 return
      end
 
