      subroutine findvar(levelmax,maxlevel,level,name,oldname,namepntr,
     *   nxold,varpos,line,il,struc,ipt,jpt,status)
 
c     Subroutine to figure out a variable's variability and position
c     (relative to namepntr) in a certain data structure.
c
c     If the variable is not found, an error message is written.  If
c     this occurs while parsing, line and il are written out to point
c     at the offending line and position.  If it occurs at a later stage,
c     set il to a negative number to suppress that part of the error
c     message.
c
c     levelmax [in] number of levels in the data
c     maxlevel [in] lowest permissible level.  If maxlevel<0, the
c                   variable must be at level abs(maxlevel)
c     level    [out] level of input variable
c     status   [out] 0 = no error
c                    1 = variable not found
c                    2 = variable found but not at the mandatory -maxlevel
c                    3 = variable found, at level=maxlevel+1
c                        (Note: useful for hazard model error diagnosis)
c                    4 = variable found, at level>maxlevel+1
c
c     Constantijn Panis, 11 July 1999
c     Last change: status return code
 
      implicit none
      integer levelmax,namepntr(levelmax),nxold(levelmax),maxlevel,
     *   level,varpos,struc,il,ipt,jpt,status,trm
      character name*(*),oldname(*)*8,line*128,string*60,idstring*60,
     *   tmp*8
 
      tmp=name
      status=0
      do 100 level=1,levelmax
         call char8match(name,oldname(namepntr(level)),nxold(level),
     *      varpos)
         if(varpos.eq.0)goto 100
         if(maxlevel.lt.0)then
            if(level.ne.-maxlevel)goto 800
         else
            if(level.gt.maxlevel)goto 810
         endif
         goto 999
  100 continue
      goto 820
 
  800 if(il.gt.0)il=il-trm(name)
      if(ipt.gt.0)then
         call caretline(8,line,il)
         call writea2ia(8,72,1,'ERROR:  '''//tmp(1:trm(tmp))//
     *      ''' is a level ',level,' variable.  Expected a level ',
     *      -maxlevel,' variable here.')
      endif
      if(jpt.gt.0)then
         call caretline(6,line,il)
         call writea2ia(6,72,1,'ERROR:  '''//tmp(1:trm(tmp))//
     *      ''' is a level ',level,' variable.  Expected a level ',
     *      -maxlevel,' variable here.')
      endif
c     2 = variable found but not at the mandatory -maxlevel
      status=2
      goto 999
  810 if(il.gt.0)il=il-trm(name)
      if(ipt.gt.0)then
         call caretline(8,line,il)
         call writea2ia(8,72,1,'ERROR:  '''//tmp(1:trm(tmp))//
     *      ''' is a level ',level,' variable.  The lowest '//
     *      'permissible level here is level ',maxlevel,'.')
      endif
      if(jpt.gt.0)then
         call caretline(6,line,il)
         call writea2ia(6,72,1,'ERROR:  '''//tmp(1:trm(tmp))//
     *      ''' is a level ',level,' variable.  The lowest '//
     *      'permissible level here is level ',maxlevel,'.')
      endif
c     3 = variable found, at level=maxlevel+1
c     4 = variable found, at level>maxlevel+1
      if(level.eq.maxlevel+1)status=3
      if(level.gt.maxlevel+1)status=4
      goto 999
  820 if(il.gt.0)il=il-trm(name)
      string=idstring(struc,0,0,0,0)
      if(ipt.gt.0)then
         call caretline(8,line,il)
         call wrapstring(8,0,72,0,1,'ERROR:  variable '''//
     *      tmp(1:trm(tmp))//''' not found'//string(1:trm(string)))
      endif
      if(jpt.gt.0)then
         call caretline(6,line,il)
         call wrapstring(6,0,72,0,1,'ERROR:  variable '''//
     *      tmp(1:trm(tmp))//''' not found'//string(1:trm(string)))
      endif
c     1 = variable not found
      status=1
      goto 999
  999 return
      end
