      subroutine getname(unit,line,il,i1,i2)
 
c     Subroutine to determine begin and end of a name.  A name must
c     start with a letter (a-z, A-Z) or underscore (_) and may contain
c     only letters, numbers, and underscores.  This definition
c     corresponds to aML's definition of a valid variable name.
c     The beginning is the first character that is not a blank or a
c     horizontal tab (HT); the end is any character that is not a
c     number, letter, or underscore.
c
c     Upon entering, il is the first position to be checked; upon
c     return, il is the position immediately following the word.
c     i1 and i2 are beginning and end positions of the name.
c
c     If 0<=i2<i1, no name could be read;
c     if i1=0, i2=-1, a next line read was attempted but the program
c     encountered an error or end-of-file.
c
c     Constantijn Panis,  24 December 1997
c     Last change: delimiters changed; no more error return
 
      implicit none
      integer unit,il,i1,i2,cc,aa,zz,uu,zero,nine,invascii
      character line*128,c*1,ht*1,ascii
 
c     ht is the horizontal tab character
      ht=ascii(9)
c     aa is the ascii number for 'A'; zz for 'Z'; uu for '_';
c     zero for '0'; and nine for '9'
      aa=invascii('A')
      zz=invascii('Z')
      uu=invascii('_')
      zero=invascii('0')
      nine=invascii('9')
 
      if(il.gt.80)goto 810
c     skip blanks and tabs:
  110 call uppercase(line(il:il),c)
      if(c.ne.' '.and.c.ne.ht)goto 120
      il=il+1
      if(il.gt.80)then
c        until position 80 all white characters; read next line.
         read(unit,'(a80)',end=810,err=810)line
         il=1
      endif
      goto 110
 
c     Position il is not a blank or tab.  Name starts here:
  120 i1=il
c     Make sure the first character is a letter or underscore
      cc=invascii(c)
      if((cc.lt.aa.or.cc.gt.zz).and.(cc.ne.uu))goto 800
 
c     Scan next position for delimiter:
      il=il+1
  130 if(il.eq.81)then
         i2=il-1
         goto 999
      endif
      call uppercase(line(il:il),c)
      cc=invascii(c)
      if((cc.lt.aa.or.cc.gt.zz).and.(cc.ne.uu).and.
     *   (cc.lt.zero.or.cc.gt.nine))then
         i2=il-1
         goto 999
      endif
      il=il+1
      goto 130
 
c     First character is neither a letter nor an underscore.
  800 i2=i1-1
      goto 999
c     Error or end-of-file
  810 i1=0
      i2=-1
      goto 999
  999 return
      end
 
