      subroutine gridsearch(maxcc,ccmax,dd,rr,ii,ll,cc,ipt,jpt,nchol,
     *   ngrid,nfull,nfreemax,gridpos,gridnum,gridpoint,
     *   gridbest,iiffull,iffull,fulltype,locdim,chol,
     *   dfreebeta,fp,fp2,fpt,fpt2,grid,fullbeta,
     *   fullname,freename,gridfile)
 
c     Subroutine to perform a grid search.
c
c     BUG: if parameter value is at any point illegal, the search may
c           not proceed elegantly.
c
c     Constantijn Panis, 29 August 2002
 
      implicit none
      integer maxcc,ccmax
      double precision dd(maxcc/8)
      real rr(maxcc/4)
      integer ii(maxcc/4)
      logical ll(maxcc/4)
      character cc(maxcc)*1
      integer ipt,jpt,nchol,ngrid,nfull,nfreemax,
     *   gridpos(ngrid),gridnum(ngrid),gridpoint(ngrid),gridbest(ngrid),
     *   iiffull(nfull),iffull(nfull),fulltype(nfull),locdim(120)
      double precision chol(nchol),dfreebeta(nfreemax),fp(nfreemax),
     *   fp2(nfreemax),fpt(nfreemax),fpt2(nfreemax),
     *   grid(ngrid,2),fullbeta(nfull)
      character fullname(nfull)*8,freename(nfreemax)*8,gridfile*128
c     Locally required:
      double precision lnltry,step,lnl,tmp,dpmax,dpnan
      integer gridunit,igrid,time1,numgrid,i,n,nfree,trm,intvecprod,
     *   readonly
      integer time
c     integer time
      character fmt*24,i2c*12
      logical allnan,exist,isnan
 
      allnan=.true.
      nfree=nfreemax
 
c     Determine where to write grid search output.
      if(gridfile.ne.' ')then
         n=trm(gridfile)
         if(exist(gridfile(1:n)))call backup(2,gridfile(1:n))
         if(gridfile(n-3:n).eq.'.dct'.or.gridfile(n-3:n).eq.'.DCT')then
c           Stata dictionary format
            gridunit=10
         else
c           Generic ascii format
            gridunit=9
         endif
         open(gridunit,file=gridfile(1:n),err=830)
         if(readonly(gridunit).eq.1)goto 850
      elseif(ipt.ge.3)then
c        Write grid search information to regular output file
         gridunit=8
      else
c        Do not write grid search information
         gridunit=-1
      endif
 
      if(ipt.gt.0)write(8,101)
      if(jpt.gt.0)write(6,101)
  101 format(/,70('='),/,'=',28x,'GRID SEARCH',29x,'=',/,70('='))
 
      if(gridunit.gt.0)write(8,'(1x)')
      if(gridunit.eq.8)then
         write(8,102)
  102    format('NOTE: use "option gridfile" to direct the output ',
     *      'of the grid search to',/,'a different file.')
      elseif(gridunit.eq.9)then
         call wrapstring(8,0,72,0,1,'NOTE: see '''//
     *      gridfile(1:trm(gridfile))//''' for detailed grid '//
     *      'search output.')
      elseif(gridunit.eq.10)then
         call wrapstring(8,0,72,0,1,'NOTE: see '''//
     *      gridfile(1:trm(gridfile))//''' for detailed grid '//
     *      'search output in Stata''s dictionary format.')
      endif
 
c     Perform grid search:
      lnl=-dpmax()
      call putint(1,gridpoint,ngrid)
      time1 = time()
c     time1 = time()
      igrid=1
  100 do 110 i=1,ngrid
         step=(grid(i,2)-grid(i,1))/(gridnum(i)-1.d0)
         fullbeta(gridpos(i))=grid(i,1) + dble(gridpoint(i)-1)*step
  110 continue
      if(nchol.gt.0)call prepchol(nfull,fulltype,ipt,jpt,nchol,
     *   chol,fullbeta,fullname)
c     Note task=-100, so that function blows up if covariance matrix is not
c     positive definite.
      call amlfunc(-100,maxcc,ccmax,nfull,nfree,fullbeta,fulltype,
     *   iffull,lnltry,nchol,chol,dfreebeta,fp,fp2,fpt,fpt2,0.d0,0.d0,
     *   locdim,dd,rr,ii,ll,cc,0,0,0)
 
c     For first grid point provide estimate of total required time and
c     write out headers.
      if(igrid.eq.1)then
         time1 = time()-time1
c        time1 = time()-time1
         if(time1.ge.3)then
            numgrid=intvecprod(gridnum,ngrid)
            if(ipt.ge.2)call writea2ia(8,67,1,'NOTE: the grid search '//
     *         'involves evaluating the function ',numgrid,' times.  '//
     *         'The estimated required time is roughly ',time1*numgrid,
     *         ' seconds.')
            if(jpt.ge.2)call writea2ia(6,67,1,'NOTE: the grid search '//
     *         'involves evaluating the function ',numgrid,' times.  '//
     *         'The estimated required time is roughly ',time1*numgrid,
     *         ' seconds.')
         endif
         igrid=igrid+1
c        Write out headers.  To screen and non-dct file something like:
c        beta1       beta2       beta3       ln-L
         fmt='('//i2c(ngrid+1,0)
         fmt=fmt(1:trm(fmt))//'a12,/,2x,'//i2c(12*ngrid+10,0)
         fmt=fmt(1:trm(fmt))//'a)'
         if(jpt.ge.3)then
            write(6,'(1x)')
            write(6,fmt)(fullname(gridpos(i))
     *         (1:trm(fullname(gridpos(i)))),i=1,ngrid),'ln-L',
     *         ('-',i=1,12*ngrid+10)
         endif
         if(gridunit.eq.8)then
            write(gridunit,'(1x)')
            write(gridunit,fmt)(fullname(gridpos(i))
     *         (1:trm(fullname(gridpos(i)))),i=1,ngrid),'ln-L',
     *         ('-',i=1,12*ngrid+10)
         elseif(gridunit.eq.9)then
            write(gridunit,fmt)(fullname(gridpos(i))
     *         (1:trm(fullname(gridpos(i)))),i=1,ngrid),'ln-L',
     *         ('-',i=1,12*ngrid+10)
         elseif(gridunit.eq.10)then
            fmt='(a,'//i2c(ngrid+1,0)
            fmt=fmt(1:trm(fmt))//'(/,3x,a),/,a)'
            write(gridunit,fmt)'dictionary {',(fullname(gridpos(i)),
     *         i=1,ngrid),'lnl       "Log-Likelihood"','}'
         endif
      endif
 
c     Write out grid points and log-likelihood:
      do 120 i=1,ngrid
c        If amlfunc changed parameter values, something was apparently wrong.
c        For example, if a covariance matrix becomes non-positive definite,
c        subroutine ensposdef reduces correlations until the matrix is OK.
c        This is not acceptable in grid searches.  In such cases, set lnl=NaN.
         step=(grid(i,2)-grid(i,1))/(gridnum(i)-1.d0)
         tmp=grid(i,1) + dble(gridpoint(i)-1)*step
         if(fullbeta(gridpos(i)).ne.tmp)then
            lnltry=dpnan()
            fullbeta(gridpos(i))=tmp
         endif
         if(jpt.ge.3)call writedble(6,fullbeta(gridpos(i)),2,10,0,0)
         if(gridunit.gt.0)call writedble(gridunit,fullbeta(gridpos(i)),
     *      2,10,0,0)
  120 continue
      if(isnan(lnltry))then
         if(jpt.ge.3)write(6,'(9x,a3)')'NaN'
      elseif(lnltry.gt.lnl)then
c        Current grid point is improvement:
         call copyint(gridpoint,gridbest,ngrid)
         lnl=lnltry
         if(jpt.ge.3)call writedble(6,lnltry,2,10,0,0)
         if(jpt.ge.3)write(6,'(1x,a)')'*'
         allnan=.false.
      else
         if(jpt.ge.3)call writedble(6,lnltry,2,10,0,1)
         allnan=.false.
      endif
      if(gridunit.gt.0)call writedble(gridunit,lnltry,2,10,0,1)
 
c     Go to next grid point:
      do 130 i=1,ngrid
         if(gridpoint(i).eq.gridnum(i))then
            gridpoint(i)=1
         else
            gridpoint(i)=gridpoint(i)+1
            goto 100
         endif
  130 continue
 
c     All grid points evaluated; write out best parameter values and
c     update fullbeta:
      if(allnan)goto 800
      if(jpt.ge.2)then
         write(6,'(1x)')
         call writeada(6,0,1,'Optimal grid point (ln-L = ',12,lnl,'):')
         write(6,'(1x)')
      endif
      if(ipt.gt.2)then
         write(8,'(1x)')
         call writeada(8,0,1,'Optimal grid point (ln-L = ',12,lnl,'):')
         write(8,'(1x)')
      endif
      do 140 i=1,ngrid
         step=(grid(i,2)-grid(i,1))/(gridnum(i)-1.d0)
         fullbeta(gridpos(i))=grid(i,1) + dble(gridbest(i)-1)*step
         if(jpt.ge.2)call writeada(6,0,1,fullname(gridpos(i))//'  ',
     *      10,fullbeta(gridpos(i)),' ')
         if(ipt.ge.2)call writeada(8,0,1,fullname(gridpos(i))//'  ',
     *      10,fullbeta(gridpos(i)),' ')
  140 continue
 
      if(gridunit.eq.9.or.gridunit.eq.10)close(gridunit)
      goto 999
 
  800 if(ipt.gt.0)write(8,801)
      if(jpt.gt.0)write(6,801)
  801 format(/,'ERROR: All function evaluations in the grid search ',
     *   'failed.',/,'Please specify better starting values.')
      stop
  830 if(ipt.gt.0)write(8,'(1x)')
      if(jpt.gt.0)write(6,'(1x)')
      if(ipt.gt.0)call wrapstring(8,0,72,0,1,'ERROR opening "'//
     *   gridfile(1:n)//'"; you probably do not have write '//
     *   'permission or another program is currently writing '//
     *   'to this file.  Specify another name in the "option '//
     *   'gridfile".')
      if(jpt.gt.0)call wrapstring(6,0,72,0,1,'ERROR opening "'//
     *   gridfile(1:n)//'"; you probably do not have write '//
     *   'permission or another program is currently writing '//
     *   'to this file.  Specify another name in the "option '//
     *   'gridfile".')
      stop
  850 if(ipt.gt.0)write(8,'(1x)')
      if(jpt.gt.0)write(6,'(1x)')
      if(ipt.gt.0)call wrapstring(8,0,72,0,1,'ERROR:  '''//
     *   gridfile(1:n)//''' is read-only.  Please delete or rename it.')
      if(jpt.gt.0)call wrapstring(6,0,72,0,1,'ERROR:  '''//
     *   gridfile(1:n)//''' is read-only.  Please delete or rename it.')
      stop
  999 return
      end
