      integer function intlen(i)
 
c     Integer function to determine the number of spaces that writing
c     an integer would require.  For example, -647 requires 4 spaces to
c     write.   This function is particularly useful in format statements.
c
c     Example:
c     write(*,1)i
c   1 format(.......,i<intlen(i)>,.....)
c
c     This format statement requires an extension to Fortran 77.
c
c     Note: rounding problem occurs in red Hat Linux GNU g77.
c     For example, dlog10(dble(10)) - 1 = -2.nnnE-17.
c
c     Constantijn Panis,  21 April 2001
c     Last change: account for g77
 
      implicit none
      integer i
 
      if(i.lt.0)then
         intlen = int(dlog10(dble(-i)+.5d0)) + 2
      elseif(i.eq.0)then
         intlen = 1
      else
         intlen = int(dlog10(dble(i)+.5d0)) + 1
      endif
 
c     If i was the equivalent of NaN in integers (which is possible
c     for example if i=int(x), where x is too large), then intlen
c     may be negative and cause write errors.
      intlen = max(1,intlen)
 
      return
      end
