      double precision function invnrm(x)
 
c     Function for the inverse of the standard normal probability.
c     method: upper and lower bound of outcome is determined;
c     solution found through halvation of intervals until
c     convergence is achieved.   Precision depends on parameter
c     eps and on precision of pnorm1.
c     NOTE: the calling routine must declare invnrm to be double
c           precision:    "double precision invnrm"
c
c     Note that efficiency can be gained by providing better initial
c     guesses for certain ranges of x.
c
c     Note the existence of IMSL function dnorin with the same function.
c
c     Constantijn Panis,   2 September 1991
 
      implicit none
      double precision x,eps,dpnan,au,al,a,pnorm1
      parameter(eps=1.d-8)
 
      if(x.le.0.d0.or.x.ge.1.d0)then
c        Invalid argument:
         invnrm = dpnan()
         goto 999
      endif
 
      if(x.lt..5d0)then
c        outcome is negative; upper level .39(x-.5)
         au=.39d0*(x-.5d0)
  110    if(x-pnorm1(2.d0*au).gt.0.d0)goto 120
         au=2.d0*au
         goto 110
  120    al=2.d0*au
      elseif(x.eq..5d0)then
         invnrm=0.d0
         goto 999
      else
c        outcome is positive; lower level .39(x-.5)
         al=.39d0*(x-.5d0)
  210    if(x-pnorm1(2.d0*al).lt.0.d0)goto 220
         al=2.d0*al
         goto 210
  220    au=2.d0*al
      endif
 
  100 a=.5d0*(al+au)
      if(a-al.lt.eps.or.pnorm1(a).eq.x)then
         invnrm=a
         goto 999
      elseif(pnorm1(a).lt.x)then
c        a is too small
         al=a
      else
c        a is too large
         au=a
      endif
      goto 100
 
  999 return
      end
