      subroutine m3mult(task,n1,n2,n3,n4,a,b,c,d,factor)
 
c     Subroutine to compute A*B*C = D:
c
c            A(n1,n2)   B(n2,n3)   C(n3,n4)   D(n1,n4)
c
c     The four matrices may be in transposed format.  Integer task
c     may be 1000,0100,0010, et cetera, depending on which matrices
c     are transposed.  The first digit refers to A; the second to B;
c     the third to C; the fourth to D.  For example,
c        task = 1100 means that A and B are transposed: A'*B'*C = D
c        task =   10 means that C is transposed: A*B*C' = D
c
c     Dimensions as given above assume no transpositions; if a matrix
c     is transposed, its dimensions really are reversed.
c
c     Integer factor determines:
c        factor = 0     D = A*B*C
c              else     D = D + factor*A*B*C
c
c     Constantijn Panis, 21 March 1993
 
      implicit none
      integer task,n1,n2,n3,n4,factor,i,j,k,l,ia,ib,ic,id,ja,jb,jc,jd,
     *   sa,sb,sc,sd,jsa,jsb,jsc,jsd
      double precision a(n1*n2),b(n2*n3),c(n3*n4),d(n1*n4),abij
 
c     In its most basic form, D = D + A*B*C, the routine is very compact:
c
c     do 110 i=1,n1
c        do 120 j=1,n3
c           abij = 0.d0
c           do 130 k=1,n2
c              abij = abij + a(i,k)*b(k,j)
c 130       continue
c           do 140 l=1,n4
c              d(i,l) = d(i,l) + abij*c(j,l)
c 140       continue
c 120    continue
c 110 continue
c
c     The 130 loop computes element (i,j) of A*B, hence the name abij.
c     This element is computed with every element of row j of C and
c     added to D(i,.).
c
c     Complications arise because this routine allows for A, B, C and D to
c     be stored in transposed form.  Note that A and B are only used in the
c     130 loop, and C and D are only used in the 140 loop.  The positions
c     that are used are ia, ib, ic and id.  In the 130 and 140 loops, they
c     increase by ja, jb, jc and jd, where j stands for "jump".  If A, B,
c     C and D are not-transposed, these jumps are nrowa, 1, nrowc and nrowd,
c     respectively, where nrow stands for the number of rows.  If A is
c     transposed, this changes to 1; if B is transposed, it becomes nrowb,
c     et cetera.  The first time an element is accessed in the 130 and 140
c     loops, we need A(i,1), B(1,j), C(j,1) and D(i,1); these are coded
c     as sa, sb, sc and sd, where s stands for "start".  These start off at
c     1 in all four cases, the first time the 130/140 loops are used.
c     The next time they increase by 1, nrowb, 1 and 1; these are coded
c     as jsa, jsb, jsc and jsd, where js stands for "jump in start".
c     If a matrix is transposed, the jumps are different.
 
      if(mod(task/1000,10).eq.0)then
         ja=n1
         jsa=1
      else
         ja=1
         jsa=n2
      endif
      if(mod(task/100,10).eq.0)then
         jb=1
         jsb=n2
      else
         jb=n3
         jsb=1
      endif
      if(mod(task/10,10).eq.0)then
         jc=n3
         jsc=1
      else
         jc=1
         jsc=n4
      endif
      if(mod(task,10).eq.0)then
         jd=n1
         jsd=1
      else
         jd=1
         jsd=n4
      endif
 
      if(factor.eq.0)call zerodp(n1*n4,d)
      sa=1
      sd=1
      do 210 i=1,n1
         sb=1
         sc=1
         do 220 j=1,n3
            abij = 0.d0
            ia=sa
            ib=sb
            ic=sc
            id=sd
            do 230 k=1,n2
               abij = abij + a(ia)*b(ib)
               ia=ia+ja
               ib=ib+jb
  230       continue
            if(factor.ne.0.and.factor.ne.1)abij=dble(factor)*abij
            do 240 l=1,n4
               d(id) = d(id) + abij*c(ic)
               id=id+jd
               ic=ic+jc
  240       continue
            sb=sb+jsb
            sc=sc+jsc
  220    continue
         sa=sa+jsa
         sd=sd+jsd
  210 continue
 
      return
      end
