      subroutine map_u(task,nfullplus,numdist,sumintdim,sumdim2,
     *   sumbigintdim,intdist,intdim,intdimc,nec,allint6,posi1,posi2,
     *   bigposi1,maxpnts,intpnts,intpntr,dimused,indx,distyp,distpos,
     *   fullbeta,ddfullbeta,z,wt,weight,dweight,cdm,ddcdm,ui,ddui,cmui)
 
c     Subroutine to
c     Task = 1: compute multivariate point u from Cholesky decomposed
c        covariance matrix D and a combination of standard normal
c        approximation points z (u = D'z);
c     Task = 2: unravel derivatives wrt u into wrt D.
c
c     Also compute the weight of this point.
c
c     Constantijn Panis, 22 June 1997
 
      implicit none
      integer task,nfullplus,numdist,sumintdim,sumdim2,sumbigintdim,
     *   intdist,intdim,intdimc,nec,allint6,posi1,posi2,bigposi1,
     *   maxpnts,intpnts(sumintdim),intpntr(sumintdim),
     *   dimused(sumintdim),indx(sumintdim),distyp(numdist),
     *   distpos(numdist),i,j,ii,jj,pospnt,poswt
      double precision fullbeta(nfullplus),ddfullbeta(nfullplus),
     *   z(maxpnts),wt(maxpnts),weight,dweight,cdm(sumdim2),
     *   ddcdm(sumdim2),ui(sumintdim),ddui(sumintdim),
     *   cmui(sumbigintdim),pnorm1,dnorm1
 
c     Compute u or unravel derivatives?
      if(task.eq.2)goto 200
 
      weight = 1.d0
      call zerodp(intdim,ui(posi1))
c     If indx=0, the 0-th draw of some irrelevant distribution applies.
      if(indx(posi1).eq.0)goto 999
 
c     Compute u and its appropriate weight.  Distinguish between
c     finite mixture and normal distributions.
      if(mod(distyp(intdist),10).eq.4)then
         pospnt=distpos(intdist)/10000-1+indx(posi1)
         poswt=mod(distpos(intdist),10000)-1+indx(posi1)
         ui(posi1)=fullbeta(pospnt)
         if(indx(posi1).eq.1)then
            weight=pnorm1(fullbeta(poswt))
         elseif(indx(posi1).lt.intpnts(posi1))then
            weight=pnorm1(fullbeta(poswt))-pnorm1(fullbeta(poswt-1))
         else
            weight=1.d0-pnorm1(fullbeta(poswt-1))
         endif
      else
c        Compute u = D'z, where D is the Cholesky decomposition of the
c        covariance matrix.
         ii=-intdimc
         do 140 i=posi1,posi1+intdim-1
            ui(i)=0.d0
            if(dimused(i).eq.0)goto 140
            ii=ii+intdimc
            weight = weight*wt(intpntr(i)+indx(i))
            jj=posi2-1
c           Note that D is upper triangular.
            do 150 j=posi1,i
               if(dimused(j).eq.0)goto 150
               jj=jj+1
c              Access _compressed_ element (j,i) of cdm
               ui(i)=ui(i)+cdm(jj+ii)*z(intpntr(j)+indx(j))
  150       continue
  140    continue
c        If the integrated distribution is not integrated out of
c        continuous modules, add the resulting conditional mean:
         if(nec.gt.0.and.allint6.eq.-1)call vadd(ui(posi1),
     *      cmui(bigposi1),ui(posi1),intdim)
      endif
      goto 999
 
c     Unravel derivatives.
c     If indx=0, the 0-th draw of some irrelevant distribution applies.
  200 if(indx(posi1).eq.0)goto 999
      if(mod(distyp(intdist),10).eq.4)then
         pospnt=distpos(intdist)/10000-1+indx(posi1)
         poswt=mod(distpos(intdist),10000)-1+indx(posi1)
         ddfullbeta(pospnt)=ddfullbeta(pospnt)+ddui(posi1)
         if(indx(posi1).eq.1)then
            ddfullbeta(poswt)=ddfullbeta(poswt)+
     *         dweight*dnorm1(fullbeta(poswt))
         elseif(indx(posi1).lt.intpnts(posi1))then
            ddfullbeta(poswt-1)=ddfullbeta(poswt-1)-
     *         dweight*dnorm1(fullbeta(poswt-1))
            ddfullbeta(poswt)=ddfullbeta(poswt)+
     *         dweight*dnorm1(fullbeta(poswt))
         else
            ddfullbeta(poswt-1)=ddfullbeta(poswt-1)-
     *         dweight*dnorm1(fullbeta(poswt-1))
         endif
      else
         ii=-intdimc
         do 220 i=posi1,posi1+intdim-1
            if(dimused(i).eq.0)goto 220
            ii=ii+intdimc
            jj=posi2-1
            do 230 j=posi1,i
               if(dimused(j).eq.0)goto 230
               jj=jj+1
               ddcdm(jj+ii)=ddcdm(jj+ii)+ddui(i)*z(intpntr(j)+indx(j))
  230       continue
  220    continue
      endif
 
  999 return
      end
