      subroutine mapmodel1(task,neh,nsb,nfullplus,numdist,dimlam,
     *   distdim,lampos1,dimv1,mapmoel,nds,nngam,
     *   rsh3max,rsh4max,rsh5max,xbh3,xbh4,xbh5,d1xbh3,d2xbh3,
     *   fullbeta,dlnla,dlnlb,
     *   gamma,d1gamma,d2gamma,lam,dlam,dfullbeta,v1,censor,
     *   moelpos,vpntr,refspl,beginspl,numnodes,spltype,ipt,jpt)
 
c     Subroutine to process model elements for hazard models,
c     one spell only.  This routine is akin to prepfunc for other
c     types of models.
c
c     If task=1, regressor sets and parameters are placed in xb and
c     interacted if appropriate.  If the resulting regressor set enters
c     as part of Beta'X, it is accumulated into xb(.,0).  If it is
c     interacted with a slope, it is placed in gamma; if it is to be
c     interacted with a residual component, it is placed in Lambda.
c
c     If task=2, derivatives wrt XB and gamma are unraveled.  Note that
c     there are two sets of derivatives, namely wrt like1 and like2; in
c     censored cases, the (plain) likelihood is like1; in non-censored
c     cases it is like1-like2.
c
c     If task=3, derivatives wrt Lambda are unraveled.  This is done
c     after heterogeneity is integrated out, i.e., there is only one
c     set of derivatives wrt Lambda.
c
c     In general, codes 2 correspond to parameters; 3 to regressor sets;
c     4 to integrated residuals; 5 to duration splines and 8 to
c     regressor splines.
c
c     Constantijn Panis, 15 May 2004
c     Last change: interaction of variable and integrated residual
c     triggered bug.  See e-mail Siddhart Singh of 5/14/2004.
 
      implicit none
      integer task,neh,nsb,nfullplus,numdist,dimlam,distdim(numdist),
     *   lampos1(numdist),dimv1,mapmoel(*),nds,
     *   refspl(*),beginspl(*),numnodes(*),spltype(*),rsh3max,rsh4max,
     *   rsh5max,ipt,jpt,nngam,censor,vpntr,termtype,numterm,rs3,rs4,
     *   rs5,firstrs3,firstrs4,firstrs5,moelpos,pos,gampos,
     *   nx1,dim,ngam,ref,dist,draw,col
      double precision xbh3(nsb,0:rsh3max),xbh4(rsh4max),xbh5(rsh5max),
     *   d1xbh3(nsb),d2xbh3(nsb),dxbh4,d1xbh5,d2xbh5,
     *   fullbeta(nfullplus),dfullbeta(nfullplus),dlnla(nfullplus),
     *   dlnlb(nfullplus),gamma(*),d1gamma(*),d2gamma(*),lam(dimlam),
     *   dlam(dimlam),temp,sign,dpvecprod,innerp
      real v1(dimv1)
 
      if(task.eq.1)goto 100
      if(task.eq.2)goto 200
      if(task.eq.3)goto 300
 
c     Compute the overall regressor set, slopes and residual load matrix.
c     First initialize grand XB to zero:
  100 call zerodp(nsb,xbh3)
      nds=0
      nngam=0
      gampos=1
      termtype=0
      rs3=0
      rs4=0
      rs5=0
  120 if(mapmoel(moelpos).lt.0)then
c        End of this term, beginning of new term.
c        If previous terms were regressor set(s) only, add to grand XB:
         if(termtype.eq.-3)call
     *      addrset(numterm,nsb,nsb,xbh3,xbh3(1,firstrs3),sign)
c        Next term:
         if(mapmoel(moelpos).eq.-99)then
c           End of model for this episode
            moelpos=moelpos+1
            goto 999
         endif
c        Next is a regressor set: termtype=-3
c            integrated residual: termtype=-4
c                duration spline: termtype=-5
         termtype=mapmoel(moelpos)
         numterm=mapmoel(moelpos+1)
         sign=1.d0
         if(numterm.lt.0)then
            sign=-1.d0
            numterm=-numterm
         endif
         if(termtype.eq.-4.or.termtype.eq.-5)numterm=numterm-1
         moelpos=moelpos+2
         firstrs3=rs3+1
         firstrs4=rs4+1
         firstrs5=rs5+1
         goto 120
      elseif(mapmoel(moelpos).eq.2)then
c        Parameter: place in xb as if it were a regressor set.
c        Note that pos>nfull for inverse matrix elements.
         pos=mapmoel(moelpos+1)
         moelpos=moelpos+2
         if(termtype.eq.-3)then
            rs3=rs3+1
            call compute_xb1(5,numterm,0,0,nsb,0.d0,fullbeta(pos),
     *         xbh3(1,rs3),0.d0,0.d0,0.d0,0.d0,0.d0,0.d0,0.d0,0,0.)
         elseif(termtype.eq.-4)then
            rs4=rs4+1
            call compute_xb1(7,numterm,0,0,1,0.d0,fullbeta(pos),
     *         xbh4(rs4),0.d0,0.d0,0.d0,0.d0,0.d0,0.d0,0.d0,0,0.)
         elseif(termtype.eq.-5)then
            rs5=rs5+1
            call compute_xb1(7,numterm,0,0,1,0.d0,fullbeta(pos),
     *         xbh5(rs5),0.d0,0.d0,0.d0,0.d0,0.d0,0.d0,0.d0,0,0.)
         endif
      elseif(mapmoel(moelpos).eq.3)then
c        Regressor set: place in xb
         pos=mapmoel(moelpos+1)
         nx1=mapmoel(moelpos+2)
         moelpos=moelpos+3
         if(termtype.eq.-3)then
            rs3=rs3+1
            call compute_xb1(1,numterm,0,nx1,nsb,fullbeta(pos),
     *         0.d0,xbh3(1,rs3),0.d0,0.d0,0.d0,0.d0,
     *         0.d0,0.d0,0.d0,0,v1(vpntr))
c           Even though only nu branches are used, the data contain nsb
c           (see figure_rset, figure_rspl, as call by figdata1-figdata).
            vpntr=vpntr+nx1*nsb
         elseif(termtype.eq.-4)then
            rs4=rs4+1
            call compute_xb1(3,numterm,0,nx1,1,fullbeta(pos),
     *         0.d0,xbh4(rs4),0.d0,0.d0,0.d0,0.d0,
     *         0.d0,0.d0,0.d0,0,v1(vpntr))
            vpntr=vpntr+nx1
         elseif(termtype.eq.-5)then
            rs5=rs5+1
            call compute_xb1(3,numterm,0,nx1,1,fullbeta(pos),
     *         0.d0,xbh5(rs5),0.d0,0.d0,0.d0,0.d0,
     *         0.d0,0.d0,0.d0,0,v1(vpntr))
            vpntr=vpntr+nx1
         endif
      elseif(mapmoel(moelpos).eq.4)then
c        Integrated residual; fill out Lambda (which is a function of
c        ones and regressor sets).  Note that Lambda is always generic
c        in hazard models, as no effort is made to recognize special
c        structures for load matrices on integrated distributions.
         dist=mapmoel(moelpos+1)
         dim=mapmoel(moelpos+2)
         draw=mapmoel(moelpos+3)
         moelpos=moelpos+4
         col=distdim(dist)*(draw-1)+dim
         pos=lampos1(dist)+neh*(col-1)+1
         if(numterm.eq.0)then
c           Plain residual, not interacted:
            lam(pos)=lam(pos)+sign
         elseif(numterm.eq.1)then
c           Residual is multiplied by one term:
            lam(pos)=lam(pos)+sign*xbh4(firstrs4)
         else
c           Residual is multiplied by multiple terms:
            temp=dpvecprod(xbh4(firstrs4),numterm)
            lam(pos)=lam(pos)+sign*temp
         endif
      elseif(mapmoel(moelpos).eq.5)then
c        Spline: multiply regressor sets (if any) times intercept/slopes
c        and place intercept/slopes in gamma.
         nds=nds+1
         ref=mapmoel(moelpos+1)
         refspl(nds)=ref
         pos=beginspl(ref)
         ngam=numnodes(ref)+1
         if(spltype(ref).eq.2.or.spltype(ref).eq.4)ngam=ngam+1
         moelpos=moelpos+2
         nngam=nngam+ngam
         if(numterm.eq.0)then
c           Plain spline, no interactions:
            call copy8(fullbeta(pos),gamma(gampos),ngam)
            if(sign.eq.-1.d0)call opposite(gamma(gampos),gamma(gampos),
     *         ngam)
         elseif(numterm.eq.1)then
c           Spline is multiplied by parameter or regressor set:
            call mult(fullbeta(pos),sign*xbh5(firstrs5),gamma(gampos),
     *         ngam)
         elseif(numterm.ge.2)then
c           First interact multiple regressor sets:
            temp=dpvecprod(xbh5(firstrs5),numterm)
            call mult(fullbeta(pos),sign*temp,gamma(gampos),ngam)
         endif
         gampos=gampos+ngam
      elseif(mapmoel(moelpos).eq.9)then
c        Variable: place in xb
         moelpos=moelpos+1
         if(termtype.eq.-3)then
            rs3=rs3+1
            call put(dble(v1(vpntr)),xbh3(1,rs3),nsb)
         elseif(termtype.eq.-4)then
            rs4=rs4+1
            xbh4(rs4)=dble(v1(vpntr))
         elseif(termtype.eq.-5)then
            rs5=rs5+1
            xbh5(rs5)=dble(v1(vpntr))
         endif
         vpntr=vpntr+1
      else
         call bug(ipt,jpt)
      endif
      goto 120
 
c     Map derivatives wrt XB and gamma back into wrt parameters; don't do
c     anything with derivatives wrt Lambda yet, since they don't exist yet.
  200 termtype=0
      gampos=1
      nds=0
      rs3=0
      rs5=0
  220 if(mapmoel(moelpos).lt.0)then
c        End of this term, beginning of new term:
         if(mapmoel(moelpos).eq.-99)then
c           end of model for this episode
            moelpos=moelpos+1
            goto 999
         endif
c        Next is a regressor set: termtype=-3
c            integrated residual: termtype=-4
c                duration spline: termtype=-5
         termtype=mapmoel(moelpos)
         numterm=mapmoel(moelpos+1)
         sign=1.d0
         if(numterm.lt.0)then
            sign=-1.d0
            numterm=-numterm
         endif
         if(termtype.eq.-4.or.termtype.eq.-5)numterm=numterm-1
         moelpos=moelpos+2
         firstrs3=rs3+1
         firstrs5=rs5+1
         if(termtype.eq.-3)then
c           Regressor set(s): use d1xbh3/d2xbh3
            if(sign.eq.-1)then
               call opposite(d1xbh3,d1xbh3,nsb)
               call opposite(d2xbh3,d2xbh3,nsb)
            endif
         elseif(termtype.eq.-4)then
c           Residual: no derivatives available yet.
            continue
         elseif(termtype.eq.-5)then
c           Spline:
            if(sign.eq.-1)then
               call opposite(d1gamma(gampos),d1gamma(gampos),ngam)
               call opposite(d2gamma(gampos),d2gamma(gampos),ngam)
            endif
c           Copy dgamma into dxbh5, if there are interactions:
            nds=nds+1
            if(numterm.ge.1)then
c              Figure out where the intercepts and slopes are in fullbeta,
c              and how many gammas there are:
               ref=refspl(nds)
               pos=beginspl(ref)
               ngam=numnodes(ref)+1
               if(spltype(ref).eq.2.or.spltype(ref).eq.4)ngam=ngam+1
               d1xbh5=innerp(d1gamma(gampos),fullbeta(pos),ngam)
               if(censor.eq.0)d2xbh5=innerp(d2gamma(gampos),
     *            fullbeta(pos),ngam)
            endif
         endif
         goto 220
      elseif(mapmoel(moelpos).eq.2)then
c        Parameter: unravel dxb
         pos=mapmoel(moelpos+1)
         moelpos=moelpos+2
         if(termtype.eq.-3)then
            rs3=rs3+1
            call compute_xb1(6,numterm,rs3-firstrs3+1,0,nsb,0.d0,
     *         0.d0,0.d0,xbh3(1,firstrs3),0.d0,dlnla(pos),
     *         d1xbh3,0.d0,dlnlb(pos),d2xbh3,censor,0.)
         elseif(termtype.eq.-4)then
            continue
         elseif(termtype.eq.-5)then
            rs5=rs5+1
            call compute_xb1(8,numterm,rs5-firstrs5+1,0,1,0.d0,
     *         0.d0,0.d0,xbh5(firstrs5),0.d0,dlnla(pos),
     *         d1xbh5,0.d0,dlnlb(pos),d2xbh5,censor,0.)
         endif
      elseif(mapmoel(moelpos).eq.3)then
c        Regressor set
         pos=mapmoel(moelpos+1)
         nx1=mapmoel(moelpos+2)
         moelpos=moelpos+3
         if(termtype.eq.-3)then
            rs3=rs3+1
            call compute_xb1(2,numterm,rs3-firstrs3+1,nx1,nsb,
     *         fullbeta(pos),0.d0,0.d0,xbh3(1,firstrs3),
     *         dlnla(pos),0.d0,d1xbh3,dlnlb(pos),0.d0,
     *         d2xbh3,censor,v1(vpntr))
c           Even though only nu branches are used, the data contain nsb
c           (see figure_rset, figure_rspl, as call by figdata1-figdata).
            vpntr=vpntr+nx1*nsb
         elseif(termtype.eq.-4)then
            vpntr=vpntr+nx1
         elseif(termtype.eq.-5)then
            rs5=rs5+1
            call compute_xb1(4,numterm,rs5-firstrs5+1,nx1,1,
     *         fullbeta(pos),0.d0,0.d0,xbh5(firstrs5),
     *         dlnla(pos),0.d0,d1xbh5,dlnlb(pos),0.d0,
     *         d2xbh5,censor,v1(vpntr))
            vpntr=vpntr+nx1
         endif
      elseif(mapmoel(moelpos).eq.4)then
c        Integrated residual; yet to be taken care of.
         moelpos=moelpos+4
      elseif(mapmoel(moelpos).eq.5)then
c        Duration spline: unravel wrt gamma into wrt fullbeta:
         ref=mapmoel(moelpos+1)
         pos=beginspl(ref)
         ngam=numnodes(ref)+1
         if(spltype(ref).eq.2.or.spltype(ref).eq.4)ngam=ngam+1
         moelpos=moelpos+2
         if(numterm.eq.0)then
c           Plain spline, no interactions:
            call vadd(d1gamma(gampos),dlnla(pos),dlnla(pos),ngam)
            if(censor.eq.0)call vadd(d2gamma(gampos),dlnlb(pos),
     *         dlnlb(pos),ngam)
         else
c           Spline is multiplied by one or more regressor sets:
            temp=dpvecprod(xbh5(firstrs5),numterm)
            call add(d1gamma(gampos),dlnla(pos),temp,ngam)
            if(censor.eq.0)call add(d2gamma(gampos),dlnlb(pos),
     *         temp,ngam)
         endif
         gampos=gampos+ngam
      elseif(mapmoel(moelpos).eq.9)then
c        Variable
         moelpos=moelpos+1
         if(termtype.eq.-3)then
            rs3=rs3+1
         elseif(termtype.eq.-5)then
            rs5=rs5+1
         endif
         vpntr=vpntr+1
      else
         call bug(ipt,jpt)
      endif
      goto 220
 
c     Map derivatives wrt Lambda back into wrt parameters; don't do
c     anything with derivatives wrt XB and gamma.
  300 termtype=0
      nds=0
      rs4=0
  320 if(mapmoel(moelpos).lt.0)then
c        End of this term, beginning of new term:
         if(mapmoel(moelpos).eq.-99)then
c           end of model for this episode
            moelpos=moelpos+1
            goto 999
         endif
c        Next is a regressor set: termtype=-3
c            integrated residual: termtype=-4
c                duration spline: termtype=-5
         termtype=mapmoel(moelpos)
         numterm=mapmoel(moelpos+1)
         sign=1.d0
         if(numterm.lt.0)then
            sign=-1.d0
            numterm=-numterm
         endif
         if(termtype.eq.-4.or.termtype.eq.-5)numterm=numterm-1
         moelpos=moelpos+2
         firstrs4=rs4+1
         if(termtype.eq.-4)then
c           Residual: place derivatives wrt Lambda in wrt Xbeta, if any:
            if(numterm.eq.0)then
c              This residual entered without load factor:
               moelpos=moelpos+4
            else
c              One or more load factors; figure out which dimension applies:
               pos=moelpos
  330          if(mapmoel(pos).eq.2)then
                  pos=pos+2
                  goto 330
               elseif(mapmoel(pos).eq.3)then
                  pos=pos+3
                  goto 330
               elseif(mapmoel(pos).eq.4)then
                  dist=mapmoel(pos+1)
                  dim=mapmoel(pos+2)
                  draw=mapmoel(pos+3)
                  col=distdim(dist)*(draw-1)+dim
                  pos=lampos1(dist)+neh*(col-1)+1
                  dxbh4=sign*dlam(pos)
               elseif(mapmoel(pos).eq.9)then
                  pos=pos+1
                  goto 330
               else
                  call bug(ipt,jpt)
               endif
            endif
         endif
c        Don't search for next nonpositive mapmoel in order to skip
c        regressor sets and duration splines.  These namely involve
c        elements in v1, so vpntr needs to be updated.
         goto 320
      elseif(mapmoel(moelpos).eq.2)then
c        Parameter: unravel dxb
         pos=mapmoel(moelpos+1)
         moelpos=moelpos+2
         if(termtype.ne.-4)goto 320
         rs4=rs4+1
         call compute_xb1(8,numterm,rs4-firstrs4+1,0,1,0.d0,
     *      0.d0,0.d0,xbh4(firstrs4),0.d0,dfullbeta(pos),
     *      dxbh4,0.d0,0.d0,0.d0,1,0.)
      elseif(mapmoel(moelpos).eq.3)then
c        Regressor set
         pos=mapmoel(moelpos+1)
         nx1=mapmoel(moelpos+2)
         moelpos=moelpos+3
         if(termtype.eq.-3)then
c           Even though only nu branches are used, the data contain nsb
c           (see figure_rset, figure_rspl, as called by figdata1-figdata).
            vpntr=vpntr+nx1*nsb
         elseif(termtype.eq.-4)then
            rs4=rs4+1
            call compute_xb1(4,numterm,rs4-firstrs4+1,nx1,1,
     *         fullbeta(pos),0.d0,0.d0,xbh4(firstrs4),
     *         dfullbeta(pos),0.d0,dxbh4,0.d0,0.d0,0.d0,1,v1(vpntr))
            vpntr=vpntr+nx1
         elseif(termtype.eq.-5)then
            vpntr=vpntr+nx1
         endif
      elseif(mapmoel(moelpos).eq.4)then
c        Integrated residual:
         moelpos=moelpos+4
      elseif(mapmoel(moelpos).eq.5)then
c        Duration spline:
         nds=nds+1
         moelpos=moelpos+2
      elseif(mapmoel(moelpos).eq.9)then
c        Variable
         moelpos=moelpos+1
         if(termtype.eq.-4)rs4=rs4+1
         vpntr=vpntr+1
      else
         call bug(ipt,jpt)
      endif
      goto 320
 
  999 return
      end
