      subroutine nextdraw(task,numnest,thisnest,nest,intdist,intdim,
     *   intdimc,numdist,sumintdim,sumdim2,sumdim4,sumbigdim,dimii,
     *   nfullplus,maxpnts,nec,allint6,posi1,posi2,posi4,bigposi1,
     *   bigposi2,nested,ne,thisdraw,draw,cdraw,ccdraw,ndraw,distdim,
     *   distpos,distyp,dimused,indx,intpnts,intpntr,sigii,cdm,
     *   dcdmdsigii,fullbeta,z,wt,weight,ui,cmui,lnl,ifdrv,ipt,jpt,
     *   status)
 
c     Subroutine to figure out the next draw of integrated distributions.
c
c     Upon output, task indicates what subroutine integrate needs to do:
c        4 = log-likelihood at level nest<numnest has been computed;
c            now weight and accumulate likelihood at level nest.
c        5 = all done.
c
c     Upon output, status indicates
c        0 = normal termination
c        1 = Cholesky decomposition failed
c
c     Constantijn Panis, 20 March 2002
c     Last change: lots of arrays only pertain to integrated distributions
 
      implicit none
      integer task,numnest,thisnest,nest,intdist(numnest),intdim,
     *   intdimc,numdist,sumintdim,sumdim2,sumdim4,sumbigdim,dimii,
     *   nfullplus,maxpnts,nec,allint6,posi1,posi2,posi4,bigposi1,
     *   bigposi2,nested(numdist,2),ne,thisdraw(numdist),
     *   draw(numdist,ne),cdraw(ne),ccdraw,ndraw(numdist),
     *   distdim(numdist),distpos(numdist),distyp(numdist),
     *   dimused(sumintdim),indx(sumintdim),intpnts(sumintdim),
     *   intpntr(sumintdim),i,ipt,jpt,status,countint,getposi1
      double precision sigii(dimii),cdm(sumdim2),dcdmdsigii(sumdim4),
     *   fullbeta(nfullplus),z(maxpnts),wt(maxpnts),weight(numnest),
     *   ui(sumintdim),cmui(sumbigdim),lnl(0:numnest)
      logical ifdrv
 
c     Select the next draw of intdist(nest), or move on to the next
c     distribution at the same nesting level (if any) or to the next
c     outer distribution for accumulating its likelihood and selecting
c     its next point.
  100 status=0
      if(thisdraw(intdist(nest)).ne.0)then
c        Go to the next draw of intdist and start with the first points.
         thisdraw(intdist(nest))=thisdraw(intdist(nest))-1
         if(thisdraw(intdist(nest)).eq.0)then
c           Note: if two or more independent distributions are nested at
c           the same level, we don't want to evaluate draw=0 for both,
c           as this would evaluate modules with such draw multiple times.
c           See, for example, Misc\nested6a.  Only evaluate the zero draw
c           for the first sister distribution.
            do 110 i=1,intdist(nest)-1
               if(nested(i,1).eq.nested(intdist(nest),1).and.
     *            nested(i,2).eq.nested(intdist(nest),2))goto 100
  110       continue
         endif
c        Initialize points:
         if(thisdraw(intdist(nest)).gt.0)then
            call putint(1,indx(posi1),intdim)
         else
            call putint(0,indx(posi1),intdim)
         endif
c        Compute Cholesky decomposition (if first point) and ui; initialize
c        more inner draws and points (if nest<numnest).  Determine thisnest.
         call initdrawpoint(nest,numnest,thisnest,ne,intdist,intdim,
     *      intdimc,numdist,sumintdim,sumdim2,sumdim4,sumbigdim,
     *      dimii,nfullplus,maxpnts,nec,allint6,posi1,posi2,posi4,
     *      bigposi1,bigposi2,nested,thisdraw,draw,cdraw,ccdraw,ndraw,
     *      distdim,distpos,distyp,indx,dimused,intpnts,intpntr,sigii,
     *      cdm,dcdmdsigii,fullbeta,z,wt,weight,ui,cmui,ifdrv,ipt,jpt,
     *      status)
         if(status.gt.0)goto 999
c        Are there any modules with this draw?
         if(status.ne.0)goto 100
c        Ready to integrate the innermost distribution.
         task=1
         goto 999
      endif
 
c     All draws of intdist(nest) have been evaluated.  Check whether there
c     are more distributions nested within intdist(nest-1).  If so,
c     change intdist(nest) and integrate all its draws.
c     Note: this features greatly saves computations where multiple integrated
c     distributions separate in outcomes.  For example, the joint model of
c     Speed/joint.aml (bivariate distribution, zero correlation) requires 4^2
c     conditional likelihoods, whereas Speed/independent.aml (two univariate
c     distributions) only requires 4*2 conditional likelihoods.  Also see the
c     500 loop in fignest.
      do 120 i=intdist(nest)+1,numdist
         if(nested(i,1).ne.nested(intdist(nest),1).or.
     *      nested(i,2).ne.nested(intdist(nest),2))goto 120
c        Distribution (i) is also at nest and also nested under intdist(nest-1):
         intdist(nest)=i
c        Initialize draw to highest draw.  May be too high, in which case
c        control will later be passed to label 100.
         thisdraw(i)=ndraw(i)
         posi1=getposi1(i,nested,distdim)
         intdim=distdim(i)
         call putint(1,indx(posi1),intdim)
c        Initialize more inner draws and points:
         call initdrawpoint(nest,numnest,thisnest,ne,intdist,intdim,
     *      intdimc,numdist,sumintdim,sumdim2,sumdim4,sumbigdim,
     *      dimii,nfullplus,maxpnts,nec,allint6,posi1,posi2,posi4,
     *      bigposi1,bigposi2,nested,thisdraw,draw,cdraw,ccdraw,ndraw,
     *      distdim,distpos,distyp,indx,dimused,intpnts,intpntr,sigii,
     *      cdm,dcdmdsigii,fullbeta,z,wt,weight,ui,cmui,ifdrv,ipt,jpt,
     *      status)
c        Are there any modules with this draw?
         if(status.gt.0)goto 999
c        Are there any modules with this draw?
         if(status.ne.0)goto 100
c        Ready to integrate the innermost distribution(s)
         task=1
         goto 999
  120 continue
 
c     All draws of intdist(nest) and other integrated distributions
c     under intdist(nest-1) (if any) have been evaluated.  Before moving
c     on to the next point of the next outer distribution, invoke sumlnl2l
c     to convert the log-likelihood of its current draw into the likelihood,
c     weight it, and accumulate it.
  200 if(nest.eq.1.and.countint(0,thisdraw,numdist).eq.numdist)then
c        Integration complete.
         task=5
         goto 999
      endif
      nest=nest-1
      call getposi(intdist(nest),intdim,intdimc,numdist,sumintdim,
     *   distdim,thisdraw,ndraw,dimused,nested,posi1,posi2,posi4,
     *   bigposi1,bigposi2)
c     Only accumulate in sumlnl2l if one or more modules have been evaluated.
      if(lnl(nest).eq.0.d0)goto 100
      task=4
      goto 999
 
  999 return
      end
 
