      subroutine nextpoint(task,numnest,thisnest,nest,intdist,intdim,
     *   intdimc,numdist,sumintdim,sumdim2,sumdim4,sumbigintdim,dimii,
     *   nfullplus,maxpnts,nec,allint6,posi1,posi2,posi4,bigposi1,
     *   bigposi2,nested,ne,thisdraw,draw,cdraw,ccdraw,ndraw,distdim,
     *   distpos,distyp,dimused,indx,intpnts,intpntr,sigii,cdm,
     *   dcdmdsigii,fullbeta,z,wt,weight,ui,cmui,ifdrv,ipt,jpt,status)
 
c     Subroutine to figure out what the next support point (vector) is of
c     integrated distribution intdist.  This point (vector) may be a
c     finite mixture group or the approximation to a normal distribution.
c
c     Upon input, task determines what this routine does:
c        1 = initialize draws and points for first time
c        2 = determine the next points
c
c     Upon output, task indicates what subroutine integrate needs to do:
c        1 = Next point has been assigned.  Compute conditional log-
c            likelihood (onepoint), weight and accumulate likelihood
c            (sumlnl2l) (only for nest=thisnest);
c        2 = All points have been evaluated.  Accumulate like(nest) into
c            lnl(nest-1) (suml2lnl), then draw next draw (nextdraw).
c        3 = Assignment of next point required initialization of more
c            inner distributions, resulting in a draw that never occurs.
c            Draw the next draw (nextdraw).
c
c     Upon output, status indicates:
c        0 = normal termination
c        1 = Cholesky decomposition failed
c
c     Constantijn Panis, 3 January 2000
c     Last change: set initial draw to highest integrated draw, not
c           to ndraw(.).  Fixed in aML 2.52.
 
      implicit none
      integer task,numnest,thisnest,nest,intdist(numnest),intdim,
     *   intdimc,numdist,sumintdim,sumdim2,sumdim4,sumbigintdim,dimii,
     *   nfullplus,maxpnts,nec,allint6,posi1,posi2,posi4,bigposi1,
     *   bigposi2,nested(numdist,2),ne,draw(numdist,ne),cdraw(ne),
     *   ccdraw,thisdraw(numdist),ndraw(numdist),distdim(numdist),
     *   distpos(numdist),distyp(numdist),dimused(sumintdim),
     *   indx(sumintdim),intpnts(sumintdim),intpntr(sumintdim),
     *   ipt,jpt,status,i,introwmax,getposi1
      double precision sigii(dimii),cdm(sumdim2),dcdmdsigii(sumdim4),
     *   fullbeta(nfullplus),z(maxpnts),wt(maxpnts),weight(numnest),
     *   ui(sumintdim),cmui(sumbigintdim)
      logical ifdrv
 
      status=0
      if(task.eq.1)goto 100
      if(task.eq.2)goto 200
      call bug(ipt,jpt)
 
c     Very first call; start with (the first) outermost nested distribution
c     (nested=1), its highest draw and first point.  Initdrawpoint will
c     initialize inner distributions to highest draws and first points.
c     It will also determine whether thisnest (the number of nesting levels
c     for the current integration branch) is numnest, or less.
  100 nest=1
      call intmatch(nest,nested,numdist,intdist(nest))
      intdim=distdim(intdist(nest))
      posi1=getposi1(intdist(nest),nested,distdim)
c     Next line used to set thisdraw(.) to ndraw(.), but there may be
c     draws generated by non-integrated residuals in continuous models;
c     see Test\Trouble\trouble10.
      thisdraw(intdist(nest))=
     *   introwmax(draw(intdist(nest),1),numdist,ne)
      call putint(1,indx(posi1),intdim)
      goto 400
 
c     Intdist(nest) denotes the distribution currently being integrated out.
c     Go to the next heterogeneity point:
  200 if(indx(posi1).eq.0)goto 300
      do 210 i=posi1+intdim-1,posi1,-1
         if(dimused(i).eq.0)goto 210
         if(indx(i).eq.intpnts(i))then
            indx(i)=1
         else
            indx(i)=indx(i)+1
            goto 400
         endif
  210 continue
      goto 300
 
c     All approximation points of thisdraw(intdist(nest)) have been evaluated,
c     including the case where thisdraw(.)=0, so that only one
c     "approximation point" (namely zero) was evaluated.  Accumulate the
c     log-likelihood at nest into the likelihood at nest-1 (suml2lnl).
  300 task=2
      goto 999
 
c     Figure out the Cholesky decomposition of the covariance matrix of
c     the current draw of intdist(nest), and the integration points ui.
c     Also initialize the draws and points of more inner distributions
c     and set intdist, intdim, intdimc, and lots of pointers to refer
c     to the innermost distribution (thisnest).
  400 call initdrawpoint(nest,numnest,thisnest,ne,intdist,intdim,
     *   intdimc,numdist,sumintdim,sumdim2,sumdim4,sumbigintdim,dimii,
     *   nfullplus,maxpnts,nec,allint6,posi1,posi2,posi4,bigposi1,
     *   bigposi2,nested,thisdraw,draw,cdraw,ccdraw,ndraw,distdim,
     *   distpos,distyp,indx,dimused,intpnts,intpntr,sigii,cdm,
     *   dcdmdsigii,fullbeta,z,wt,weight,ui,cmui,ifdrv,ipt,jpt,status)
      if(status.gt.0)goto 999
 
c     Initialization of more inner distributions should not(!) imply a draw
c     which never occurs, not here in nextpoint.
      if(status.lt.0)call bug(ipt,jpt)
 
c     Evaluate the conditional log-likelihood of this draw/point
c     (onepoint) and accumulate it to the likelihood (sumlnl2l).
      task=1
      goto 999
 
  999 return
      end
 
