      integer function nmbrnch(level1,branch,level2,nbdim,nb)
 
c     Integer function to determine the number of level level2 branches
c     in level level 1 branch indicated by branch.  Arguments:
c        level1 [in] level from which subbranches need to be counted
c        branch [in] vector indicating which level 2 and higher branch
c                    is taken to level level1
c        level2 [in] level of which subbranches need to be counted
c        nbdim  [in] dimension of nb
c        nb     [in] matrix containing numbers of subbranches
c
c     For example, to determine the number of level 5 branches in the 2nd
c     level 3 branch off the 6-th level 2 branch:
c        branch(2)=6
c        branch(3)=2
c        num = nmbrnch(3,branch,5,nbdim,nb)
c
c     Note:  if level2<level1, nmbrnch=0
c            if a branch does not exist, nmbrnch=0
c            if level2 exceeds the level of this data structure,
c               anything can happen.
c
c     Constantijn Panis, 4 July 1996
 
      implicit none
      integer level1,level2,branch(level1),nbdim,nb(nbdim,2),
     *   num,pos,i,n,level,posnb
 
      nmbrnch=0
      if(level2.lt.level1)goto 999
 
c     Where to start looking in nb?
      pos=posnb(level1,branch,nbdim,nb)
      if(pos.eq.0)goto 999
 
c     Accumulate number of subbranches:
      num=1
      do 100 level=level1+1,level2
         if(pos.eq.0)goto 999
         n=num
         num=0
         do 110 i=pos,pos+n-1
            if(nb(i,2).ne.0.or.level.eq.level2)num=num+nb(i,1)
  110    continue
         if(level.eq.level2)goto 100
c        To get to the numbers of subbranches of the next lower level,
c        find the first nonzero position at or after nb(pos,2):
         do 120 i=0,n-1
            if(nb(pos+i,2).eq.0)goto 120
            pos=nb(pos+i,2)
            goto 100
  120    continue
c        There are no more subbranches; nmbrnch=0
         goto 999
  100 continue
      nmbrnch=num
 
  999 return
      end
