      subroutine onepoint(numdist,sumintdim,ne,neh,nel,nep,nec,neb,nen,
     *   nepo,neml,nneml,nfullplus,dimlam,dimv,vpntr1,vpntr6,vpntr5,
     *   vpntr5n,vpntr5po,vpntr7,distdim,nested,allint6,lampos,lamtyp,
     *   cdraw,thisdraw,ccdraw,bdc,ipt,jpt,lnl,ui,dfullbeta,lam,epsh,
     *   epsl,epsp,epsc,epsb,epsn,epspo,epsml,dxbh,xbc,dxbc,dxblp,lnla,
     *   lnlb,dlnla,dlnlb,a6,siga4a4,dsiga4a4,siga6a6,dsiga6a6,
     *   siga6a6ia6,cmua4,sigl4l6,tauminxb,dtau,lndeta6a6,a5,da5,p5,dp5,
     *   xbml,dxbml,v,ifdrv,iid4,ibnnid4,status)
 
c     Subroutine to evaluate the joint log-likelihood, conditional on
c     a particular heterogeneity point.  Note that this routine
c     accumulates lnl and derivatives; the user is responsible for
c     initialization to zero prior to calling this routine.
c
c     Upon exit, status indicates:
c       -1 = likelihood underflowed to zero (lnl=minexp())
c        0 = normal termination
c        1 = a probit standard deviation was zero
c        2 = logit/probit thresholds were not ordered
c        3 = illegitimate binomial or negative binomial probability
c
c     Note: xbc has dimension (nec,0:rscmax), but only the first
c     column is used.  Similar things apply to a5, p5, xbml.
c
c     Note:  siga6a6 contains the inverse of the loaded and
c     accumulated covariance matrix siga6a6.
c
c     Note:  if residuals are integrated-out, the corresponding load
c     matrices are always generic (lamtyp=0), so that there is no need
c     for special checks.
c
c     Constantijn Panis, 8 December 2003
c     Last change: added check for lamtyp.  Was bug; see Test\Trouble\trouble13
 
      implicit none
      integer numdist,sumintdim,ne,neh,nel,nep,nec,neb,nen,nepo,neml,
     *   nneml,nfullplus,dimlam,dimv,vpntr1,vpntr6,vpntr5,vpntr5n,
     *   vpntr5po,vpntr7,distdim(numdist),nested(numdist),allint6,
     *   lampos(numdist,8),lamtyp(numdist,8),cdraw(ne),ccdraw,bdc(nep),
     *   thisdraw(numdist),match,posl4l6,lp,ie1,i,dim,pos,ipt,jpt,status
      double precision lnl,ui(sumintdim),
     *   dfullbeta(nfullplus),lam(dimlam),
     *   epsh(neh),epsl(nel),epsp(nep),epsc(nec),epsb(neb),epsn(nen),
     *   epspo(nepo),epsml(nneml),dxbh(neh),xbc(nec),lnla(neh),
     *   lnlb(neh),dxbc(nec),dlnla(nfullplus,neh),dlnlb(nfullplus,neh),
     *   a6(nec),siga6a6(nec,nec),siga6a6ia6(nec),
     *   cmua4(nep),sigl4l6(numdist*nep*nec),
     *   dsiga6a6(nec,nec),tauminxb(2,nel+nep),dtau(2,nel+nep),
     *   dxblp(nel+nep),siga4a4(nep,nep),dsiga4a4(nep,nep),
     *   lndeta6a6,a5(nen),da5(nen),p5(neb+nen+nepo),dp5(neb+nen+nepo),
     *   xbml(nneml),dxbml(nneml),minexp
      real v(dimv)
      logical ifdrv,iid4,ibnnid4
 
      status=0
 
c     Nextdraw ensures at least one likelihood module is to be evaluated.
      call intmatch(ccdraw,cdraw,ne,match)
      if(match.eq.0)call bug(ipt,jpt)
 
c     Start with continuous models (if they have integrated residuals):
      if(nec.gt.0.and.allint6.eq.1)then
         call zerodp(nec,epsc)
         pos=1
         do 140 i=1,numdist
            dim=distdim(i)
            if(thisdraw(i).gt.0.and.lamtyp(i,4).ne.-1)then
               lp=lampos(i,4)+(thisdraw(i)-1)*dim*nec+1
               call m2mult(000,nec,dim,1,lam(lp),ui(pos),epsc,1)
            endif
            if(nested(i).gt.0)pos=pos+dim
  140    continue
         call onecontin(nec,cdraw(neh+nel+nep+1),ccdraw,
     *      xbc,epsc,dxbc,a6,lnl,siga6a6,dsiga6a6,siga6a6ia6,
     *      lndeta6a6,v(vpntr6),ifdrv)
c        Compute the conditional mean for probits, cmua4:
         if(nep.gt.0.and..not.iid4)then
            call zerodp(nep,cmua4)
            posl4l6=1
            do 120 i=1,numdist
               if(nested(i).eq.0)call m2mult(000,nep,nec,1,
     *            sigl4l6(posl4l6),siga6a6ia6,cmua4,1)
               posl4l6=posl4l6+nep*nec
  120       continue
         endif
      endif
 
      ie1=1
 
c     Compute hazard probabilities for this heterogeneity group:
      if(neh.gt.0)then
         call zerodp(neh,epsh)
         pos=1
         do 210 i=1,numdist
            dim=distdim(i)
            if(thisdraw(i).gt.0.and.lamtyp(i,1).ne.-1)then
               lp=lampos(i,1)+(thisdraw(i)-1)*dim*neh+1
               call m2mult(000,neh,dim,1,lam(lp),ui(pos),epsh,1)
            endif
            if(nested(i).gt.0)pos=pos+dim
  210    continue
         call onehazard(nfullplus,neh,cdraw(ie1),ccdraw,lnl,epsh,
     *      dxbh,lnla,lnlb,dlnla,dlnlb,dfullbeta,v(vpntr1),
     *      ifdrv,status)
         if(status.eq.-1)goto 900
         ie1=ie1+neh
      endif
 
c     Compute logit probabilities for this heterogeneity group:
      if(nel.gt.0)then
         call zerodp(nel,epsl)
         pos=1
         do 220 i=1,numdist
            dim=distdim(i)
            if(thisdraw(i).gt.0.and.lamtyp(i,2).ne.-1)then
               lp=lampos(i,2)+(thisdraw(i)-1)*dim*nel+1
               call m2mult(000,nel,dim,1,lam(lp),ui(pos),epsl,1)
            endif
            if(nested(i).gt.0)pos=pos+dim
  220    continue
         call onelogit(nel,cdraw(ie1),ccdraw,lnl,tauminxb,
     *      dtau,dxblp,epsl,ifdrv,ipt,jpt,status)
c        Status may be -1 (underflow), 2 (thresholds not ordered).
         if(status.ne.0)goto 900
         ie1=ie1+nel
      endif
 
c     Compute probit probabilities for this heterogeneity group:
      if(nep.gt.0)then
         call zerodp(nep,epsp)
         pos=1
         do 230 i=1,numdist
            dim=distdim(i)
            if(thisdraw(i).gt.0.and.lamtyp(i,3).ne.-1)then
               lp=lampos(i,3)+(thisdraw(i)-1)*dim*nep+1
               call m2mult(000,nep,dim,1,lam(lp),ui(pos),epsp,1)
            endif
            if(nested(i).gt.0)pos=pos+dim
  230    continue
         call oneprobit(nep,cdraw(ie1),ccdraw,bdc,lnl,
     *      tauminxb(1,nel+1),dtau(1,nel+1),dxblp(nel+1),siga4a4,
     *      dsiga4a4,cmua4,epsp,ifdrv,ibnnid4,ipt,jpt,status)
c        Status may be -1 (underflow), 1 (std=0), 2 (thresholds
c        not ordered), 3 (non-positive definite correlation matrix)
         if(status.ne.0)goto 900
         ie1=ie1+nep
      endif
 
c     Continuous outcomes were done above
      ie1=ie1+nec
 
c     Compute integrand for binomial modules:
      if(neb.gt.0)then
         call zerodp(neb,epsb)
         pos=1
         do 240 i=1,numdist
            dim=distdim(i)
            if(thisdraw(i).gt.0.and.lamtyp(i,5).ne.-1)then
               lp=lampos(i,5)+(thisdraw(i)-1)*dim*neb+1
               call m2mult(000,neb,dim,1,lam(lp),ui(pos),epsb,1)
            endif
            if(nested(i).gt.0)pos=pos+dim
  240    continue
         call onebin(neb,cdraw(ie1),ccdraw,p5,dp5,epsb,lnl,v(vpntr5),
     *      ifdrv,ipt,jpt,status)
c        Status may be -1 (underflow) or 3 (illegitimate probability)
         if(status.ne.0)goto 900
         ie1=ie1+neb
      endif
 
c     Compute integrand for negative binomial episodes:
      if(nen.gt.0)then
         call zerodp(nen,epsn)
         pos=1
         do 250 i=1,numdist
            dim=distdim(i)
            if(thisdraw(i).gt.0.and.lamtyp(i,6).ne.-1)then
               lp=lampos(i,6)+(thisdraw(i)-1)*dim*nen+1
               call m2mult(000,nen,dim,1,lam(lp),ui(pos),epsn,1)
            endif
            if(nested(i).gt.0)pos=pos+dim
  250    continue
         call onenegbin(nen,cdraw(ie1),ccdraw,a5,da5,p5(neb+1),
     *      dp5(neb+1),epsn,lnl,v(vpntr5n),ifdrv,ipt,jpt,status)
c        Status may be -1 (underflow), 3 (illegitimate probability)
         if(status.ne.0)goto 900
         ie1=ie1+nen
      endif
 
c     Compute integrand for Poisson modules:
      if(nepo.gt.0)then
         call zerodp(nepo,epspo)
         pos=1
         do 255 i=1,numdist
            dim=distdim(i)
            if(thisdraw(i).gt.0.and.lamtyp(i,7).ne.-1)then
               lp=lampos(i,7)+(thisdraw(i)-1)*dim*nepo+1
               call m2mult(000,nepo,dim,1,lam(lp),ui(pos),epspo,1)
            endif
            if(nested(i).gt.0)pos=pos+dim
  255    continue
         call onepoisson(nepo,cdraw(ie1),ccdraw,p5(neb+nen+1),
     *      dp5(neb+nen+1),epspo,lnl,v(vpntr5po),ifdrv,ipt,jpt)
         ie1=ie1+nepo
      endif
 
c     Compute integrand for multinomial logit outcomes:
      if(neml.gt.0)then
         call zerodp(nneml,epsml)
         pos=1
         do 260 i=1,numdist
            dim=distdim(i)
            if(thisdraw(i).gt.0.and.lamtyp(i,8).ne.-1)then
               lp=lampos(i,8)+(thisdraw(i)-1)*dim*nneml+1
               call m2mult(000,nneml,dim,1,lam(lp),ui(pos),epsml,1)
            endif
            if(nested(i).gt.0)pos=pos+dim
  260    continue
         call onemnlogit(neml,nneml,cdraw(ie1),ccdraw,xbml,dxbml,epsml,
     *      lnl,v(vpntr7),ifdrv,ipt,jpt,status)
c        Status may be -1 (underflow)
         if(status.ne.0)goto 900
         ie1=ie1+neml
      endif
      goto 999
 
c     Upon exit, status indicates:
c       -1 = likelihood underflowed to zero
c        0 = normal termination
c        1 = a probit standard deviation was zero
c        2 = logit or probit thresholds were not ordered
c        3 = illegitimate (negative) binomial probability
c        4 = non-positive definite correlation matrix
  900 if(status.eq.-1)lnl=minexp()
  999 return
      end
