      subroutine onepoisson(nepo,draw,cdraw,p5,dp5,
     *   epspo,lnlg,v5,ifdrv,ipt,jpt)
 
c     Subroutine to compute the conditional (on heterogeneity) integrand
c     of all Poisson modules.
c
c     Conditional likelihood of one outcome is:
c
c              exp(-lambda) * lambda^y
c     P(Y=y) = ------------------------
c                      y!
c
c     ln(P(Y=y)) = -lambda + y*ln(lambda) - ln(y!)
c
c     Incidence rate lambda is always an exponential:
c        lambda = exp( ln(exposure) + Beta'X + eps)
c     where eps can be any (integrated) residual structure.
c
c     Constantijn Panis, 10 December 2002
c     Last change: no more underflow detection
 
      implicit none
      integer nepo,draw(nepo),cdraw,ie,ipt,jpt
      double precision p5(nepo),dp5(nepo),epspo(nepo),
     *   lnlg,expo,y,lambda,dlgama
      real v5(2*nepo)
      logical ifdrv
 
c     Compute the log of the integrand, -lambda + y*ln(lambda)
      do 100 ie=1,nepo
c        Do we need to evaluate this (conditional) likelihood module?
         if(cdraw.ne.draw(ie))goto 100
         y=dble(v5(ie))
         expo=dble(v5(ie+nepo))
         lambda = expo*dexp(p5(ie)+epspo(ie))
         lnlg = lnlg - lambda + y*dlog(lambda) - dlgama(y+1.d0)
         if(.not.ifdrv)goto 100
 
c        Derivatives of log-integrand wrt p5:
         dp5(ie) = -lambda + y
 
  100 continue
 
  999 return
      end
