      subroutine prep5n(task,nfullplus,nmapmoel,nen,dimp,numdist,
     *   dimlam,distdim,lamtyp,lampos,dimv,mapmoel,moelpos,rsnamax,
     *   rsnpmax,fullbeta,dfullbeta,a5,da5,p5,dp5,lam,dlam,lnlobs,
     *   v,vpntr,ipt,jpt,status)
 
c     Subroutine to prepare and unravel negative binomial modules in ithobs.
c     Task=1: compute A and probability location, set up lam for
c             probability integrated residual.  Compute log of product
c             over episodes of product over intervals of gamma-ratio.
c     Task 2: unravel everything into wrt fullbeta.
c
c     Upon output, status=1 if the exposure is negative or zero.
c
c     Constantijn Panis, 20 December 2002
c     Last change: exposure
 
      implicit none
      integer task,nfullplus,nmapmoel,nen,dimp,numdist,dimlam,
     *   distdim(numdist),lamtyp(numdist),lampos(numdist),dimv,
     *   mapmoel(nmapmoel),rsnamax,rsnpmax,ipt,jpt,status,
     *   moelpos,vpntr,ie,vpntr5,exponent
      double precision fullbeta(nfullplus),dfullbeta(nfullplus),
     *   a5(nen,0:rsnamax),da5(nen),p5(dimp,0:rsnpmax),dp5(dimp),
     *   lam(dimlam),dlam(dimlam),lnlobs,aa,yy,daa,dlgama,digama
      real v(dimv)
 
      vpntr5=vpntr-1
 
c     The first positions in v are for:
c        nen negative binomial count outcomes
c        nen negative binomial probability transformation types
c        nen negative binomial dispersion transformation (exp)
c        nen negative binomial exposure values
      vpntr=vpntr+4*nen
 
      if(task.eq.2)goto 200
 
      do 120 ie=1,nen
         if(v(vpntr).ne.real(ie))call bug(ipt,jpt)
         vpntr=vpntr+1
 
c        First compute A (dispersion/scale):
         call prepfunc(1,ie,nen,0,nmapmoel,dimv,mapmoel,moelpos,vpntr,
     *      nfullplus,rsnamax,numdist,distdim,0,0,0,fullbeta,dfullbeta,
     *      a5,da5,0.d0,0.d0,0.d0,v,ipt,jpt)
 
c        Then compute p5 (probability location) and the load matrix on
c        integrated residuals.
         call prepfunc(1,ie,dimp,nen,nmapmoel,dimv,mapmoel,moelpos,
     *      vpntr,nfullplus,rsnpmax,numdist,distdim,dimlam,lamtyp,
     *      lampos,fullbeta,dfullbeta,p5,dp5,lam,dlam,0.d0,v,ipt,jpt)
 
c        The joint likelihood is the product over episodes of the product
c        over t of Gamma(A+Y)/(Gamma(A)*Gamma(Y+1)) times some integrated
c        stuff, which is dealt with later.  Here compute the part of the
c        log-likelihood that does not depend on heterogeneity.
c        Compute (the log of) the term that can be taken outside the
c        heterogeneity integral and its derivative wrt A:
         exponent=int(v(vpntr5+2*nen+ie))
         if(exponent.eq.1)then
c           dispersion= ...;   Check positive and compute 1/alpha:
            if(a5(ie,0).le.0.d0)goto 800
            aa=1.d0/a5(ie,0)
         elseif(exponent.eq.2)then
c           dispersion = exp(...);  Compute 1/alpha
            a5(ie,0)=dexp(a5(ie,0))
            aa=1.d0/a5(ie,0)
         elseif(exponent.eq.3)then
c           scale = exp(...)
            a5(ie,0)=dexp(a5(ie,0))
            aa=a5(ie,0)
         else
            call bug(ipt,jpt)
         endif
         yy=dble(v(vpntr5+ie))
c        Save computations in some common cases:
         if(yy.eq.0.d0)then
c           ratio of gammas is 1:
            continue
         elseif(yy.eq.1.d0)then
c           gamma(y+1) is 1; gamma(a+1)=a*gamma(a):
            lnlobs=lnlobs + dlog(aa)
         elseif(yy.eq.2.d0)then
c           gamma(y+1) is 2; gamma(a+2)=(a+1)*a*gamma(a):
            lnlobs=lnlobs + dlog(.5d0*aa*(aa+1))
         else
c           General formula:
            lnlobs=lnlobs+dlgama(aa+yy)-dlgama(aa)-dlgama(yy+1.d0)
         endif
 
  120 continue
      goto 999
 
c     Unravel everything into wrt fullbeta.
  200 continue
      do 220 ie=1,nen
         if(v(vpntr).ne.real(ie))call bug(ipt,jpt)
         vpntr=vpntr+1
 
c        First add derivatives wrt lnlobs (part that does not depend
c        on heterogeneity):
         exponent=int(v(vpntr5+2*nen+ie))
         if(exponent.eq.1)then
            aa=1.d0/a5(ie,0)
         elseif(exponent.eq.2)then
            aa=1.d0/a5(ie,0)
         elseif(exponent.eq.3)then
            aa=a5(ie,0)
         else
            call bug(ipt,jpt)
         endif
         yy=dble(v(vpntr5+ie))
         if(yy.eq.0.d0)then
            daa=0.d0
         elseif(yy.eq.1.d0)then
            daa=1.d0/aa
         elseif(yy.eq.2.d0)then
            daa=(2.d0*aa+1.d0)/(aa*(aa+1.d0))
         else
            daa=digama(aa+yy)-digama(aa)
         endif
c        Convert derivative wrt scale (A) or inverse dispersion (1/A) into wrt B
         if(exponent.eq.1)then
            da5(ie)=da5(ie)-daa*aa*aa
         elseif(exponent.eq.2)then
            da5(ie)=(da5(ie)-daa*aa*aa)/aa
         elseif(exponent.eq.3)then
            da5(ie)=(da5(ie)+daa)*aa
         endif
 
c        Unravel wrt the dispersion/scale parameters (A):
         call prepfunc(2,ie,nen,0,nmapmoel,dimv,mapmoel,moelpos,vpntr,
     *      nfullplus,rsnamax,numdist,distdim,0,0,0,fullbeta,dfullbeta,
     *      a5,da5,0.d0,0.d0,0.d0,v,ipt,jpt)
c        Unravel wrt the probability location and the load
c        matrix for integrated residuals in the probability:
         call prepfunc(2,ie,dimp,nen,nmapmoel,dimv,mapmoel,moelpos,
     *      vpntr,nfullplus,rsnpmax,numdist,distdim,dimlam,lamtyp,
     *      lampos,fullbeta,dfullbeta,p5,dp5,lam,dlam,0.d0,v,ipt,jpt)
 
  220 continue
      goto 999
 
  800 if(ipt.gt.0)write(8,801)
      if(jpt.gt.0)write(6,801)
  801 format(/,'ERROR: the function evaluation failed because the ',
     *   'dispersion of a negative',/,'binomial model is zero or ',
     *   'negative.',/,'Consider specifying "dispersion=exp(...)" ',
     *   'in the model statement.')
      status=1
      goto 999
  999 return
      end
 
