      subroutine quadratic032(task,nrowa,numblockb,dimblockb,
     *   a,bblock,c,factor)
 
c     Subroutine to compute a quadratic form:
c
c     task = 0: compute A*B*A' = C    A(nrowa, dimb)     dimb = numblockb*dimblo
c                                     B(dimb, dimb)
c                                     C(nrowa, nrowa)
c     task = 1: compute A'*B*A = C    A(dimb, nrowa)
c                                     B(dimb, dimb)
c                                     C(nrowa, nrowa)
c
c     Matrix types can be:
c        0 = generic
c        1 = identity (Only its dimension is stored.)
c        2 = symmetric
c        3 = symmetric and block-diagonal (Only its first block, its
c            dimension, and number of blocks are stored.)
c        4 = sparse in the sense that there is at most one nonzero
c            element in any one row.  (Only its nonzero elements
c            and their column position are stored.)
c
c     This routine only applies to A = generic, B = symmetric and block-diagonal
c     and C is symmetric.
c     If this is not the case, instead use
c        m3mult(0010,nrowa,nrowb,ncolb,nrowa,a,b,a,c,add).
c
c     Integer factor determines:
c        factor = 0     C = A*B*A'
c              else     C = C + factor*A*B*A'
c
c     If factor!=0, it is assumed that upon input, C is already
c     symmetric, so that upon output, C is still symmetric.
c
c     Note some rules for differentiation.   If C = A*B*A', then
c        dA = dC*A*B' + dC'*A*B = 2*dC*A*B if dC and B are symmetric.
c        dB = A'*dC*A
c
c     This routine is very obscure.  See m3mult (which computes the
c     product of three arbitrary matrices) for details.  The current
c     routine is a special case of m3mult, taking advantage of the
c     fact that C is symmetric.
c
c     Constantijn Panis, 2 December 2001
 
      implicit none
      integer task,nrowa,numblockb,dimblockb,factor,i,j,k,l,
     *   ia,ic,id,ja,jc,jd,sa,sc,sd,jsa,jsc,jsd,cib,dimb
      double precision a(nrowa*numblockb*dimblockb),
     *   bblock(dimblockb*dimblockb),c(nrowa*nrowa),abij
 
      if(factor.eq.0)call zerodp(nrowa*nrowa,c)
 
      dimb=numblockb*dimblockb
      if(task.eq.0)then
         ja=nrowa
         jsa=1
         jc=1
         jsc=nrowa
      else
         ja=1
         jsa=dimb
         jc=dimb
         jsc=1
      endif
      jd=nrowa
      jsd=1
 
      sa=1
      sd=1
      do 110 i=1,nrowa
         sc=1
         do 120 j=1,dimb
            abij = 0.d0
            ic=sc
            id=sd
c           Loop over k=1,dimb to compute the innerproduct of A(i,.)
c           and B(.,j).  No need to cover the entire B(.,j), since
c           B is blockdiagonal.
            ia=sa+((j-1)/dimblockb)*dimblockb*ja
            cib=1+mod(j-1,dimblockb)*dimblockb
            do 130 k=1,dimblockb
               abij = abij + a(ia)*bblock(cib)
               ia=ia+ja
               cib=cib+1
  130       continue
            if(factor.ne.0.and.factor.ne.1)abij=dble(factor)*abij
            do 140 l=1,i
               c(id) = c(id) + abij*a(ic)
               id=id+jd
               ic=ic+jc
  140       continue
            sc=sc+jsc
  120    continue
         sa=sa+jsa
         sd=sd+jsd
  110 continue
      call reflect(c,nrowa)
 
      return
      end
