      subroutine read_model46(levelmax,nfull,nummodel,maxmodel,mdllvl,
     *   datadim,numdist,numrset,numspl,numpar,nold,prof,maxmeno,meno,
     *   moel,moelpntr,distdim,mtype,strctre,datastructure,datas,
     *   numfreq,maxnumfreq,nfreq,freqloc,nxold,namepntr,modeltype,
     *   minlnl,xnodepos,maxxnode,parpos,pardim,fulltype,xnode,
     *   iid4,rsetname,splname,parname,resname,oldname,line,il,ipt,jpt)
 
c     Subroutine to read the model specification for a certain data
c     structure.  The syntax is:
c
c     TOBIT MODEL; [DATA STRUCTURE = n;]
c        [({KEEP | DROP} IF ...)];
c        [NUMERATOR;]  [DENOMINATOR;]
c        OUTCOME = varname;
c        [LOWER LIMIT = ...;] [UPPER LIMIT = ...;]
c        MODEL = REGSET ... + REGSPLINE ... + INTRES ... + RES ...;
c
c     Lower and upper limits may be scalars, variables, parameters,
c     or regressor sets.
c
c     By default the likelihood function contains the modules related to
c     any model in the numerator.  If DENOMINATOR; is specified, the
c     module enters in the denominator only; if both NUMERATOR; and
c     DENOMINATOR; are specified, the module is replicated and enters
c     both ways.
c     The default translates into minlnl=0; denominator only minlnl=1;
c     numerator and denominator minlnl=2.
c
c     Subsequent moel(.) elements specify the model:
c        0=end of model specification
c       -3=what follows is a regressor set
c       -4=what follows is an integrated residual
c       -7=what follows is a residual
c        2=next factor is a parameter
c        3=next factor is a regressor set
c        4=next factor is a residual
c        7=next factor is an integrated residual
c        8=next factor is a regressor spline
c        9=next factor is a variable
c
c     Constantijn Panis, 7 September 2002
 
      implicit none
      integer levelmax,nfull,maxmodel,nummodel,datadim,numdist,numrset,
     *   numspl,numpar,nold,prof,maxmeno,meno,moel(maxmeno),
     *   moelpntr(maxmodel),distdim(numdist),strctre(maxmodel),
     *   datas(maxmodel),
     *   numfreq,maxnumfreq,freqloc(maxnumfreq),nfreq(maxmodel),
     *   mdllvl(maxmodel),
     *   nxold(levelmax,datadim),namepntr(levelmax,datadim),
     *   modeltype(maxmodel),mtype,minlnl(maxmodel),parpos(numpar),
     *   pardim(numpar),fulltype(nfull),xnodepos,maxxnode,
     *   struc,ds,resdone,il,ipt,jpt
      real xnode(maxxnode),datastructure(datadim)
      logical iid4,smtxt
      character line*128,rsetname(numrset)*12,splname(numspl)*12,
     *   parname(numpar)*12,resname(*)*12,oldname(nold)*8
 
      nummodel=nummodel+1
      if(nummodel.gt.maxmodel)call bug(ipt,jpt)
      modeltype(nummodel)=46
      moelpntr(nummodel)=meno+1
 
c     Record beginning position of summary stats; number follows below.
c     For outcome, record both outcome types (censored/not) and summary
c     statistics of noncensored outcomes.
      if(numfreq+2.lt.maxnumfreq)then
         numfreq=numfreq+1
         nfreq(nummodel)=numfreq
         freqloc(numfreq)=-1
         numfreq=numfreq+1
         freqloc(numfreq)=-1
      else
         numfreq=maxnumfreq
         nfreq(nummodel)=0
      endif
 
c     [DATA STRUCTURE = n;]
      call getstruc(datadim,struc,ds,il,ipt,jpt,datas(nummodel),
     *   strctre(nummodel),datastructure,line)
 
c     [{KEEP | DROP} IF condition;]
      call getkeepdrop(levelmax,xnodepos,moel(moelpntr(nummodel)),nold,
     *   maxxnode,nxold(1,ds),namepntr(1,ds),numfreq,maxnumfreq,freqloc,
     *   ipt,jpt,il,struc,xnode,oldname,line)
 
c     [NUMERATOR;]  [DENOMINATOR;]
      call getnumdenom(minlnl(nummodel),line,il)
 
c     OUTCOME = varname;   (or OUTCOME=expr)
      if(.not.smtxt(4,line,il,'OUTCOME ='))goto 800
      meno=moelpntr(nummodel)+1
      moel(meno)=xnodepos+1
      call readexpr(1,levelmax,levelmax,mdllvl(nummodel),1,nold,
     *   maxxnode,xnodepos,xnode,nxold(1,ds),namepntr(1,ds),0,0,0,
     *   oldname,line,il,ipt,jpt,struc,';')
c     Outcomes may be given as, for example, OUTCOME=0; but only if
c     there is only one level in the data.  Otherwise, a variable
c     is required.
      if(mdllvl(nummodel).eq.0)then
         if(levelmax.gt.1)goto 810
         mdllvl(nummodel)=1
      endif
 
c     [LOWER LIMIT = ...;] [UPPER LIMIT = ...;]
      call readlimit(levelmax,mdllvl(nummodel),maxmeno,moel,meno,nold,
     *   maxxnode,xnodepos,xnode,nxold(1,ds),namepntr(1,ds),numfreq,
     *   maxnumfreq,freqloc,oldname,line,il,ipt,jpt,struc)
 
c     MODEL=....;
      if(.not.smtxt(4,line,il,'MODEL ='))goto 800
      resdone=0
      call read_model(levelmax,struc,mdllvl(nummodel),nfull,
     *   numrset,numspl,numdist,numpar,distdim,maxmeno,moel,meno,
     *   maxxnode,xnodepos,numfreq,maxnumfreq,freqloc,nold,prof,
     *   nxold(1,ds),namepntr(1,ds),parpos,pardim,modeltype(nummodel),
     *   resdone,99,-1,xnode,rsetname,splname,parname,oldname,
     *   resname,'MODEL = ','Tobit',';',line,il,ipt,jpt)
c     Make sure a residual has been specified:
      if(resdone.eq.0)goto 850
      iid4=iid4.and.(resdone.eq.2)
 
c     Frequency tables will be computed for selected variables.  Array
c     nfreq stores at which position in freqloc the locations of data
c     structure m start and how many such variables there are.
      numfreq=min(numfreq,maxnumfreq)
      if(nfreq(nummodel).gt.0)nfreq(nummodel) = nfreq(nummodel) +
     *   (numfreq-nfreq(nummodel)+1)*(maxnumfreq+1)
      goto 999
 
  800 if(ipt.gt.0)then
         call caretline(8,line,il)
         write(8,801)
      endif
      if(jpt.gt.0)then
         call caretline(6,line,il)
         write(6,801)
      endif
  801 format(/,'ERROR encountered while reading a Tobit model ',
     *   'specification.  The ',/,'syntax is:',/,'TOBIT MODEL;',/,
     *   '   [DATA STRUCTURE = n;]',/,
     *   '   [({KEEP | DROP} IF ...)];',/,
     *   '   [NUMERATOR;]  [DENOMINATOR;]',/,
     *   '   OUTCOME = ...;',/,
     *   '   [LOWER LIMIT = ...;] [UPPER LIMIT = ...;] ',/,
     *   '   MODEL = ...;')
      stop
  810 if(ipt.gt.0)write(8,811)
      if(jpt.gt.0)write(6,811)
  811 format('ERROR:  the outcome in a Tobit model specification does ',
     *   'not involve a',/,'data variable, so that aML does not know ',
     *   'at what level (how often) the',/,'equation contributes to ',
     *   'the likelihood.')
      stop
  850 if(ipt.gt.0)then
         call caretline(8,line,il)
         write(8,851)
      endif
      if(jpt.gt.0)then
         call caretline(6,line,il)
         write(6,851)
      endif
  851 format('ERROR: a Tobit model specification is incomplete.',/,
     *   'At a minimum, there must be a RESIDUAL model element.')
      stop
  999 return
      end
