      subroutine read_model5(levelmax,nfull,nummodel,maxmodel,
     *   mdllvl,datadim,strctre,datastructure,datas,mtype,modeltype,
     *   minlnl,maxmeno,moel,meno,moelpntr,numrset,numpar,numspl,
     *   numdist,distdim,numfreq,maxnumfreq,nfreq,freqloc,nxold,
     *   namepntr,nold,prof,maxxnode,xnodepos,il,ipt,jpt,version,pardim,
     *   parpos,xnode,line,rsetname,parname,splname,resname,oldname,
     *   mname)
 
c     Subroutine to read the hazard model specification for a certain
c     data structure.  The syntax is:
c
c     BINOMIAL MODEL; [DATA STRUCTURE = n;]
c        [{KEEP | DROP} IF ...];
c        OUTCOME = varname;
c        EXPOSURE = varname;
c        PROBABILITY = {LOGISTIC(...) | NORMPROB(...) | LINEAR(...)};
c
C     Version 2:
c     NEGATIVE BINOMIAL MODEL; [DATA STRUCTURE = n;]
c        [{KEEP | DROP} IF ...];
c        OUTCOME = varname;
c        EXPOSURE = varname;
c        DISPERSION = {EXP(...) | ...};
c        INCIDENCE = EXP(...);
c
c     For backward compatibility (version=1):
c     NEGATIVE BINOMIAL MODEL; [DATA STRUCTURE = n;]
c        [{KEEP | DROP} IF ...];
c        OUTCOME = varname;
c        SCALE = EXP(...);
c        PROBABILITY = {LOGISTIC(...) | NORMPROB(...) | LINEAR(...)};
c
c     POISSON MODEL; [DATA STRUCTURE = n;]
c        [{KEEP | DROP} IF ...];
c        OUTCOME = varname;
c        [EXPOSURE = ...;]
c        INCIDENCE = EXP(...);
c
c     Upon output,
c        modeltype = 51 for binomial with logistic probability
c                    52 for binomial with probit probability
c                    53 for binomial with untransformed probability
c                    54 for old-style negative binomial with logistic probabilit
c                    55 for old-style negative binomial with probit probability
c                    56 for old-style negative binomial with untransformed proba
c                    57 for negative binomial with log-link function (exponentia
c                    58 for Poisson
c
c     Constantijn Panis, 20 December 2002
c     Last change:  exposure in nb models
 
      implicit none
      integer levelmax,nfull,nummodel,maxmodel,datadim,
     *   strctre(maxmodel),datas(maxmodel),modeltype(maxmodel),
     *   minlnl(maxmodel),mtype,mdllvl(maxmodel),maxmeno,
     *   moel(maxmeno),meno,moelpntr(maxmodel),numrset,numpar,numspl,
     *   numdist,distdim(numdist),nold,prof,
     *   numfreq,maxnumfreq,nfreq(maxmodel),freqloc(maxnumfreq),
     *   nxold(levelmax,datadim),namepntr(levelmax,datadim),
     *   pardim(numpar),parpos(numpar),maxxnode,xnodepos,struc,
     *   ds,il,level,ipt,jpt,version
      real xnode(maxxnode),datastructure(datadim)
      logical smtxt
      character line*128,rsetname(numrset)*12,parname(numpar)*12,
     *   splname(numspl)*12,resname(*)*12,oldname(*)*8,mname*(*)
 
      nummodel=nummodel+1
      if(nummodel.gt.maxmodel)call bug(ipt,jpt)
c     To be refined below:
      modeltype(nummodel)=mtype
      moelpntr(nummodel)=meno+1
 
c     Record beginning position of summary stats; number follows below.
      if(numfreq.lt.maxnumfreq)then
         numfreq=numfreq+1
         nfreq(nummodel)=numfreq
         freqloc(numfreq)=-1
      else
         numfreq=maxnumfreq
         nfreq(nummodel)=0
      endif
 
c     [DATA STRUCTURE = n;]
      call getstruc(datadim,struc,ds,il,ipt,jpt,datas(nummodel),
     *   strctre(nummodel),datastructure,line)
 
c     [{KEEP | DROP} IF condition;]
      call getkeepdrop(levelmax,xnodepos,moel(moelpntr(nummodel)),nold,
     *   maxxnode,nxold(1,ds),namepntr(1,ds),numfreq,maxnumfreq,freqloc,
     *   ipt,jpt,il,struc,xnode,oldname,line)
 
c     [NUMERATOR;]  [DENOMINATOR;]
      call getnumdenom(minlnl(nummodel),line,il)
 
      if(.not.smtxt(4,line,il,'OUTCOME ='))goto 800
c     OUTCOME = varname;   (where varname may be an expression)
      meno=moelpntr(nummodel)+1
      moel(meno)=xnodepos+1
      call readexpr(1,levelmax,levelmax,mdllvl(nummodel),1,nold,
     *   maxxnode,xnodepos,xnode,nxold(1,ds),namepntr(1,ds),0,0,0,
     *   oldname,line,il,ipt,jpt,struc,';')
 
c     Careful with various combinations of mtype and version 1/2 syntax
 
      if(mtype.eq.51)then
c        Binomial:
         if(.not.smtxt(4,line,il,'EXPosure='))goto 800
         meno=meno+1
         moel(meno)=xnodepos+1
         call readexpr(1,levelmax,mdllvl(nummodel),level,1,nold,
     *      maxxnode,xnodepos,xnode,nxold(1,ds),namepntr(1,ds),numfreq,
     *      maxnumfreq,freqloc,oldname,line,il,ipt,jpt,struc,';')
      elseif(mtype.eq.57)then
c        Negative binomial: exposure optional
         meno=meno+1
         if(smtxt(4,line,il,'EXPosure='))then
            if(version.eq.1)goto 840
            if(smtxt(4,line,il,';'))then
               moel(meno)=-99
            else
               moel(meno)=xnodepos+1
               call readexpr(1,levelmax,mdllvl(nummodel),level,1,nold,
     *            maxxnode,xnodepos,xnode,nxold(1,ds),namepntr(1,ds),
     *            numfreq,maxnumfreq,freqloc,oldname,line,il,ipt,jpt,
     *            struc,';')
            endif
         else
            moel(meno)=-99
         endif
         meno=meno+1
         if(smtxt(4,line,il,'DISPersion='))then
            if(version.eq.1)goto 840
            if(smtxt(4,line,il,'EXP('))then
               moel(meno)=2
               call read_model(levelmax,struc,mdllvl(nummodel),nfull,
     *            numrset,numspl,numdist,numpar,distdim,maxmeno,moel,
     *            meno,maxxnode,xnodepos,numfreq,maxnumfreq,freqloc,
     *            nold,prof,nxold(1,ds),namepntr(1,ds),parpos,pardim,
     *            mtype,-1,-1,-1,xnode,rsetname,splname,parname,oldname,
     *            resname,'DISPERSION = EXP(',mname,');',line,il,
     *            ipt,jpt)
            else
               moel(meno)=1
               call read_model(levelmax,struc,mdllvl(nummodel),nfull,
     *            numrset,numspl,numdist,numpar,distdim,maxmeno,moel,
     *            meno,maxxnode,xnodepos,numfreq,maxnumfreq,freqloc,
     *            nold,prof,nxold(1,ds),namepntr(1,ds),parpos,pardim,
     *            mtype,-1,-1,-1,xnode,rsetname,splname,parname,oldname,
     *            resname,'DISPERSION = ',mname,';',line,il,ipt,jpt)
            endif
         elseif(smtxt(4,line,il,'SCALE='))then
c           Support version=1 syntax scale=exp(...).
            if(version.ge.2)goto 840
            moel(meno)=3
            if(.not.smtxt(4,line,il,'EXP('))goto 820
            call read_model(levelmax,struc,mdllvl(nummodel),nfull,
     *         numrset,numspl,numdist,numpar,distdim,maxmeno,moel,meno,
     *         maxxnode,xnodepos,numfreq,maxnumfreq,freqloc,nold,prof,
     *         nxold(1,ds),namepntr(1,ds),parpos,pardim,mtype,-1,-1,-1,
     *         xnode,rsetname,splname,parname,oldname,resname,
     *         'SCALE = EXP(',mname,');',line,il,ipt,jpt)
         else
            goto 800
         endif
      elseif(mtype.eq.58)then
c        Poisson: exposure optional
         meno=meno+1
         if(smtxt(4,line,il,'EXPosure='))then
            if(smtxt(4,line,il,';'))then
               moel(meno)=-99
            else
               moel(meno)=xnodepos+1
               call readexpr(1,levelmax,mdllvl(nummodel),level,1,nold,
     *            maxxnode,xnodepos,xnode,nxold(1,ds),namepntr(1,ds),
     *            numfreq,maxnumfreq,freqloc,oldname,line,il,ipt,jpt,
     *            struc,';')
            endif
         else
            moel(meno)=-99
         endif
      else
         call bug(ipt,jpt)
      endif
 
c     PROBABILITY = ...;
c     INCIDENCE = ...;
  100 if(mtype.eq.51.or.(version.eq.1.and.mtype.eq.57))then
         if(.not.smtxt(4,line,il,'PROBability='))goto 800
         if(smtxt(4,line,il,'LOGIsTic('))then
            if(mtype.eq.51)modeltype(nummodel)=51
            if(mtype.eq.57)modeltype(nummodel)=54
            call read_model(levelmax,struc,mdllvl(nummodel),nfull,
     *         numrset,numspl,numdist,numpar,distdim,maxmeno,moel,meno,
     *         maxxnode,xnodepos,numfreq,maxnumfreq,freqloc,nold,prof,
     *         nxold(1,ds),namepntr(1,ds),parpos,pardim,mtype,-1,99,-1,
     *         xnode,rsetname,splname,parname,oldname,resname,
     *         'PROB = LOGISTIC(',mname,');',line,il,ipt,jpt)
         elseif(smtxt(4,line,il,'NORMprob('))then
            if(mtype.eq.51)modeltype(nummodel)=52
            if(mtype.eq.57)modeltype(nummodel)=55
            call read_model(levelmax,struc,mdllvl(nummodel),nfull,
     *         numrset,numspl,numdist,numpar,distdim,maxmeno,moel,meno,
     *         maxxnode,xnodepos,numfreq,maxnumfreq,freqloc,nold,prof,
     *         nxold(1,ds),namepntr(1,ds),parpos,pardim,mtype,-1,99,-1,
     *         xnode,rsetname,splname,parname,oldname,resname,
     *         'PROB = NORMPROB(',mname,');',line,il,ipt,jpt)
         elseif(smtxt(4,line,il,'LINEAR('))then
c           No transformation.  Integrated residuals are not allowed.
            if(mtype.eq.51)modeltype(nummodel)=53
            if(mtype.eq.57)modeltype(nummodel)=56
            call read_model(levelmax,struc,mdllvl(nummodel),nfull,
     *         numrset,numspl,numdist,numpar,distdim,maxmeno,moel,meno,
     *         maxxnode,xnodepos,numfreq,maxnumfreq,freqloc,nold,prof,
     *         nxold(1,ds),namepntr(1,ds),parpos,pardim,mtype,-1,-1,-1,
     *         xnode,rsetname,splname,parname,oldname,resname,
     *         'PROB = LINEAR(',mname,');',line,il,ipt,jpt)
         else
            goto 800
         endif
      elseif((version.eq.2.and.mtype.eq.57).or.mtype.eq.58)then
         modeltype(nummodel)=mtype
         if(.not.smtxt(4,line,il,'INCidence='))goto 800
         if(.not.smtxt(4,line,il,'EXP('))goto 830
         call read_model(levelmax,struc,mdllvl(nummodel),nfull,
     *      numrset,numspl,numdist,numpar,distdim,maxmeno,moel,meno,
     *      maxxnode,xnodepos,numfreq,maxnumfreq,freqloc,nold,prof,
     *      nxold(1,ds),namepntr(1,ds),parpos,pardim,mtype,-1,99,-1,
     *      xnode,rsetname,splname,parname,oldname,resname,
     *      'INCIDENCE = EXP(',mname,');',line,il,ipt,jpt)
      endif
 
c     Frequency tables will be computed for selected variables.  Array
c     nfreq stores at which position in freqloc the locations of data
c     structure m start and how many such variables there are.
      numfreq=min(numfreq,maxnumfreq)
      if(nfreq(nummodel).gt.0)nfreq(nummodel) = nfreq(nummodel) +
     *   (numfreq-nfreq(nummodel)+1)*(maxnumfreq+1)
      goto 999
 
  800 if(ipt.gt.0)then
         call caretline(8,line,il)
         if(mtype.eq.51)write(8,801)
         if(version.eq.1.and.mtype.eq.57)write(8,802)
         if(version.ge.2.and.mtype.eq.57)write(8,803)
         if(mtype.eq.58)write(8,804)
      endif
      if(jpt.gt.0)then
         call caretline(6,line,il)
         if(mtype.eq.51)write(6,801)
         if(version.eq.1.and.mtype.eq.57)write(6,802)
         if(version.ge.2.and.mtype.eq.57)write(6,803)
         if(mtype.eq.58)write(6,804)
      endif
  801 format('ERROR in a binomial model specification.  ',
     *   'The syntax is:',//,'BINOMIAL MODEL;',/,
     *   3x,'[DATA STRUCTURE = n;]',/,3x,'[{KEEP|DROP} IF ...;]',/,
     *   3x,'OUTCOME = ...;',/,3x,'EXPOSURE = ...;',/,3x,
     *   'PROBABILITY = {LOGISTIC(...) | NORMPROB(...) | LINEAR(...)};')
  802 format('ERROR in a negative binomial model specification.  ',
     *   'The version 1 syntax',/,'is:',//,
     *   'NEGATIVE BINOMIAL MODEL;',/,
     *   3x,'[DATA STRUCTURE = n;]',/,3x,'[{KEEP|DROP} IF ...;]',/,
     *   3x,'OUTCOME = ...;',/,3x,'SCALE = EXP(...);',/,3x,
     *   'PROBABILITY = {LOGISTIC(...) | NORMPROB(...) | LINEAR(...)};',
     *   //,'The above syntax is obsolete and only applies in ',
     *   'combination with',/,'"option version=1".  ',
     *   'See the manual for details.')
  803 format('ERROR in a negative binomial model specification.  ',
     *   'The syntax is:',//,'NEGATIVE BINOMIAL MODEL;',/,
     *   3x,'[DATA STRUCTURE = n;]',/,3x,'[{KEEP|DROP} IF ...;]',/,
     *   3x,'OUTCOME = ...;',/,3x,'[EXPOSURE = ...;]',/,3x,
     *   3x,'DISPERSION = {EXP(...) | ...};',/,
     *   3x,'INCIDENCE = EXP(...);')
  804 format('ERROR in a Poisson model specification.  ',
     *   'The syntax is:',//,'POISSON MODEL;',/,
     *   3x,'[DATA STRUCTURE = n;]',/,3x,'[{KEEP|DROP} IF ...;]',/,
     *   3x,'OUTCOME = ...;',/,3x,'[EXPOSURE = ...;]',/,3x,
     *   'INCIDENCE = EXP(...);')
      stop
  810 if(ipt.gt.0)then
         call caretline(8,line,il)
         call writeaia(8,72,1,'ERROR: structure ',struc,
     *      ' not found in the data.')
      endif
      if(jpt.gt.0)then
         call caretline(6,line,il)
         call writeaia(6,72,1,'ERROR: structure ',struc,
     *      ' not found in the data.')
      endif
      stop
  820 if(ipt.gt.0)then
         call caretline(8,line,il)
         write(8,821)
      endif
      if(jpt.gt.0)then
         call caretline(6,line,il)
         write(6,821)
      endif
  821 format('ERROR: Scale specifications of (old-style) negative ',
     *   'binomial models must',/,'be exponential transformations of ',
     *   'permissible building blocks.',/,'The syntax is:',//,
     *   3x,'SCALE = EXP(...);')
      stop
  830 if(ipt.gt.0)then
         call caretline(8,line,il)
         write(8,831)
      endif
      if(jpt.gt.0)then
         call caretline(6,line,il)
         write(6,831)
      endif
  831 format('ERROR: Incidence specifications of Poisson and negative ',
     *   'binomial models',/,'must be exponential transformations of ',
     *   'permissible building blocks.',/,'The syntax is:',//,
     *   3x,'INCIDENCE = EXP(...);')
      stop
  840 if(ipt.gt.0)then
         call caretline(8,line,il)
         if(version.eq.1)write(8,841)
         if(version.ge.2)write(8,842)
      endif
      if(jpt.gt.0)then
         call caretline(6,line,il)
         if(version.eq.1)write(6,841)
         if(version.ge.2)write(6,842)
      endif
  841 format('ERROR in a negative binomial model specification.  In ',
     *   'combination with',/,'"option version=1", the scale ',
     *   '(exposure) needs to be specified as',/,'"SCALE = ',
     *   'EXP(...);".  See the manual for details.')
  842 format('ERROR in a negative binomial model specification.  ',
     *   'Your scale',/,'specification follows the syntax of aML ',
     *   'Version 1.  To continue using',/,'Version 1 syntax and ',
     *   'algorithms, specify "option version=1;" in your',/,
     *   'control file.  See the manual for details on current syntax ',
     *   'and',/,'algorithms.')
      stop
  999 return
      end
