      subroutine read_res(levelmax,maxmeno,meno,moel,numdist,distdim,
     *   struc,nxold,namepntr,maxxnode,xnodepos,nold,mdllvl,modeltype,
     *   numterm,maxnumfreq,numfreq,freqloc,xnode,resdone,intresdone,
     *   prof,oldname,resname,integrated,line,il,ipt,jpt)
 
c     Subroutine to read in a (integrated or non-integrated) residual
c     of any model specification.
c
c     The syntax for non-integrated residuals is:
c        RES(DRAW=varname, {REF=resname | REFVAR=varname})
c     and for integrated residuals:
c        INTRES(DRAW=varname, {REF=resname | REFVAR=varname})
c
c     where resname is the name of a dimension of a distribution (that is
c     not integrated out) and varname is the name of a variable.  Residuals
c     with the same value for (the first) varname will be correlated.
c
c     Independent draws may be achieved as follows:
c        RES(DRAW=_iid, {REF=resname | REFVAR=varname})
c
c     Independent draws at, say, level 3 may be achieved as follows:
c        RES(DRAW=_level3, {REF=resname | REFVAR=varname})
c
c     Instead of resname, N(0,1) is understood to be the standard normal
c     distribution, which the user need not define.  N(0,1) may only
c     be used in combination with independent draws at the outcome level,
c     denoted by draw=_idd.
c     They may not be interacted with anything.  They may not be used
c     in combination with other, explicitly-referenced non-integrated
c     residuals.  Only one IID N(0,1) residual may appear in any one
c     model specification.  Integer resdone stores information on
c     the type of non-integrated residuals specified:
c        resdone=-1  [in] no non-integrated residual allowed
c                 0   no non-integrated residual specified yet
c                 1   one or more explicit non-integrated residuals specified
c                 2   implicit IID N(0,1) was explicitly specified
c                99  [in] non-integrated residuals OK; do not change
c                     this value (saves variables)
c
c     The same exists for intresdone, except that value 2 is irrelevant.
c
c     Reference information is stored in moel.
c        moel(1) = 4  This is a integrated residual;
c                  7  this is a non-integrated residual
c        moel(2) = 1  draw is given by expression
c                  2  draw is _leveln
c                  3  draw is _iid
c                  4  draw is _iid, residual is N(0,1) (see read_model4)
c        moel(3) = position in xnode where expression starts if moel(2)=1
c                  level if moel(2)=2
c                  n/a if moel(2)=3
c        moel(4) = 1  residual is directly referenced
c                  2  residual is indirectly referenced
c        moel(5) = distribution rank number if moel(4)=1
c                  expression position in xnode if moel(4)=2
c        moel(6) = dimension rank number if moel(4)=1
c
c     Constantijn Panis, 12 September 1998
c     Last change: draw=_iid truly independent
 
      implicit none
      integer levelmax,maxmeno,meno,moel(maxmeno),numdist,
     *   distdim(numdist),struc,nxold(levelmax),namepntr(levelmax),
     *   maxxnode,xnodepos,nold,mdllvl,modeltype,numterm,maxnumfreq,
     *   numfreq,freqloc(maxnumfreq),integrated,resdone,intresdone,
     *   prof,il,ipt,jpt,level,varpos,i,j,ref,i1,i2,status,trm
      real xnode(maxxnode)
      character line*128,oldname(*)*8,resname(*)*12,string*60,
     *   idstring*60
      logical smtxt
 
      if(meno+5.gt.maxmeno)call
     *   maxexceed('maxmeno',meno+5,maxmeno,ipt,jpt)
 
c     This is a residual:
      meno=meno+1
      if(integrated.eq.1)then
         moel(meno)=4
         if(intresdone.eq.-1)goto 820
         if(intresdone.ne.99)intresdone=1
      else
         moel(meno)=7
         if(resdone.eq.-1)goto 820
      endif
 
      if(.not.smtxt(4,line,il,'('))goto 800
      if(.not.smtxt(4,line,il,'DRAW='))goto 800
 
      if(smtxt(4,line,il,'_LEVEL'))then
         if(prof.lt.4)goto 850
         call getint(4,level,line,il,status)
         if(status.gt.0)goto 810
         if(level.gt.mdllvl)goto 830
         varpos=0
         moel(meno+1)=2
         moel(meno+2)=level
      elseif(smtxt(4,line,il,'_IID'))then
         level=mdllvl
         varpos=0
         moel(meno+1)=3
c        Following is irrelevant, as _iid does not depend on level.
         moel(meno+2)=level
      else
         moel(meno+1)=1
         moel(meno+2)=xnodepos+1
         call readexpr(1,levelmax,mdllvl,level,1,nold,maxxnode,
     *      xnodepos,xnode,nxold,namepntr,numfreq,maxnumfreq,freqloc,
     *      oldname,line,il,ipt,jpt,struc,',')
      endif
 
      if(smtxt(4,line,il,','))continue
 
      if(smtxt(4,line,il,'REFVAR='))then
c        Indirectly referenced:
         if(integrated.eq.0.and.resdone.ne.99)then
c           Implicit and explicit residuals may not be mixed:
            if(resdone.eq.2)goto 840
            resdone=1
         endif
         moel(meno+3)=2
         moel(meno+4)=xnodepos+1
         meno=meno+4
         call readexpr(1,levelmax,mdllvl,level,1,nold,maxxnode,
     *      xnodepos,xnode,nxold,namepntr,numfreq,maxnumfreq,freqloc,
     *      oldname,line,il,ipt,jpt,struc,')')
      elseif(smtxt(4,line,il,'REFerence='))then
c        Directly referenced:
         if(smtxt(4,line,il,'N(0,1)'))then
c           IID N(0,1) residual.  Only allowed in probits:
            if(modeltype.ne.43.and.modeltype.ne.44)goto 840
c           Cannot be integrated out:
            if(integrated.eq.1)goto 800
c           Must not be interacted with anything.
            if(numterm.ne.0)goto 840
c           Cannot be mixed with explicitly defined residuals, and only
c           one N(0,1) may be specified for any one model.
            if(resdone.ne.99)then
               if(resdone.ne.0)goto 840
               resdone=2
            endif
c           Only allowed with draw=_iid (but recode to imply N(0,1))
            if(moel(meno+1).ne.3)goto 800
            moel(meno+1)=4
            meno=meno+1
            if(.not.smtxt(4,line,il,')'))goto 800
            goto 999
         else
            if(integrated.eq.0.and.resdone.ne.99)then
c              Implicit and explicit residuals may not be mixed:
               if(resdone.eq.2)goto 840
               resdone=1
            endif
            call getword(4,line,il,i1,i2,status)
            if(status.gt.0)goto 800
            if(.not.smtxt(4,line,il,')'))goto 800
            ref=0
            do 100 i=1,numdist
               do 110 j=1,distdim(i)
                  ref=ref+1
                  if(line(i1:i2).eq.resname(ref))then
                     moel(meno+3)=1
                     moel(meno+4)=i
                     moel(meno+5)=j
                     meno=meno+5
                     goto 999
                  endif
  110          continue
  100       continue
            goto 800
         endif
      else
         goto 800
      endif
      goto 999
 
  800 string=idstring(struc,0,0,0,0)
      if(ipt.gt.0)then
         call caretline(8,line,il)
         call wrapstring(8,0,72,0,1,'ERROR in the syntax of a '//
     *      'residual specification in a model specification'//
     *      string(1:trm(string))//'  The syntax is:')
         if(integrated.eq.1)write(8,801)
         if(integrated.eq.0)write(8,802)
         write(8,803)
      endif
      if(jpt.gt.0)then
         call caretline(6,line,il)
         call wrapstring(6,0,72,0,1,'ERROR in the syntax of a '//
     *      'residual specification in a model specification'//
     *      string(1:trm(string))//'  The syntax is:')
         if(integrated.eq.1)write(6,801)
         if(integrated.eq.0)write(6,802)
         write(6,803)
      endif
  801 format(3x,'INTRES(DRAW=varname, {REF=resname | REFVAR=varname})')
  802 format(3x,'RES(DRAW=varname, {REF=resname | REFVAR=varname})')
  803 format('where the first varname is the name of a variable ',
     *   'indicating which',/,'residual draws of this distribution ',
     *   'are correlated; resname is the name',/,'of a dimension of ',
     *   'a distribution; and the second (indirect reference)',/,
     *   'varname is a variable name or integer expression.  Note ',
     *   'that independent',/,'draws may be specified by (DRAW=_iid, ',
     *   '...).')
      stop
  810 string=idstring(struc,0,0,0,0)
      if(ipt.gt.0)then
         call caretline(8,line,il)
         call writeaia(8,72,1,'ERROR in a residual specification of '//
     *      'a model specification'//string(1:trm(string))//
     *      '  The syntax for independent draws at, say, level ',mdllvl,
     *      ', is:')
         call writeaia(8,72,1,'   RESIDUAL(DRAW=_level',mdllvl,
     *      ', {REF=resname | REFVAR=varname})')
         write(8,811)
      endif
      if(jpt.gt.0)then
         call caretline(6,line,il)
         call writeaia(6,72,1,'ERROR in a residual specification of '//
     *      'a model specification'//string(1:trm(string))//
     *      '  The syntax for independent draws at, say, level ',mdllvl,
     *      ', is:')
         call writeaia(8,72,1,'   RESIDUAL(DRAW=_level',mdllvl,
     *      ', {REF=resname | REFVAR=varname})')
         write(6,811)
      endif
  811 format('Note that independent draws may be specified as ',
     *   '(DRAW=_iid, ...).')
      stop
  820 if(ipt.gt.0)then
         call caretline(8,line,il)
         if(integrated.eq.0)write(8,821)
         if(integrated.eq.1)write(8,822)
      endif
      if(jpt.gt.0)then
         call caretline(6,line,il)
         if(integrated.eq.0)write(6,821)
         if(integrated.eq.1)write(6,822)
      endif
  821 format(/,'ERROR:  no non-integrated residual allowed here.')
  822 format(/,'ERROR:  no integrated residual allowed here.')
      stop
  830 if(ipt.gt.0)then
         call caretline(8,line,il)
         if(varpos.eq.0)then
            call writea2ia(8,72,1,'ERROR:  ''_level',level,''' may '//
     *         'not be used here to specify residual draws, because '//
     *         'the outcome is at level ',mdllvl,'.')
         else
            call writea2ia(8,72,1,'ERROR:  level ',level,
     *         ' variable '''//line(i1:i2)//''' may not be used to '//
     *         'specify residual draws here, because the outcome is '//
     *         'at level ',mdllvl,'.')
         endif
      endif
      if(jpt.gt.0)then
         call caretline(6,line,il)
         if(varpos.eq.0)then
            call writea2ia(6,72,1,'ERROR:  ''_level',level,''' may '//
     *         'not be used here to specify residual draws, because '//
     *         'the outcome is at level ',mdllvl,'.')
         else
            call writea2ia(6,72,1,'ERROR:  level ',level,
     *         ' variable '''//line(i1:i2)//''' may not be used to '//
     *         'specify residual draws here, because the outcome is '//
     *         'at level ',mdllvl,'.')
         endif
      endif
      stop
  840 string=idstring(struc,0,0,0,0)
      if(ipt.gt.0)then
         call caretline(8,line,il)
         call wrapstring(8,0,72,0,1,'ERROR in the syntax of a '//
     *      'residual specification in a model specification'//
     *      string(1:trm(string)))
         write(8,841)
      endif
      if(jpt.gt.0)then
         call caretline(6,line,il)
         call wrapstring(6,0,72,0,1,'ERROR in the syntax of a '//
     *      'residual specification in a model specification'//
     *      string(1:trm(string)))
         write(6,841)
      endif
  841 format('The following restrictions apply to the use of ',
     *   'implicit N(0,1)',/,'residuals:',/,'(1) implicit N(0,1) ',
     *   'residuals may only be used in probit models.',/,
     *   '(2) implicit N(0,1) residuals may not be interacted.',/,
     *   '(3) implicit N(0,1) residuals may not be combined with ',
     *   'other,',/,'    explicitly-defined non-integrated residuals.',
     *   /,'(4) only one implicit N(0,1) residual may appear in any ',
     *   'one model',/,'    statement.',/,'(5) implicit N(0,1) ',
     *   'residuals must be drawn IID at the same level as the',/,4x,
     *   'outcome, i.e., res(draw=_iid, ref=N(0,1))')
      stop
  850 il=il-6
      if(ipt.gt.0)then
         call caretline(8,line,il)
         write(8,851)line(il:il+6)
      endif
      if(jpt.gt.0)then
         call caretline(6,line,il)
         write(6,851)line(il:il+6)
      endif
  851 format('ERROR: the specification of residual draws with ',a,
     *   ' should only be',/,'done by extraordinarily experienced ',
     *   'aML users.  If you feel this applies',/,'to you, specify ',
     *   '"option proficiency = guru;" in the control file.')
      stop
  999 return
      end
 
