      subroutine read_seed(ngrid,nfull,grid,fullname,gridpos,iiffull,
     *   fullbeta,fulltype,unit,ipt,jpt,line,il)
 
c     Subroutine to read starting values.  Up to eight T/F's may be
c     specified, for up to eight rounds of estimation.
c
c     Constantijn Panis, 4 April 2003
c     Last change: initialize igrid
 
      implicit none
      integer unit,ngrid,nfull,gridpos(ngrid,2),iiffull(nfull),
     *   fulltype(nfull),ipt,jpt,il,i1,i2,i,j,nround,status,igrid,trm
      double precision grid(ngrid,2),fullbeta(nfull)
      logical intallzero,smtxt,isna
      character line*128,fullname(nfull)*8
 
      status=0
      igrid=0
 
      do 100 i=1,nfull
c        Read parameter name:
         call getstring(unit,line,il,i1,i2)
         if(i2.eq.-1.or.i2-i1.gt.7)goto 810
         if(line(i1:i2).eq.';')goto 800
c        Read T/F:
         fullname(i)=line(i1:i2)
         call getstring(unit,line,il,i1,i2)
         if(i2.eq.-1)goto 810
         if(i.eq.1)nround=i2-i1+1
         if(i.gt.1.and.nround.ne.i2-i1+1)goto 840
c        Translate, e.g., FFT into binary 100; FTT into 110; et cetera
         iiffull(i)=0
         do 110 j=i1,i2
            if(line(j:j).eq.'T'.or.line(j:j).eq.'t')then
               iiffull(i)=iiffull(i)+2**(j-i1)
            elseif(line(j:j).ne.'F'.and.line(j:j).ne.'f')then
               goto 840
            endif
  110    continue
c        Read starting value:
         if(smtxt(unit,line,il,'GRID('))then
c           Syntax:   parname {T|F}  grid(min,max,num)
c              grid(.,1) = minimum value
c              grid(.,2) = maximum value
c              gridpos(.,1) = position in fullbeta
c              gridpos(.,2) = number of points from minimum to maximum
            igrid=igrid+1
            if(igrid.gt.ngrid)call bug(ipt,jpt)
            grid(igrid,1)=0
            gridpos(igrid,1)=i
            call getdp(unit,grid(igrid,1),line,il,status)
            if(status.ne.0)goto 810
            call getdp(unit,grid(igrid,2),line,il,status)
            if(status.ne.0)goto 810
            call getint(unit,gridpos(igrid,2),line,il,status)
            if(status.ne.0)goto 810
            if(gridpos(igrid,2).le.1)goto 810
            if(.not.smtxt(unit,line,il,')'))goto 810
            if(grid(igrid,1).ge.grid(igrid,2))goto 810
            fullbeta(i)=grid(igrid,1)
         elseif(smtxt(unit,line,il,'AUTOmatic'))then
            if(isna(fullbeta(i)))goto 860
         else
            call getdp(unit,fullbeta(i),line,il,status)
            if(status.gt.0)goto 810
         endif
  100 continue
 
c     Check that number of grid variables is correct
      if(igrid.ne.ngrid)call bug(ipt,jpt)
 
c     Check that a semicolon follows.
      if(.not.smtxt(unit,line,il,';'))goto 820
 
c     Check domain restrictions:
      call chkrange(nfull,iiffull,fulltype,nround,fullbeta,
     *   fullname,ipt,jpt,status)
      if(status.ne.0)stop
 
c     Make sure that there is at least one free parameter:
      if(intallzero(iiffull,nfull))goto 850
      goto 999
 
  800 i=i-1
      if(ipt.gt.0)call writea2ia(8,72,1,'ERROR:  ',nfull,' starting '//
     *   'values were expected; you only specified ',i,'.')
      if(jpt.gt.0)call writea2ia(6,72,1,'ERROR:  ',nfull,' starting '//
     *   'values were expected; you only specified ',i,'.')
      stop
  810 if(ipt.gt.0)then
         call caretline(8,line,il)
         write(8,811)
      endif
      if(jpt.gt.0)then
         call caretline(6,line,il)
         write(6,811)
      endif
  811 format('ERROR encountered while attempting to read a starting ',
     *   'value.',/,'The syntax is:',//,3x,'name  {T|F}  value',//,
     *   'where T is specified if the parameter is to be estimated ',
     *   'and F if not.',/,'Multiple T''s and F''s may be specified ',
     *   '(without spaces) if multiple',/,'rounds of estimation are ',
     *   'desired. Names may contain up to eight characters.',/,
     *   'Instead of a value, you may specify grid(min,max,num), ',
     *   'which will',/,'result in a grid search of num points ',
     *   'from min to max,',/,'followed by normal optimization.')
      stop
  820 read(unit,'(a)',err=830,end=830)line
      if(ipt.gt.0)call writeaia(8,72,1,'ERROR: ',nfull,' starting '//
     *   'values were expected to be read; you specified too many.')
      if(jpt.gt.0)call writeaia(6,72,1,'ERROR: ',nfull,' starting '//
     *   'values were expected to be read; you specified too many.')
      stop
  830 if(ipt.gt.0)write(8,831)
      if(jpt.gt.0)write(6,831)
  831 format(/,'ERROR encountered while reading the starting values.  ',
     *   'Did you forget',/,'to close with a semicolon?')
      stop
  840 il=i1
      if(ipt.gt.0)then
         call caretline(8,line,il)
         if(nround.eq.1)write(8,841)
         if(nround.gt.1)write(8,842)nround
      endif
      if(jpt.gt.0)then
         call caretline(6,line,il)
         if(nround.eq.1)write(6,841)
         if(nround.gt.1)write(6,842)nround
      endif
  841 format(/,'ERROR: ''T'' or ''F'' expected.')
  842 format(/,'ERROR: the T/F string for the first starting value ',
     *  'indicates that there',/,'will be',i2,' estimation rounds.  ',
     *   'All other starting values must then',/,'provide T/F''s for ',
     *   'the same number of rounds.')
      stop
  850 if(ipt.gt.0)write(8,851)
      if(jpt.gt.0)write(6,851)
  851 format(/,'ERROR: all parameters are fixed, none is estimated.')
      stop
  860 if(ipt.gt.0)write(8,861)fullname(i)(1:trm(fullname(i))),
     *   fullname(i)(1:trm(fullname(i)))
      if(jpt.gt.0)write(6,861)fullname(i)(1:trm(fullname(i))),
     *   fullname(i)(1:trm(fullname(i)))
  861 format(/,'ERROR:  unable to automatically generate a starting ',
     *   'value for ''',a,'''.',/,'(If ''',a,''' relates to a vector ',
     *   'for a finite mixture distribution,',/,
     *   'specify "INITIAL = ..." in the vector definition.)')
      stop
  999 return
      end
