      subroutine read_tau(levelmax,nfull,maxmeno,moel,meno,moelpntr,
     *   modeltype,nxold,namepntr,nold,mdllvl,numpar,numfreq,maxnumfreq,
     *   freqloc,xnodepos,maxxnode,pardim,fulltype,parpos,xnode,il,
     *   struc,oldname,parname,mname,line,ipt,jpt)
 
c     Subroutine to read THRESHOLD specifications in probit/logit models.
c     Thresholds are required in ordered probit/logit models; in
c     simple probit/logit models the default is one threshold, fixed
c     at zero.  The syntax is:
c
c        THRESHOLDS = vectorname;
c        THRESHOLDS = (REFVAR=.....);
c
c     where vectorname is the name of a vector, and the REFVAR
c     construction allows for indirect referencing of a vector.
c
c     In simple probits/logits, the syntax is:
c
c        THRESHOLD = parname;
c        THRESHOLD = (REFVAR=.....);
c
c     Constantijn Panis,  6 April 1998
 
      implicit none
      integer levelmax,nfull,maxmeno,moel(maxmeno),meno,moelpntr,
     *   modeltype,nxold(levelmax),namepntr(levelmax),nold,mdllvl,
     *   numpar,numfreq,maxnumfreq,freqloc(maxnumfreq),xnodepos,
     *   pardim(numpar),fulltype(nfull),parpos(numpar),maxxnode,
     *   ref,il,struc,ipt,jpt,i1,i2,level,status,trm
      real xnode(maxxnode)
      character oldname(*)*8,parname(numpar)*12,line*128,
     *   mname*(*),tmp*32
      logical paren,smtxt
 
      if(moelpntr+4.gt.maxmeno)call
     *   maxexceed('maxmeno',moelpntr+4,maxmeno,ipt,jpt)
 
      if(smtxt(4,line,il,'THRESHOLDs ='))then
         if(smtxt(4,line,il,';'))then
c           THRESHOLD = ;   /* default applies, but
c           only for simple logit/probit models  */
            if(modeltype.ne.41.and.modeltype.ne.43)goto 810
            moel(moelpntr+1)=-64
         else
c           Thresholds are explicitly specified:
            if(modeltype.eq.41.or.modeltype.eq.43)then
               moel(moelpntr+1)=-63
            else
               moel(moelpntr+1)=-61
            endif
         endif
      else
c        No threshold; default applies.
         if(modeltype.ne.41.and.modeltype.ne.43)goto 810
         moel(moelpntr+1)=-64
      endif
 
c     Update meno to after outcome position in xnode
      meno=moelpntr+2
      if(moel(moelpntr+1).eq.-64)goto 999
 
c     Thresholds given by parameter or vector
      if(smtxt(4,line,il,'(REFVAR='))then
c        Indirectly referenced
         moel(meno+1)=3
         moel(meno+2)=xnodepos+1
         meno=meno+2
         call readexpr(1,levelmax,mdllvl,level,1,nold,maxxnode,
     *      xnodepos,xnode,nxold,namepntr,numfreq,maxnumfreq,freqloc,
     *      oldname,line,il,ipt,jpt,struc,')')
      else
c        Directly referenced:
         paren=smtxt(4,line,il,'(REFerence=')
c        Regular parameter, vector, or matrix element.
         call getword(4,line,il,i1,i2,status)
         if(status.gt.0)goto 800
         call char12match(line(i1:i2),parname,numpar,ref)
         if(ref.eq.0)goto 800
c        If the parameter is a parameter or vector, just record which
c        one it is.  If a matrix is referenced, read in which element
c        and record the location in fullbeta.
         if(abs(pardim(ref)).gt.nfull)goto 820
         if(fulltype(parpos(ref)).ne.41.and.pardim(ref).gt.1)goto 830
         moel(meno+1)=1
         moel(meno+2)=ref
         meno=meno+2
         if(paren)then
            if(.not.smtxt(4,line,il,')'))goto 800
         endif
      endif
      if(.not.smtxt(4,line,il,';'))goto 800
      goto 999
 
  800 if(ipt.gt.0)then
         call caretline(8,line,il)
         write(8,801)
      endif
      if(jpt.gt.0)then
         call caretline(6,line,il)
         write(6,801)
      endif
  801 format('ERROR in the syntax of a threshold(s) statement.  ',
     *   'The syntax is:',/,6x,'THRESHOLD = {parname | vectorname}',/,
     *   'or, in case the theshold vector or parameter is indirectly ',
     *   'referenced:',/,6x,'THRESHOLD = (REFVAR = varname)',/,
     *   'where varname may also be an integer expression.')
      stop
  810 tmp=mname
      if(ipt.gt.0)then
         call caretline(8,line,il)
         write(8,811)tmp(1:trm(tmp))
      endif
      if(jpt.gt.0)then
         call caretline(6,line,il)
         write(6,811)tmp(1:trm(tmp))
      endif
  811 format('ERROR:  ',a,' models must contain a THRESHOLD ',
     *   'specification.',/,'(If thresholds are given by data ',
     *   'variables, use the THRESHOLD VARS',/,'specification.)')
      stop
  820 il=i1
      if(ipt.gt.0)then
         call caretline(8,line,il)
         write(8,821)
      endif
      if(jpt.gt.0)then
         call caretline(6,line,il)
         write(6,821)
      endif
  821 format('ERROR:  parameter or vector expected here, not a ',
     *   'matrix element.')
      stop
  830 tmp=mname
      il=il-(i2-i1+1)
      if(ipt.gt.0)then
         call caretline(8,line,il)
         write(8,831)tmp(1:trm(tmp))
      endif
      if(jpt.gt.0)then
         call caretline(6,line,il)
         write(6,831)tmp(1:trm(tmp))
      endif
  831 format('ERROR:  thresholds vectors in ',a,' models must be ',
     *   'strictly increasing.',/,'Specify option "INCREASING;" ',
     *   'in the vector definition.')
      stop
  999 return
      end
