      subroutine readrefs(maxref,beginref,refnums,nref,line,il,ipt,jpt)
 
c     Subroutine to read in reference numbers.  The "ref =" has
c     already been read.  This routine reads in strictly positive
c     integers until ";" and stores them in refnums(beginref) and beyond.
c     There may only be maxref-beginref-1 numbers (plus a terminating zero),
c     otherwise the allocated space is exceeded.
c
c     Constantijn Panis, 1 February 2000
 
      implicit none
      integer maxref,beginref,refnums(maxref),nref,il,ipt,jpt,status,i
      character line*128
      logical dupint
 
c     Allow only maxref-beginref-1 to leave room for the terminating zero.
      call getints(4,refnums(beginref),maxref-beginref-1,nref,
     *   line,il,status)
      if(status.gt.0)goto 800
 
c     Check that the numbers are strictly positive and unique
      do 100 i=beginref,beginref+nref-1
         if(refnums(i).le.0)goto 810
         if(dupint(refnums(i),refnums,i-1))goto 820
  100 continue
 
c     Terminate reference numbers by a zero (even if there are none):
      refnums(beginref+nref)=0
      beginref=beginref+nref+1
      goto 999
 
c     Control may be passed here if the number of reference numbers
c     is too large or if a non-integer is encountered.
  800 if(nref.gt.maxref-beginref)call maxexceed('maxref',
     *   beginref+nref+1,maxref,ipt,jpt)
      if(ipt.gt.0)then
         call caretline(8,line,il)
         write(8,801)
      endif
      if(jpt.gt.0)then
         call caretline(6,line,il)
         write(6,801)
      endif
  801 format('ERROR in reference numbers.  Did you forget a semicolon?')
      stop
  810 if(ipt.gt.0)then
         call caretline(8,line,il)
         write(8,811)
      endif
      if(jpt.gt.0)then
         call caretline(6,line,il)
         write(6,811)
      endif
  811 format('ERROR: reference numbers must be strictly positive.',/,
     *   '(NOTE: you need not assign reference numbers to any ',
     *   'definition.',/,'A name is enough.)')
      stop
  820 if(ipt.gt.0)then
         call caretline(8,line,il)
         write(8,821)
      endif
      if(jpt.gt.0)then
         call caretline(6,line,il)
         write(6,821)
      endif
  821 format('ERROR: you assigned duplicate reference numbers to ',
     *   'building blocks',/,'of the same type.')
      stop
  999 return
      end
