      subroutine sqz_covar(task,dim1,dim2,covar1,covar2,
     *   dcovar1,dcovar2,dimused)
 
c     Subroutine to reduce the size of a covariance matrix such that
c     irrelevant dimensions are omitted, and to map derivatives with
c     respect to a reduced size matrix into the uncompressed matrix.
c
c     Arguments:
c     task    [in] = 1: reduce the size of a covariance matrix such that
c                      dimensions that are irrelevant are omitted.
c                  = 2: map derivatives wrt the squeezed covariance
c                      matrix into the full covariance matrix.
c     dim1    [in]  dimension of uncompressed matrix
c     dim2    [in]  dimension of compressed matrix
c     covar1  [in]  uncompressed matrix
c     covar2  [out] compressed matrix
c     dcovar1 [out] derivatives wrt uncompressed matrix
c     dcovar2 [in]  derivatives wrt compressed matrix
c     dimused [in]  integer array that tells which dimensions are used (1)
c                   and which not (0).
c
c     Note that covar1 and covar2 may share the same storage, as may
c     dcovar1 and dcovar2.
c     No advantage is taken of the fact that covar1 and covar2 are
c     symmetric.  Note that there may be cases in which dcovar1 and
c     dcovar2 are nonsymmetric.
c
c     Constantijn Panis, 2 August 1996
 
      implicit none
      integer task,dim1,dim2,dimused(dim1),i1,i2,j1,j2
      double precision covar1(dim1,dim1),covar2(dim2,dim2),
     *   dcovar1(dim1,dim1),dcovar2(dim2,dim2)
 
c     Squeeze the covariance matrix or unsqueeze derivatives?
      if(task.eq.2)goto 200
 
c     Squeeze the covariance matrix
      j2=0
      do 110 j1=1,dim1
         if(dimused(j1).eq.0)goto 110
         j2=j2+1
         i2=0
         do 120 i1=1,dim1
            if(dimused(i1).eq.0)goto 120
            i2=i2+1
            covar2(i2,j2)=covar1(i1,j1)
  120    continue
  110 continue
      goto 999
 
c     Unsqueeze the derivatives and set non-used dimensions to zero
  200 j2=dim2+1
      do 210 j1=dim1,1,-1
         if(dimused(j1).eq.0)goto 210
         j2=j2-1
         i2=dim2+1
         do 220 i1=dim1,1,-1
            if(dimused(i1).eq.0)goto 220
            i2=i2-1
            dcovar1(i1,j1)=dcovar2(i2,j2)
  220    continue
  210 continue
      do 230 i1=1,dim1
         do 240 j1=1,dim1
            if(dimused(i1).eq.1.and.dimused(j1).eq.1)goto 240
            dcovar1(i1,j1)=0.d0
  240    continue
  230 continue
 
  999 return
      end
