      subroutine sumlnl2l(nest,thisnest,numdist,sumintdim,sumdim2,
     *   sumbigintdim,neh,nel,nep,nec,neb,nen,nepo,neml,nneml,nfullplus,
     *   dimlam,dimii,allint6,maxpnts,intpnts,intpntr,nested,lampos,
     *   lamtyp,indx,dimused,thisdraw,distyp,distpos,posi1,posi2,
     *   bigposi1,intdist,intdim,intdimc,like,lnl,z,wt,weight,fullbeta,
     *   dfullbeta,ddfullbeta,ui,ddui,cmui,dcmui,ddcmui,lam,dlam,ddlam,
     *   ddepsh,ddepsl,ddepsp,ddepsc,ddepsb,ddepsn,ddepspo,ddepsml,dxbh,
     *   ddxbh,dxbc,ddxbc,cdm,ddcdm,siga6a6i,dsiga6a6,ddsiga6a6,
     *   siga6a6ia6,sigl4l6,dsigl4l6,ddsigl4l6,sigl4l6a6a6i,dtau,ddtau,
     *   dxblp,ddxblp,dsiga4a4,ddsiga4a4,da5,dda5,dp5,ddp5,dxbml,ddxbml,
     *   dsigii,ddsigii,ifdrv,iflam,iftau,iid4)
 
c     Subroutine to
c        (1) convert a log-likelihood into a likelihood;
c        (2) weight that likelihood by some weight; and
c        (3) aggregate the likelihood module.
c
c     Input derivatives of the log-likelihood are in d... arrays;
c     output derivatives of the likelihood in dd... arrays.
c
c     Note:  dd... arrays are accumulated, with the exception of ddui.
c     It only serves as scratch array and is further unravelled into
c     ddcdm in this routine (map_u).
c
c     Constantijn Panis, 8 December 2003
c     Last change: check lamtyp for unraveling integrated distributions.
c     Was bug; see subroutine onepoint and Trouble\trouble13.
 
      implicit none
      integer nest,thisnest,numdist,sumintdim,sumdim2,sumbigintdim,
     *   neh,nel,nep,nec,neb,nen,nepo,neml,nneml,nfullplus,dimlam,dimii,
     *   allint6,maxpnts,intpnts(sumintdim),intpntr(sumintdim),
     *   nested(numdist),lampos(numdist,8),lamtyp(numdist,8),
     *   indx(sumintdim),dimused(sumintdim),thisdraw(numdist),
     *   distyp(numdist),distpos(numdist),
     *   posi1,posi2,bigposi1,posl4l6,lp,i
      integer intdist(thisnest),intdim,intdimc
      double precision likeg,dweight,weightlike,z(maxpnts),wt(maxpnts),
     *   ui(sumintdim),ddui(sumintdim),ddepsh(neh),ddepsl(nel),
     *   ddepsp(nep),ddepsc(nec),ddepsb(neb),ddepsn(nen),ddepspo(nepo),
     *   ddepsml(nneml),cdm(sumdim2),ddcdm(sumdim2),siga6a6ia6(nec),
     *   sigl4l6(numdist*nep*nec),
     *   dsigl4l6(numdist*nep*nec,0:thisnest-1),
     *   ddsigl4l6(numdist*nep*nec,thisnest),sigl4l6a6a6i(*),minexp
      double precision like(thisnest),lnl(0:thisnest),weight(thisnest),
     *   fullbeta(nfullplus),dfullbeta(nfullplus,0:thisnest),
     *   ddfullbeta(nfullplus,thisnest),lam(dimlam),
     *   dlam(dimlam,0:thisnest-1),ddlam(dimlam,thisnest),
     *   cmui(sumbigintdim),dcmui(sumbigintdim,0:thisnest-1),
     *   ddcmui(sumbigintdim,thisnest),siga6a6i(nec,nec),
     *   dsiga6a6(nec*nec,0:thisnest),ddsiga6a6(nec*nec,thisnest),
     *   dsigii(dimii,0:thisnest-1),ddsigii(dimii,thisnest-1),
     *   dxbh(neh,0:thisnest),ddxbh(neh,thisnest),
     *   dxbc(nec,0:thisnest),ddxbc(nec,thisnest),
     *   dxblp(nel+nep,0:thisnest),ddxblp(nel+nep,thisnest),
     *   dtau(2*(nel+nep),0:thisnest),ddtau(2*(nel+nep),thisnest),
     *   dsiga4a4(nep*nep,0:thisnest),ddsiga4a4(nep*nep,thisnest),
     *   da5(nen,0:thisnest),dda5(nen,thisnest),
     *   dp5(neb+nen+nepo,0:thisnest),ddp5(neb+nen+nepo,thisnest),
     *   dxbml(nneml,0:thisnest),ddxbml(nneml,thisnest)
      logical ifdrv,iflam(numdist),iftau,iid4,ifany
 
c     Convert the log-likelihood into the likelihood and weight it:
      if(lnl(nest).le.minexp())goto 200
      likeg=dexp(lnl(nest))
      weightlike = weight(nest)*likeg
      if(weightlike.eq.0.d0)goto 200
 
c     Accumulate the likelihood:
      like(nest) = like(nest) + weightlike
 
      if(.not.ifdrv)goto 200
 
      dweight=likeg
 
c     Same with derivatives:  convert of log-likelihood into of
c     likelihood, weight, and accumulate.
c     Derivatives wrt hazard and finite mixture parameters:
      call add(dfullbeta(1,nest),ddfullbeta(1,nest),weightlike,
     *   nfullplus)
 
c     Hazard derivatives:
      if(neh.gt.0)then
         call mult(dxbh(1,nest),weightlike,ddepsh,neh)
         call add(dxbh(1,nest),ddxbh(1,nest),weightlike,neh)
      endif
 
c     Accumulate logit/probit derivatives:
      if(nel+nep.gt.0)then
c        First convert and weight dxblp; it will be needed below
         call mult(dxblp(1,nest),weightlike,dxblp(1,nest),nel+nep)
         call vadd(ddxblp(1,nest),dxblp(1,nest),ddxblp(1,nest),nel+nep)
         call copy8(dxblp(1,nest),ddepsl,nel)
         if(nep.gt.0)call copy8(dxblp(nel+1,nest),ddepsp,nep)
         if(iftau)call add(dtau(1,nest),ddtau(1,nest),weightlike,
     *      2*(nel+nep))
         if(.not.iid4)call add(dsiga4a4(1,nest),ddsiga4a4(1,nest),
     *      weightlike,nep*nep)
      endif
 
c     Accumulate continuous derivatives, if needed:
      if(allint6.eq.1)then
         call add(dsiga6a6(1,nest),ddsiga6a6(1,nest),weightlike,nec*nec)
         call mult(dxbc(1,nest),weightlike,ddepsc,nec)
         call add(dxbc(1,nest),ddxbc(1,nest),weightlike,nec)
      endif
 
c     Accumulate negative binomial derivatives:
      if(neb+nen+nepo.gt.0)then
         call add(dp5(1,nest),ddp5(1,nest),weightlike,neb+nen+nepo)
         if(neb.gt.0)call mult(dp5(1,nest),weightlike,ddepsb,neb)
         if(nen.gt.0)then
            call add(da5(1,nest),dda5(1,nest),weightlike,nen)
            call mult(dp5(neb+1,nest),weightlike,ddepsn,nen)
         endif
         if(nepo.gt.0)call mult(dp5(neb+nen+1,nest),weightlike,ddepspo,
     *      nepo)
      endif
 
c     Accumulate multinomial logit derivatives:
      if(neml.gt.0)then
         call add(dxbml(1,nest),ddxbml(1,nest),weightlike,nneml)
         call mult(dxbml(1,nest),weightlike,ddepsml,nneml)
      endif
 
      if(nest.lt.thisnest)then
         if(ifany(iflam,numdist))
     *      call add(dlam(1,nest),ddlam(1,nest),weightlike,dimlam)
         call add(dsigii(1,nest),ddsigii(1,nest),weightlike,dimii)
         call add(dcmui(1,nest),ddcmui(1,nest),weightlike,sumbigintdim)
         if(nec.gt.0.and.allint6.eq.1.and.nep.gt.0.and..not.iid4)call
     *      add(dsigl4l6(1,nest),ddsigl4l6(1,nest),weightlike,
     *      nep*nec*numdist)
      endif
 
c     If integrated distributions are not integrated out of continuous
c     modules, the conditional means of all distributions were computed
c     in compsig44, called by ithobs, and derivatives will be unraveled
c     by compsig44 as well.  If integrated distributions are also
c     integrated out of continuous modules, the conditional mean of
c     non-integrated distributions (relevant for probit modules) was
c     computed in onepoint and derivatives are unraveled here.  Note
c     that conditional covariance matrices are handled by compsig44;
c     they do not depend on a6=Y-XB-eps6.  Also note that there are
c     no conditional means of integrated distributions to worry about.
      if(nest.eq.thisnest.and.
     *   nec.gt.0.and.allint6.eq.1.and.nep.gt.0.and..not.iid4)then
         posl4l6=1
         do 110 i=1,numdist
            if(nested(i).gt.0)goto 120
c           Unravel wrt the conditional mean of probit residuals (cmua4)
c           into wrt Sl4l6, Sa6a6, eps6 and a6.
c           Note that mu = Sl4l6*Sa6a6i*(Y-BX-eps6), so
c              dSl4l6 = dmu*(Sa6a6i*a6)'
c              dSa6a6 = -Sa6a6i*Sl4l6'*dmu*a6'*Sa6a6i
c                     = -(Sl4l6*Sa6a6i)'*dmu*(Sa6a6i*a6)'
c              depsc = -(Sl4l6*Sa6a6i)'*dmu
c              dXB    = -(Sl4l6*Sa6a6i)'*dmu
c           The derivative wrt conditional mean cmua4 is equal to wrt
c           heterogeneity draws ddepsp.
c           First compute scratch array Sl4l6*Sa6a6i
            call m2mult(000,nep,nec,nec,sigl4l6(posl4l6),
     *         siga6a6i,sigl4l6a6a6i,0)
            call m2mult(010,nep,1,nec,ddepsp,siga6a6ia6,
     *         ddsigl4l6(posl4l6,nest),1)
c           The following ruins symmetry of dSa6a6, so restore it:
            call m3mult(1010,nec,nep,1,nec,sigl4l6a6a6i,ddepsp,
     *         siga6a6ia6,ddsiga6a6(1,nest),-1)
            if(nec.gt.1)call makesym(nec,ddsiga6a6(1,nest))
            call m2mult(100,nec,nep,1,sigl4l6a6a6i,ddepsp,ddepsc,-1)
  120       posl4l6=posl4l6+nep*nec
  110    continue
      endif
 
      if(thisdraw(intdist(nest)).eq.0)goto 200
 
c     Unravel wrt eps into wrt Lambda and ui.  Only deal with the
c     portions that are relevant for the current integration loop.
c     Portions related to inner distributions (>nest) were dealt with
c     in previous calls to this routine; portions related to outer
c     distributions (<nest) will be dealt with in later calls.
c     All of this is relevant only if the current distribution is
c     indeed integrated out.
c     The load matrices of integrated distributions are always generic
c     (lamtyp=0), so there is no need to check for special Lambda properties.
      call zerodp(intdim,ddui(posi1))
      if(neh.gt.0.and.lamtyp(intdist(nest),1).ne.-1)then
         lp=lampos(intdist(nest),1)+1+
     *      (thisdraw(intdist(nest))-1)*intdim*neh
         if(iflam(intdist(nest)))call m2mult(010,neh,1,intdim,ddepsh,
     *      ui(posi1),ddlam(lp,nest),1)
         call m2mult(100,intdim,neh,1,lam(lp),ddepsh,ddui(posi1),1)
      endif
      if(nel.gt.0.and.lamtyp(intdist(nest),2).ne.-1)then
         lp=lampos(intdist(nest),2)+1+
     *      (thisdraw(intdist(nest))-1)*intdim*nel
         if(iflam(intdist(nest)))call m2mult(010,nel,1,intdim,ddepsl,
     *      ui(posi1),ddlam(lp,nest),1)
         call m2mult(100,intdim,nel,1,lam(lp),ddepsl,ddui(posi1),1)
      endif
      if(nep.gt.0.and.lamtyp(intdist(nest),3).ne.-1)then
         lp=lampos(intdist(nest),3)+1+
     *      (thisdraw(intdist(nest))-1)*intdim*nep
         if(iflam(intdist(nest)))call m2mult(010,nep,1,intdim,ddepsp,
     *      ui(posi1),ddlam(lp,nest),1)
         call m2mult(100,intdim,nep,1,lam(lp),ddepsp,ddui(posi1),1)
      endif
      if(nec.gt.0.and.allint6.eq.1.and.
     *   lamtyp(intdist(nest),4).ne.-1)then
         lp=lampos(intdist(nest),4)+1+
     *      (thisdraw(intdist(nest))-1)*intdim*nec
         if(iflam(intdist(nest)))call m2mult(010,nec,1,intdim,ddepsc,
     *      ui(posi1),ddlam(lp,nest),1)
         call m2mult(100,intdim,nec,1,lam(lp),ddepsc,ddui(posi1),1)
      endif
      if(neb.gt.0.and.lamtyp(intdist(nest),5).ne.-1)then
         lp=lampos(intdist(nest),5)+1+
     *      (thisdraw(intdist(nest))-1)*intdim*neb
         if(iflam(intdist(nest)))call m2mult(010,neb,1,intdim,ddepsb,
     *      ui(posi1),ddlam(lp,nest),1)
         call m2mult(100,intdim,neb,1,lam(lp),ddepsb,ddui(posi1),1)
      endif
      if(nen.gt.0.and.lamtyp(intdist(nest),6).ne.-1)then
         lp=lampos(intdist(nest),6)+1+
     *      (thisdraw(intdist(nest))-1)*intdim*nen
         if(iflam(intdist(nest)))call m2mult(010,nen,1,intdim,ddepsn,
     *      ui(posi1),ddlam(lp,nest),1)
         call m2mult(100,intdim,nen,1,lam(lp),ddepsn,ddui(posi1),1)
      endif
      if(nepo.gt.0.and.lamtyp(intdist(nest),7).ne.-1)then
         lp=lampos(intdist(nest),7)+1+
     *      (thisdraw(intdist(nest))-1)*intdim*nepo
         if(iflam(intdist(nest)))call m2mult(010,nepo,1,intdim,ddepspo,
     *      ui(posi1),ddlam(lp,nest),1)
         call m2mult(100,intdim,nepo,1,lam(lp),ddepspo,ddui(posi1),1)
      endif
      if(neml.gt.0.and.lamtyp(intdist(nest),8).ne.-1)then
         lp=lampos(intdist(nest),8)+1+
     *      (thisdraw(intdist(nest))-1)*intdim*nneml
         if(iflam(intdist(nest)))call m2mult(010,nneml,1,intdim,ddepsml,
     *      ui(posi1),ddlam(lp,nest),1)
         call m2mult(100,intdim,nneml,1,lam(lp),ddepsml,ddui(posi1),1)
      endif
 
c     Note that continuous models only contributed to ddui, above,
c     if integrated distributions are integrated out of continuous
c     modules (allint6=1).  If integrated distributions are not
c     integrated out of continuous modules, there will be a nonzero
c     conditional mean of integrated distributions.  Those means
c     were computed in compsig44 as cmui and are accumulated here.
      if(allint6.eq.-1)call vadd(ddcmui(bigposi1,nest),ddui(posi1),
     *   ddcmui(bigposi1,nest),intdim)
 
c     Unravel the currently relevant portion of ddui into wrt the
c     Cholesky decomposition (or finite mixture parameters).
      call map_u(2,nfullplus,numdist,sumintdim,sumdim2,sumbigintdim,
     *   intdist(nest),intdim,intdimc,nec,allint6,posi1,posi2,bigposi1,
     *   maxpnts,intpnts,intpntr,dimused,indx,distyp,distpos,fullbeta,
     *   ddfullbeta(1,nest),z,wt,weight(nest),
     *   dweight,cdm,ddcdm,ui,ddui,cmui)
 
c     Re-set derivatives to zero.  Control may be passed here when the
c     conditional likelihood underflows to zero, in which case derivatives
c     are not accumulated but still need to be re-set to zero.
  200 lnl(nest)=0.d0
      if(.not.ifdrv)goto 999
      call zerodp(nfullplus,dfullbeta(1,nest))
      if(neh.gt.0)call zerodp(neh,dxbh(1,nest))
      if(nel+nep.gt.0)then
         call zerodp(nel+nep,dxblp(1,nest))
         if(iftau)call zerodp(2*(nel+nep),dtau(1,nest))
         if(.not.iid4)call zerodp(nep*nep,dsiga4a4(1,nest))
      endif
      if(allint6.eq.1)then
         call zerodp(nec*nec,dsiga6a6(1,nest))
         call zerodp(nec,dxbc(1,nest))
      endif
      if(nen.gt.0)call zerodp(nen,da5(1,nest))
      if(neb+nen+nepo.gt.0)call zerodp(neb+nen+nepo,dp5(1,nest))
      if(neml.gt.0)call zerodp(nneml,dxbml(1,nest))
      if(nest.lt.thisnest)then
         if(ifany(iflam,numdist))call zerodp(dimlam,dlam(1,nest))
         call zerodp(dimii,dsigii(1,nest))
         call zerodp(sumbigintdim,dcmui(1,nest))
         if(nec.gt.0.and.allint6.eq.1.and.nep.gt.0.and..not.iid4)call
     *      zerodp(nep*nec*numdist,dsigl4l6(1,nest))
      endif
 
  999 return
      end
 
