      subroutine write_freq3(values,freqs,ncat,anymin,levelmax,
     *   datanobs,nobs,unit,detail)
 
c     Subroutine to write frequency table.  The array with unique
c     values, values, must be real but integer-valued.  This routine
c     should only be used for datastructure.
 
c     Values are ordered before being written.  The original values
c     order is not touched though.
c     This routine is very similar to write_freq.  The difference is
c     in that write_freq3 allows for extra information regarding the
c     numbers entering in numerator and denominator; this feature is very
c     specific to model aML.  No rest category is allowed, as in
c     write_freq.
c
c     Output may take two formats, depending on whether there are any
c     episodes in the denominator of the likelihood function:
c
c      STRUCTURE |   Freq.  Percent
c     -----------+------------------
c            110 |     10      4.41   (all dropped)
c            120 |      7      3.08   (all dropped)
c           1421 |    143     83.00   (2262 outcomes)
c          22111 |     20      0.00   (all dropped)
c          22121 |      1      0.44   (all dropped)
c     -----------+------------------
c          Total |    181    100.00   (2262 outcomes)
c
c
c      STRUCTURE |   Freq.  Percent
c     -----------+------------------
c            110 |     10      4.41   (all dropped)
c            120 |      7      3.08   (0 outcomes in numerator, 7 in denominator
c           1421 |    143     83.00   (143 outcomes in numerator, 0 in denominat
c          22111 |     20      0.00   (17 outcomes in numerator, 17 in denominat
c          22121 |      1      0.44   (all dropped)
c     -----------+------------------
c          Total |    181    100.00   (160 outcomes in numerator, 24 in denomina
c
c     The first column of freqs contains he data structure frequencies
c     as appear in the input data; the second contains the frequencies
c     used in the numerator; the third those used in the denominator.
c
c     Constantijn Panis, 5 June 1998
 
      implicit none
      integer ncat,freqs(ncat,3),levelmax,i,total0,total1,total2,
     *   used1,used2,imin,cat,datanobs,nobs,unit,intmax,intvecsum
      real values(ncat),minimum,previous,percent
      logical detail,anymin
 
      total0 = intvecsum(freqs,ncat)
      total1 = intvecsum(freqs(1,2),ncat)
      total2 = intvecsum(freqs(1,3),ncat)
 
c     Write out data structure frequency table, if necessary:
      if(.not.detail.or.levelmax.eq.1.or.values(1).eq.0.)goto 200
 
c     Write out table in increasing value order; the ranking procedure
c     is far from efficient, but the original order is preserved.
      write(unit,101)
  101 format(/,'Frequency distribution of data structures:',/,
     *   ' STRUCTURE |   Freq.  Percent',/,11('-'),'+',18('-'))
      previous = -intmax()
      do 100 cat=1,ncat
c        This routine is only used for integer-valued datastructures:
         if(values(cat).ne.int(values(cat)))call bug(1,1)
         minimum = intmax()
         do 110 i=1,ncat
            if(values(i).lt.minimum.and.values(i).gt.previous)then
               minimum = values(i)
               imin = i
            endif
  110    continue
         previous = minimum
         percent = 100.*real(freqs(imin,1))/real(total0)
 
         if(anymin)then
            write(unit,'(i10,a,i7,f9.2,$)')int(values(imin)),
     *         ' |',freqs(imin,1),percent
c           write(unit,'(i10,a,i7,f9.2,$)')int(values(imin)),
c    *         ' |',freqs(imin,1),percent
cD           write(unit,'(i10,a,i7,f9.2)',advance='no')int(values(imin)),
cD    *         ' |',freqs(imin,1),percent
            used1 = freqs(imin,2)
            used2 = freqs(imin,3)
            if(used1.eq.0.and.used2.eq.0)then
               write(unit,'(a)')'   (all dropped)'
            else
               call writea2ia(unit,0,1,'   (',used1,' outcomes in '//
     *            'numerator, ',used2,' in denominator)')
            endif
         else
            write(unit,'(i10,a,i7,f9.2,$)')int(values(imin)),
     *         ' |',freqs(imin,1),percent
c           write(unit,'(i10,a,i7,f9.2,$)')int(values(imin)),
c    *         ' |',freqs(imin,1),percent
cD           write(unit,'(i10,a,i7,f9.2)',advance='no')int(values(imin)),
cD    *         ' |',freqs(imin,1),percent
            used1 = freqs(imin,2)
            if(used1.eq.0)then
               write(unit,'(a)')'   (all dropped)'
            else
               call writeaia(unit,0,1,'   (',used1,' outcomes)')
            endif
         endif
  100 continue
 
      if(anymin)then
         write(unit,113)total0,100.
c        write(unit,113)total0,100.
cD        write(unit,113,advance='no')total0,100.
  113    format(11('-'),'+',18('-'),/,5x,'Total |',i7,f9.2,$)
c 113    format(11('-'),'+',18('-'),/,5x,'Total |',i7,f9.2,$)
cD 113    format(11('-'),'+',18('-'),/,5x,'Total |',i7,f9.2)
         call writea2ia(unit,0,1,'   (',total1,' outcomes in '//
     *      'numerator, ',total2,' in denominator)')
      else
         write(unit,125)total0,100.
c        write(unit,125)total0,100.
cD        write(unit,125,advance='no')total0,100.
  125    format(11('-'),'+',18('-'),/,5x,'Total |',i7,f9.2,$)
c 125    format(11('-'),'+',18('-'),/,5x,'Total |',i7,f9.2,$)
cD 125    format(11('-'),'+',18('-'),/,5x,'Total |',i7,f9.2)
         if(total1.eq.0)then
            write(unit,'(a)')'   (all dropped)'
         else
            call writeaia(unit,0,1,'   (',total1,' outcomes)')
         endif
      endif
 
c     Summarize the number of observations (in data, in estimation),
c     level 2 branches, and outcomes.
  200 write(unit,'(1x)')
      if(anymin)then
c        There are modules in both the numerator and the denominator
         if(levelmax.eq.1)then
c           Only one level, no data structures
            if(nobs.eq.datanobs)call writea3ia(unit,72,1,'Note: the '//
     *         'number of observations is ',nobs,'; they generated ',
     *         total1,' outcomes in the numerator of the likelihood '//
     *         'function and ',total2,' outcomes in the denominator.')
            if(nobs.ne.datanobs)call writea4ia(unit,72,1,'Note: the '//
     *         'number of observations in the data set is ',datanobs,
     *         '.  Of these, ',nobs,' are used in the estimation.  '//
     *         'They generated ',total1,' outcomes in the numerator '//
     *         'of the likelihood function and ',total2,' outcomes '//
     *         'in the denominator.')
         else
            if(nobs.eq.datanobs)call writea4ia(unit,72,1,'Note: the '//
     *         'number of observations is ',nobs,', the number of '//
     *         'level 2 branches ',total0,'; they generated ',total1,
     *         ' outcomes in the numerator of the likelihood '//
     *         'function and ',total2,' outcomes in the denominator.')
            if(nobs.ne.datanobs)call writea5ia(unit,72,1,'Note: the '//
     *         'number of observations in the data set is ',datanobs,
     *         '.  Of these, ',nobs,' are used in the estimation.  '//
     *         'They contain ',total0,' level 2 branches and '//
     *         'generated ',total1,' outcomes in the numerator of '//
     *         'the likelihood function and ',total2,' outcomes in '//
     *         'the denominator.')
         endif
      else
c        No likelihood denominator
         if(levelmax.eq.1)then
c           Only one level, no data structures
            if(nobs.eq.datanobs)call writea2ia(unit,72,1,'Note: the '//
     *         'number of observations is ',datanobs,'; they '//
     *         'generated ',total1,' outcomes.')
            if(nobs.ne.datanobs)call writea3ia(unit,72,1,'Note: the '//
     *         'number of observations in the data set is ',datanobs,
     *         '.  Of these, ',nobs,' are used in the estimation.  '//
     *         'They generated ',total1,' outcomes.')
         else
            if(nobs.eq.datanobs)call writea3ia(unit,72,1,'Note: the '//
     *         'number of observations is ',datanobs,
     *         ', the number of level 2 branches ',total0,'; they '//
     *         'generated ',total1,' outcomes.')
            if(nobs.ne.datanobs)call writea4ia(unit,72,1,'Note: the '//
     *         'number of observations in the data set is ',datanobs,
     *         '.  Of these, ',nobs,' are used in the estimation.  '//
     *         'They contain ',total0,' level 2 branches and '//
     *         'generated ',total1,' outcomes.')
         endif
      endif
 
      return
      end
 
