      subroutine writeiter(nfull,nfree,fulltype,iffull,output,ipt,jpt,
     *   iter,fullbeta,newfullbeta,fp,eigenvalues,nchol,chol,searchdir,
     *   oldlnl,lnl,convinfo,fullname)
 
c     Subroutine to write out information per iteration.
c
c     Array fullbeta contains untransformed parameters; fp derivatives wrt
c     untransformed parameters; searchdir the search direction wrt transformed
c     parameters (i.e., to be untransformed before printed); newfullbeta is a
c     scratch array.
c
c     Constantijn Panis, 16 May 2004
c     Last change:  add non-concavity statement if needed.
 
      implicit none
      integer nfull,nfree,fulltype(nfull),iffull(nfull),output(10),
     *   ipt,jpt,iter,nchol,i,j,status,trm,intlen
      double precision fullbeta(nfull),newfullbeta(nfull),fp(nfree),
     *   eigenvalues(5),chol(nchol),searchdir(nfree),oldlnl,lnl,
     *   convinfo(4),temp,dpinf
      character fullname(nfull)*8,fmt*16,i2c*12
      logical isinf
 
      if(iter.gt.1.or.output(3).eq.1)then
         if(ipt.ge.3)write(8,'(/,70a)')('-',i=1,70)
         if(jpt.ge.3)write(6,'(/,70a)')('-',i=1,70)
      endif
      fmt='(/,a,i'//i2c(intlen(iter),0)
      fmt=fmt(1:trm(fmt))//',t24,a,$)'
      if(ipt.ge.2)write(8,fmt)'ITERATION ',iter,'LOG-LIKELIHOOD:'
      if(jpt.ge.2)write(6,fmt)'ITERATION ',iter,'LOG-LIKELIHOOD:'
      if(eigenvalues(1).ge.1.d-3)then
         if(ipt.ge.2)call writedble(8,lnl,2,-15,6,0)
         if(jpt.ge.2)call writedble(6,lnl,2,-15,6,0)
         if(ipt.ge.2)write(8,'(a)')'  (not concave)'
         if(jpt.ge.2)write(6,'(a)')'  (not concave)'
      else
         if(ipt.ge.2)call writedble(8,lnl,2,-15,6,1)
         if(jpt.ge.2)call writedble(6,lnl,2,-15,6,1)
      endif
      if(iter.gt.1)then
         if(ipt.ge.3)write(8,102)
         if(jpt.ge.3)write(6,102)
         if(ipt.ge.3)call writedble(8,lnl-oldlnl,2,-13,6,1)
         if(jpt.ge.3)call writedble(6,lnl-oldlnl,2,-13,6,1)
      endif
  102 format(t24,'ABSOLUTE IMPROVEMENT:',$)
 
      if(ipt.ge.3)write(8,112)
      if(jpt.ge.3)write(6,112)
  112 format(/,'PARAMETER        VALUE     GRADIENT   SEARCH DIR')
      if(ipt.ge.3.or.jpt.ge.3)then
         call fignewbeta(1,nfull,nfree,fulltype,iffull,fullbeta,
     *      searchdir,1.d0,newfullbeta,0.d0,nchol,chol,status)
         j=0
         do 100 i=1,nfull
            if(iffull(i).eq.0)goto 100
            j=j+1
            if(status.eq.1.and.isinf(newfullbeta(i)))then
               temp=dpinf()
            else
               temp=newfullbeta(i)-fullbeta(i)
            endif
            if(ipt.ge.3)then
               write(8,'(a8,$)')fullname(i)
               call writedble(8,fullbeta(i),-6,9,0,0)
               call writedble(8,fp(j),-5,9,0,0)
               call writedble(8,temp,-5,9,0,1)
            endif
            if(jpt.ge.3)then
               write(6,'(a8,$)')fullname(i)
               call writedble(6,fullbeta(i),-6,9,0,0)
               call writedble(6,fp(j),-5,9,0,0)
               call writedble(6,temp,-5,9,0,1)
            endif
  100    continue
      endif
 
c     Write out the Hessian's smallest eigenvalues (or, more precisely,
c     the opposite of the largest eigenvalues.  Eigenvalues are all
c     supposed to be negative, so that the largest are close to zero.)
      if(ipt.ge.3)write(8,121)
      if(jpt.ge.3)write(6,121)
  121 format(/,'SMALLEST EIGENVALUES:')
      do 110 i=1,min(5,nfree)
         if(ipt.ge.3)call writedble(8,-eigenvalues(i),4,8,0,0)
         if(jpt.ge.3)call writedble(6,-eigenvalues(i),4,8,0,0)
  110 continue
 
      if(ipt.ge.3)then
         write(8,'(//,a,$)')'REL PARAM CHG:'
         call writedble(8,convinfo(4),2,8,0,0)
         write(8,'(6x,a,$)')'WGTD GRAD NORM:'
         call writedble(8,convinfo(1),2,8,0,1)
         write(8,'(4x,a,$)')'GRAD NORM:'
         call writedble(8,convinfo(3),2,8,0,0)
         write(8,'(7x,a,$)')'REL LN-L IMPR:'
         call writedble(8,convinfo(2),2,8,0,1)
      endif
      if(jpt.ge.3)then
         write(6,'(//,a,$)')'REL PARAM CHG:'
         call writedble(6,convinfo(4),2,8,0,0)
         write(6,'(6x,a,$)')'WGTD GRAD NORM:'
         call writedble(6,convinfo(1),2,8,0,1)
         write(6,'(4x,a,$)')'GRAD NORM:'
         call writedble(6,convinfo(3),2,8,0,0)
         write(6,'(7x,a,$)')'REL LN-L IMPR:'
         call writedble(6,convinfo(2),2,8,0,1)
      endif
      if(ipt.ge.2)call flush(8)
      goto 999
 
  999 return
      end
 
