      subroutine wrmodel4(levelmax,unit,nfull,numdist,sumdim,
     *   numpar,meno,moel,distdim,pardim,parpos,namepntr,nold,
     *   maxxnode,xnode,oldname,parname,rsetname,resname,splname)
 
c     Subroutine to write out a probit model specification.
c     There is a number of different types:
c     (1) ordered probit/logit: outcomes are two categorical variables,
c         thresholds is parameter set:
c         moel(+1)    -61
c         moel(+2)    position in xnode for two outcome variables
c         moel(+3)    1 direct threshold vector reference
c                     3 indirect reference
c         moel(+4)    vector number if moel(+3)=1
c                     reference expression if moel(+3)=3
c     (2) ordered probit/logit: thresholds in data
c         moel(+1) = -62
c         moel(+2) = variable/expression for first threshold;
c                    -Infinity; second threshold; +Infinity
c     (3) simple probit/logit: outcome is binary variable,
c         threshold is parameter
c         moel(+1)    -63
c         moel(+2)    position in xnode for outcome variable
c         moel(+3)    1 direct threshold vector reference
c                     3 indirect reference
c         moel(+4)    vector number if moel(+3)=1
c                     reference expression if moel(+3)=3
c     (4) simple probit/logit: outcome is binary variable,
c         threshold is assumed fixed at zero
c         moel(+1)    -64
c         moel(+2)    position in xnode for outcome variable
c
c     Constantijn Panis,  10 April 1998
 
      implicit none
      integer levelmax,unit,nfull,numdist,sumdim,numpar,meno,
     *   moel(*),distdim(numdist),pardim(numpar),parpos(numpar),
     *   namepntr(levelmax),nold,maxxnode,xnodepntr,termtype,trm
      real xnode(maxxnode)
      character oldname(nold)*8,parname(*)*12,rsetname(*)*12,
     *   resname(sumdim)*8,splname(*)*12
 
      termtype=moel(meno)
      meno=meno+1
      if(termtype.eq.-61)then
c     (1) ordered probit/logit: outcomes are two categorical variables,
c         thresholds is parameter set:
c         moel(+1)    -61
c         moel(+2)    position in xnode for two outcome variables
c         moel(+3)    1 direct threshold vector reference
c                     3 indirect reference
c         moel(+4)    vector number if moel(+3)=1
c                     reference expression if moel(+3)=3
         write(unit,'(3x,a,$)')'outcomes = '
c        write(unit,'(3x,a,$)')'outcomes = '
cD       write(unit,'(3x,a)',advance='no')'outcomes = '
         xnodepntr=moel(meno)
         call writeexpr(100,unit,maxxnode,0,levelmax,1,xnodepntr,
     *      0,0,0,0,nold,0,namepntr,-1,0.,xnode,0.,oldname,0)
         write(unit,'(a,$)')' '
c        write(unit,'(a,$)')' '
cD       write(unit,'(a)',advance='no')' '
         call writeexpr(100,unit,maxxnode,0,levelmax,1,xnodepntr,
     *      0,0,0,0,nold,0,namepntr,-1,0.,xnode,0.,oldname,0)
         write(unit,'(a)')';'
         meno=meno+1
         write(unit,'(3x,a,$)')'thresholds = '
c        write(unit,'(3x,a,$)')'thresholds = '
cD       write(unit,'(3x,a)',advance='no')'thresholds = '
         if(moel(meno).eq.1)then
c           Directly referenced:
            write(unit,'(a,a)')parname(moel(meno+1))
     *         (1:trm(parname(moel(meno+1)))),';'
         elseif(moel(meno).eq.3)then
c           Indirectly referenced:
            write(unit,'(3x,a,$)')'(refvar='
c           write(unit,'(3x,a,$)')'(refvar='
cD          write(unit,'(3x,a)',advance='no')'(refvar='
            call writeexpr(100,unit,maxxnode,0,levelmax,1,-moel(meno+1),
     *         0,0,0,0,nold,0,namepntr,-1,0.,xnode,0.,oldname,0)
            write(unit,'(a)')');'
         else
            call bug(1,1)
         endif
         meno=meno+2
      elseif(termtype.eq.-62)then
c     (2) ordered probit/logit: thresholds in data
c         moel(+1) = -62
c         moel(+2) = variable/expression for first threshold;
c                    -Infinity; second threshold; +Infinity
         write(unit,'(3x,a,$)')'threshold vars = '
c        write(unit,'(3x,a,$)')'threshold vars = '
cD       write(unit,'(3x,a)',advance='no')'threshold vars = '
         xnodepntr=moel(meno)
         call writeexpr(100,unit,maxxnode,0,levelmax,1,xnodepntr,
     *      0,0,0,0,nold,0,namepntr,-1,0.,xnode,0.,oldname,0)
         write(unit,'(1x,a,$)')'(-Inf='
c        write(unit,'(1x,a,$)')'(-Inf='
cD       write(unit,'(1x,a)',advance='no')'(-Inf='
         call writeexpr(100,unit,maxxnode,0,levelmax,1,xnodepntr,
     *      0,0,0,0,nold,0,namepntr,-1,0.,xnode,0.,oldname,0)
         write(unit,'(a,$)')') '
c        write(unit,'(a,$)')') '
cD       write(unit,'(a)',advance='no')') '
         call writeexpr(100,unit,maxxnode,0,levelmax,1,xnodepntr,
     *      0,0,0,0,nold,0,namepntr,-1,0.,xnode,0.,oldname,0)
         write(unit,'(1x,a,$)')'(Inf='
c        write(unit,'(1x,a,$)')'(Inf='
cD       write(unit,'(1x,a)',advance='no')'(Inf='
         call writeexpr(100,unit,maxxnode,0,levelmax,1,xnodepntr,
     *      0,0,0,0,nold,0,namepntr,-1,0.,xnode,0.,oldname,0)
         write(unit,'(a)')');'
         meno=meno+1
      elseif(termtype.eq.-63)then
c     (3) simple probit/logit: outcome is binary variable,
c         threshold is parameter
c         moel(+1)    -63
c         moel(+2)    position in xnode for outcome variable
c         moel(+3)    1 direct threshold vector reference
c                     3 indirect reference
c         moel(+4)    vector number if moel(+3)=1
c                     reference expression if moel(+3)=3
         write(unit,'(3x,a,$)')'outcome = '
c        write(unit,'(3x,a,$)')'outcome = '
cD       write(unit,'(3x,a)',advance='no')'outcome = '
         call writeexpr(100,unit,maxxnode,0,levelmax,1,-moel(meno),
     *      0,0,0,0,nold,0,namepntr,-1,0.,xnode,0.,oldname,0)
         write(unit,'(a)')';'
         meno=meno+1
         write(unit,'(3x,a,$)')'threshold = '
c        write(unit,'(3x,a,$)')'threshold = '
cD        write(unit,'(3x,a)',advance='no')'threshold = '
         if(moel(meno).eq.1)then
c           Directly referenced:
            write(unit,'(a,a)')parname(moel(meno+1))
     *         (1:trm(parname(moel(meno+1)))),';'
         elseif(moel(meno).eq.3)then
c           Indirectly referenced:
            write(unit,'(3x,a,$)')'(refvar='
c           write(unit,'(3x,a,$)')'(refvar='
cD          write(unit,'(3x,a)',advance='no')'(refvar='
            call writeexpr(100,unit,maxxnode,0,levelmax,1,-moel(meno+1),
     *         0,0,0,0,nold,0,namepntr,-1,0.,xnode,0.,oldname,0)
            write(unit,'(a)')');'
         else
            call bug(1,1)
         endif
         meno=meno+2
      elseif(termtype.eq.-64)then
c     (4) simple probit/logit: outcome is binary variable,
c         threshold is assumed fixed at zero
c         moel(+1)    -64
c         moel(+2)    position in xnode for outcome variable
         write(unit,'(3x,a,$)')'outcome = '
c        write(unit,'(3x,a,$)')'outcome = '
cD       write(unit,'(3x,a)',advance='no')'outcome = '
         call writeexpr(100,unit,maxxnode,0,levelmax,1,-moel(meno),
     *      0,0,0,0,nold,0,namepntr,-1,0.,xnode,0.,oldname,0)
         write(unit,'(a)')';'
         meno=meno+1
      endif
 
      write(unit,'(3x,a,$)')'model = '
c     write(unit,'(3x,a,$)')'model = '
cD     write(unit,'(3x,a)',advance='no')'model = '
      call wrmodel(levelmax,unit,nfull,numdist,sumdim,numpar,meno,
     *   moel,maxxnode,distdim,pardim,parpos,namepntr,nold,xnode,
     *   oldname,parname,rsetname,resname,splname)
      write(unit,'(/,6x,a)')';'
 
  999 return
      end
