      subroutine wrmodel7(mtype,levelmax,unit,nfull,numdist,sumdim,
     *   numpar,meno,moel,maxxnode,distdim,pardim,parpos,namepntr,nold,
     *   xnode,oldname,parname,rsetname,resname,splname,
     *   nfreq,maxfreqcat,maxnumfreq,freqs,freqcat,prof,status)
 
c     Subroutine to write a multinomial logit/probit model specification.
c
c     For multinomial logit, moel contains:
c        1 = keep/drop (pointer to xnode)
c        2 = number of equations (=number of choices-1)
c        3 = outcome (pointer to xnode)
c        4 = potential outcome 1
c        5 = potential outcome 2
c        ++  et cetera
c        4+neqn  model specification, first equation
c        ++  model specification, next equation,
c        ++  etc.
c
c     For multinomial probit, moel contains:
c        1 = keep/drop (pointer to xnode)
c        2 = number of equations (=number of choices-1)
c        3 = outcome (pointer to xnode)
c        4 = potential outcome 1
c        5 = potential outcome 2
c        ++  et cetera
c        4+neqn  pointer to model specification for first outcome (=4+2*neqn)
c        5+neqn  pointer to model specification for second outcome
c        ++  et cetera
c        4+2*neqn  model specification, first equation
c        ++  model specification, next equation,
c        ++  etc.
c
c     Status [out] 1 indicates that there is no omitted category in
c     multinomial probit.  This signal is suppressed for proficiency=guru
c     (prof=4).
c
c     Constantijn Panis,  7 January 2003
c     Last change: proficiency
 
      implicit none
      integer mtype,levelmax,unit,nfull,numdist,sumdim,numpar,meno,
     *   moel(*),maxxnode,distdim(numdist),pardim(numpar),
     *   parpos(numpar),namepntr(levelmax),nold,prof,status,eqn,neqn,
     *   outpos,case,num,pos,match,i
      real xnode(maxxnode)
      character oldname(nold)*8,rsetname(*)*12,splname(*)*12,
     *   resname(sumdim)*12,parname(*)*12
      integer nfreq,maxfreqcat,maxnumfreq,freqs(maxfreqcat,maxnumfreq)
      real freqcat(maxfreqcat,maxnumfreq)
 
      neqn=moel(meno)
      meno=meno+1
 
c     Outcome:
      call wrapstring(unit,0,0,1,0,'   outcome = ')
      call writeexpr(100,unit,maxxnode,0,levelmax,1,-moel(meno),
     *   0,0,0,0,nold,0,namepntr,-1,0.,xnode,0.,oldname,0)
      write(unit,'(a)')';'
      meno=meno+1
 
c     Figure out what outcome(s) have not been specified.  The possibilities are
c        1.  Too few variables are tabulated (maxnumfreq)
c            (option maximum number of summary statistics = 100;)
c        2.  Too few frequency categories allowed (maxfreqcat<=neqn)
c            (option maximum number of frequency categories = 20;)
c        3.  Nothing omitted -- write a warning for logit, abort for probit
c        4.  One value omitted
c        5.  Multiple values omitted
      case=0
      num=nfreq/(maxnumfreq+1)
      if(num.le.0)then
         case=1
         goto 100
      endif
      pos=mod(nfreq,maxnumfreq+1)
      if(maxfreqcat.lt.neqn.or.freqs(1,pos).lt.0)then
         case=2
         goto 100
      endif
      call realmatch(-99999.,freqcat(1,pos),maxfreqcat,num)
      if(num.eq.0)num=maxfreqcat+1
      num=num-1
      if(num.le.neqn)then
         case=3
      elseif(num.eq.neqn+1)then
         case=4
      else
         case=5
      endif
 
  100 outpos=meno-1
      meno=meno+neqn
c     Multinomial probits have an additional neqn spaces for pointers to model
c     specifications in moel.
      if(mtype.eq.72)meno=meno+neqn
 
      do 200 eqn=1,neqn
         call writeaia(unit,72,0,'   model ',moel(outpos+eqn),' = ')
         call wrmodel(levelmax,unit,nfull,numdist,sumdim,numpar,meno,
     *      moel,maxxnode,distdim,pardim,parpos,namepntr,nold,xnode,
     *      oldname,parname,rsetname,resname,splname)
         write(unit,'(a)')';'
  200 continue
 
      if(case.eq.1.or.case.eq.2)then
         write(unit,201)
  201    format(3x,'/*  Omitted category:  unknown  */')
      elseif(case.eq.3)then
         write(unit,202)
  202    format(3x,'/*  Omitted category:  none  */')
      elseif(case.eq.4)then
         call wrapstring(unit,0,0,1,0,'   /*  Omitted category =')
      elseif(case.eq.5)then
         call wrapstring(unit,0,0,1,0,'   /*  Omitted categories =')
      else
         call bug(1,1)
      endif
      if(case.eq.4.or.case.eq.5)then
         do 210 i=1,num
            call intmatch(int(freqcat(i,pos)),moel(outpos+1),neqn,match)
            if(match.eq.0)call writeint(unit,int(freqcat(i,pos)),1,0)
  210    continue
         write(unit,'(a)')'  */'
      endif
 
      if(case.eq.1.or.case.eq.2.or.case.eq.3)write(unit,'(1x)')
      if(case.eq.1)then
         call writeaia(unit,72,1,'NOTE:  the omitted category is '//
     *      'unknown because too few variables are being tabulated.  '//
     *      'Specify "option maximum number of summary statistics" '//
     *      'sufficiently high.  (Its current value is ',
     *      maxnumfreq,'.)')
      elseif(case.eq.2)then
         call writeaia(unit,72,1,'NOTE:  the omitted category is '//
     *      'unknown because too little space is allocated for '//
     *      'tabulations.  Specify "option maximum number of '//
     *      'frequency categories" sufficiently high.  '//
     *      '(Its current value is ',maxfreqcat,'.)')
      elseif(case.eq.3)then
         if(mtype.eq.71)write(unit,301)
c        Multinomial probit will abort (in write_model) unless proficiency=guru.
         if(mtype.eq.72.and.prof.eq.4)write(unit,301)
         if(mtype.eq.72.and.prof.lt.4)status=1
  301    format('WARNING:  there are model specifications for all ',
     *      'outcomes in the data,',/,'i.e., there is no omitted ',
     *      'category.  Your model is not identified.',/,
     *      'Omit at least one model equation.')
      endif
      goto 999
 
  999 return
      end
