      program mktab
 
c     This program reads multiple aML output files and turns them into
c     a more easily readable table, with one column per output file.
c     Usage:
c        mktab [-c] [-s] [-{NS|ns|t|p}] [-n] name [name...[+]...name]
c
c     where -c places commas between columns and eliminates spaces;
c              convenient for importing the table into MS Word;
c           -s uses strict significance criteria: *=5%, **=1%, ***=.1%.
c              Default is 10-5-1%.
c           -NS only tabulates parameter estimates, omits significance asterisks
c              and standard errors.  May not be combined with -ns, -t or -p.
c           -ns only tabulates parameter estimates and significance asterisks,
c              omits standard errors.  May not be combined with -NS, -t or -p.
c           -t puts absolute t-statistics in parentheses.  Default is
c              standard deviations.  May not be combined with -ns, -NS, or -p.
c           -p puts two-sided p-values in parentheses.  Default is
c              standard deviations.  May not be combined with -ns, -NS or -t.
c           -n (where n is a positive integer) leads to n digits after
c              the decimal point.  The default is 4.
c           name is the name of the aML output file being read in.  If name
c              does not exist, name.out is tried.
c
c     The first output file determines the order in which the coefficients
c     are listed.
c
c     If an output file contains results from multiple runs, the last
c     is taken.
c
c     Multiple output files may be combined in one column by specifying
c     their names separated by a plus sign (+).
c
c     Constantijn Panis, 4 April 2005
c     Last change: maxpar=600 rather than maxpar=300
 
      implicit none
      integer iargc,narg,maxcol,maxpar
      parameter(maxcol=20,maxpar=600)
      double precision beta(maxpar,0:maxcol),std(maxpar,0:maxcol),
     *   lnl(maxcol),dpnan
      integer basis(maxcol),npar(maxcol),df(maxcol),
     *   inparens,width,ncol,numpar,prevpar,nt,il,status
c     integer(2) arg
      integer arg
      character parname(maxpar,0:maxcol)*8,argument*256,filename*256,
     *   head(maxcol)*12
      logical tf(maxpar,maxcol),comma,strict,
     *   prevargdone,prevcoldone
      data status,prevpar/0,0/
      data basis,npar,df/maxcol*0,maxcol*0,maxcol*0/
      data head,lnl/maxcol*'            ',maxcol*0.d0/
      data arg,ncol,width,prevargdone,prevcoldone/0,0,4,2*.true./
      call put(dpnan(),beta,maxpar*(maxcol+1))
      call put(dpnan(),std,maxpar*(maxcol+1))
      data comma,strict,inparens/.false.,.false.,2/
c     inparens = 0 = suppress standard errors and significance asterisks
c                1 = suppress standard errors
c                2 = standard errors in parentheses (default)
c                3 = t-statistics in parentheses (default)
c                4 = p-values in parentheses (default)
 
      narg=iargc()
      if(narg.eq.0)goto 800
      arg=0
  100 arg=arg+1
      call getarg(arg,filename)
      if(filename.eq.'-c')then
         comma=.true.
      elseif(filename.eq.'-s')then
         strict=.true.
      elseif(filename.eq.'-NS')then
         if(inparens.ne.2)goto 810
         inparens=0
      elseif(filename.eq.'-ns')then
         if(inparens.ne.2)goto 810
         inparens=1
      elseif(filename.eq.'-t')then
         if(inparens.ne.2)goto 810
         inparens=3
      elseif(filename.eq.'-p')then
         if(inparens.ne.2)goto 810
         goto 820
         inparens=4
      elseif(filename.eq.'-h'.or.filename.eq.'-?')then
         goto 800
      elseif(filename(1:1).eq.'-')then
         il=2
         call getint(0,width,filename,il,status)
         if(status.gt.0)goto 800
      else
         arg=arg-1
  200    call openfile(arg,narg,ncol,maxcol,argument,filename,nt,
     *      head,prevargdone,prevcoldone,status)
         if(status.eq.-1)goto 300
         call readfile(filename(1:nt),maxpar,npar(ncol),df(ncol),
     *      lnl(ncol),basis(ncol),parname(1,ncol),beta(1,ncol),
     *      std(1,ncol),tf(1,ncol),prevcoldone,prevpar)
         goto 200
      endif
      if(arg.lt.narg)goto 100
 
  300 call sortpar(ncol,maxpar,npar,numpar,beta,std,parname,tf)
      call writetable(width,ncol,numpar,maxpar,beta,std,head,df,
     *   basis,parname,lnl,comma,strict,inparens)
      goto 999
 
  800 write(6,801)
  801 format(/,'Usage: mktab [-c] [-s] [-{NS|ns|t|p}] [-n] ',
     *   'name [name...[+]...name]',//,
     *   'where -c places commas between columns and eliminates spaces',
     *   /,9x,'(convenient for importing the table into spreadsheets',/,
     *   9x,'and wordprocessors);',/,6x,'-s uses strict ',
     *   'criteria for significance asterisks:',/,9x,'*=5%, **=1%, ',
     *   '***=0.1% (the default is 10-5-1%);',/,6x,
     *   '-NS suppresses standard errors and significance asterisks',/,
     *   6x,'-ns suppresses standard errors',/,
     *   6x,'-t puts absolute t-statistics in parentheses',/,
     *   6x,'-p puts two-sided p-values in parentheses',/,
     *   6x,'-n (where n is a positive integer) leads to n digits ',
     *   'after ',/,9x,'the decimal point (the default is -4);',/,
     *   4x,'name is the name of an aML output file to be tabulated.')
      goto 999
  810 write(6,811)
  811 format(/,'ERROR: options -NS, -ns, -t, and -p may not be ',
     *   'combined.')
      goto 999
  820 write(6,811)
  821 format(/,'I am sorry; p-values are not yet implemented on UNIX ',
     *   'platforms.')
      goto 999
  999 continue
      end
c=========================================================================
      subroutine openfile(arg,narg,ncol,maxcol,argument,filename,
     *   nt,head,prevargdone,prevcoldone,status)
      implicit none
      integer narg,ncol,maxcol,nt,status,nh,first,last,i1,i2,i,trm
c     integer(2) arg
      integer arg
      character argument*256,filename*256,head(maxcol)*12
      logical prevargdone,prevcoldone,foundplus,exist
      save first,last
 
      status=0
      foundplus=.false.
  100 if(prevargdone)then
         arg=arg+1
         if(arg.gt.narg)then
            status=-1
            goto 999
         endif
         call getarg(arg,argument)
         first=1
      else
         first=last+1
      endif
      last=trm(argument)
 
      if(argument(first:first).eq.'+')then
         foundplus=.true.
         prevcoldone=.false.
         first=first+1
         if(first.gt.last)then
            prevargdone=.true.
            goto 100
         endif
      elseif(.not.foundplus)then
         prevcoldone=.true.
      endif
 
  120 if(exist(argument(first:last)))then
         filename=argument(first:last)
         open(4,file=filename)
         if(prevcoldone)ncol=ncol+1
         prevargdone = (last.eq.trm(argument))
      elseif(exist(argument(first:last)//'.out'))then
         filename=argument(first:last)//'.out'
         open(4,file=filename)
         if(prevcoldone)ncol=ncol+1
         prevargdone = (last.eq.trm(argument))
      else
c        Perhaps the argument is/contains a '+', indicating multiple files
c        to be combined into one column.
         do 110 i=first,last
            if(argument(i:i).eq.'+')then
               last=i-1
               goto 120
            endif
  110    continue
c        Nothing matches:
         goto 800
      endif
 
  200 if(ncol.gt.maxcol)goto 810
 
c     Determine the title corresponding to this file
      nt=trm(filename)
      do 210 i=nt,1,-1
c        if(filename(i:i).eq.'/'.or.filename(i:i).eq.'\\')then
         if(filename(i:i).eq.'/'.or.filename(i:i).eq.'\\')then
            i1=i+1
            goto 220
         endif
  210 continue
      i1=1
  220 do 230 i2=i1,nt
         if(filename(i2:i2).eq.'.')goto 240
  230 continue
      i2=nt+1
  240 if(head(ncol).eq.' ')then
         head(ncol)=filename(i1:i2-1)
      else
         nh=trm(head(ncol))
         if(nh+1+i2-i1.le.12)then
            head(ncol)=head(ncol)(1:nh)//'+'//filename(i1:i2-1)
         else
            head(ncol)=head(ncol)(1:nh)//'+'//'...........'
         endif
      endif
      goto 999
 
  800 if(argument(last-2:last).eq.'out'.or.
     *   argument(last-2:last).eq.'OUT')then
         call not_found(6,argument(first:last))
      else
         call not_found(6,argument(first:last)//'.out')
      endif
      stop
  810 call writeaia(6,72,1,'Error: the maximum number of columns (',
     *   maxcol,') is exceeded.  Contact Technical Support at '//
     *   '<support@applied-ml.com> for a complimentary larger version.')
      stop
  999 return
      end
c=========================================================================
      subroutine readfile(filename,maxpar,npar,df,lnl,basis,parname,
     *   beta,std,tf,prevcoldone,prevpar)
      implicit none
      integer maxpar,npar,df,basis,prevpar,
     *   round,i,ipar,ii,i1,i2,il,status,nobs,trm
      double precision lnl,beta(maxpar),std(maxpar),templnl
      character filename*(*),line*128,parname(maxpar)*8
      logical converged,tf(maxpar),prevcoldone,isnan,smtxt
 
      if(prevcoldone)then
         prevpar=0
         lnl=0.d0
      endif
      npar=0
 
c     Determine the number of observations and (final) model parameters
c     and compute the number of degrees of freedom.
  100 read(4,'(a80)',err=800,end=800)line
      if(line(1:32).ne.'Note: the number of observations')goto 100
      if(line(34:35).eq.'is')then
c        Note: the number of observations is nobs; they generated ...
         il=37
         call getint(4,nobs,line,il,status)
         if(status.gt.0)goto 800
      elseif(line(34:51).eq.'in the data set is')then
c        Note: the number of observations in the data set is datanobs.  Of these
         il=53
         call gotochar(4,'.',line,il,status)
         if(status.gt.0)goto 800
         if(.not.smtxt(4,line,il,'Of these,'))goto 800
         call getint(4,nobs,line,il,status)
         if(status.gt.0)goto 800
      else
         goto 800
      endif
  110 read(4,'(a80)',err=800,end=800)line
      if(.not.line(1:31).eq.'Number of parameters estimated:')goto 110
c     Number of parameters estimated:    n1;  n2;  n3
      il=trm(line)-2
  120 if(line(il:il).ne.' ')then
         il=il-1
         goto 120
      endif
      call getint(4,df,line,il,status)
      if(status.gt.0)goto 800
      df=nobs-df
 
c     There may be multiple rounds of optimization; skip intermediate ones.
c     Ultimately, scan for "Log Likelihood", and check for convergence.
  130 round=0
  200 read(4,'(a80)',err=800,end=800)line
 
c     Read the number of model parameters.  Note that this information is
c     only written for file info level>=4.
      if(line(1:30).eq.'Number of parameters in model:')then
         backspace 4
         read(4,'(35x,i4)',err=800,end=800)npar
         if(npar.eq.0)goto 800
         if(prevpar+npar.gt.maxpar)goto 810
 
c...+....1....+....2....+....3....+....4....+....5....+....6....+....7....+....8
c                   ROUND 1 CONVERGED SUCCESSFULLY                   =
c                 ESTIMATION CONVERGED SUCCESSFULLY                  =
c                   ESTIMATION FAILED TO CONVERGE                    =
 
      elseif(line(21:25).eq.'ROUND'.and.
     *   line(29:50).eq.'CONVERGED SUCCESSFULLY')then
         round=round+1
         converged=.true.
      elseif(line(21:49).eq.'ESTIMATION FAILED TO CONVERGE')then
         round=round+1
         converged=.false.
      elseif(line(19:51).eq.'ESTIMATION CONVERGED SUCCESSFULLY')then
         round=round+1
         converged=.true.
      endif
      if(line(1:15).ne.'Log Likelihood:')goto 200
 
      backspace 4
      read(4,*,end=800,err=800)line,line,templnl
c     Are standard errors based on BHHH, Huber, or numerical Hessian?
      read(4,'(42x,a38)',err=800,end=800)line
      if(line.eq.'==== Huber-corrected ====')then
         basis=1
      elseif(line.eq.'Based on numerical Hessian')then
         basis=2
      elseif(line.eq.'BHHH-based, non-corrected')then
         basis=3
      elseif(line.eq.' ')then
c        Blank line: probably gradient check only.
         goto 820
      else
         call bug(0,1)
      endif
 
      if(.not.converged)write(6,101)filename
  101 format('WARNING: ''',a,''' did not converge.')
 
      read(4,'(a80)',err=800,end=800)(line,i=1,2)
      ipar=0
  220 read(4,'(a80)',err=800,end=800)line
      il=1
      call getint(0,ii,line,il,status)
      if(status.gt.0)goto 230
      ipar=ipar+1
      if(prevpar+ipar.gt.maxpar)goto 810
      if(ipar.ne.ii)goto 800
      call getstring(0,line,il,i1,i2)
      if(i2.eq.-1)goto 800
      parname(prevpar+ipar)=line(i1:i2)
      call getstring(0,line,il,i1,i2)
      if(i2.eq.-1)goto 800
      tf(prevpar+ipar)=(line(i1:i2).eq.'T')
      call getdp(0,beta(prevpar+ipar),line,il,status)
      if(status.gt.0)goto 800
      if(tf(prevpar+ipar))call getdp(0,std(prevpar+ipar),line,il,status)
      if(status.gt.0)goto 800
      goto 220
 
  230 if(line.ne.' ')goto 800
      if(npar.eq.0)npar=ipar
      if(npar.ne.ipar)goto 800
 
c     End of output file reached.  Make sure results were read in.
      if(isnan(beta(1)))goto 800
 
      npar=prevpar+npar
      prevpar=npar
      lnl=lnl+templnl
      close(4)
      goto 999
 
  800 write(6,801)filename
  801 format('ERROR or END-OF-FILE encountered while reading ''',
     *   a,'''.')
      stop
  810 call writeaia(6,72,1,'ERROR: maximum number of parameters (',
     *   maxpar,') is exceeded.  Contact Technical Support at '//
     *   '<support@applied-ml.com> for a complimentary larger version.')
      stop
  820 write(6,821)filename
  821 format('ERROR: ''',a,''' does not contain estimates.')
      stop
  999 return
      end
c=========================================================================
      subroutine sortpar(ncol,maxpar,npar,numpar,beta,std,parname,tf)
      implicit none
      integer ncol,maxpar,npar(ncol),numpar,match,first,i,j
      double precision beta(maxpar,0:ncol),std(maxpar,0:ncol),
     *   dpnan
      character parname(maxpar,0:ncol)*8
      logical tf(maxpar,ncol),duplicate,isnan
      data duplicate/.false./
 
c     The zero-th column of parname will be comprehensive.  The first
c     file determines the sequence first, etc.
      call copychar8(parname(1,1),parname(1,0),npar(1))
      numpar=npar(1)
 
      do 100 i=2,ncol
c        Use the zero-th column for scratch space:
         call copy8(beta(1,i),beta(1,0),npar(i))
         call copy8(std(1,i),std(1,0),npar(i))
         call put(dpnan(),beta(1,i),maxpar)
         call put(dpnan(),std(1,i),maxpar)
         do 200 j=1,npar(i)
            first=0
  210       call char8match(parname(j,i),parname(first+1,0),
     *         numpar-first,match)
            if(match.eq.0)goto 220
            match=first+match
            if(.not.isnan(beta(match,i)))then
               duplicate=.true.
               first=match
               goto 210
            endif
            beta(match,i)=beta(j,0)
            if(tf(j,i))std(match,i)=std(j,0)
            goto 200
  220       numpar=numpar+1
            if(numpar.gt.maxpar)goto 800
            parname(numpar,0)=parname(j,i)
            beta(numpar,i)=beta(j,0)
            if(tf(j,i))std(numpar,i)=std(j,0)
  200    continue
  100 continue
 
      if(duplicate)write(6,301)
  301 format('WARNING: there are duplicate parameter names.  The ',
     *   'program aligned them',/,'in the order they appeared.  ',
     *   'Please check that this is correct.')
      goto 999
 
  800 call writeaia(6,72,1,'ERROR: the maximum number of distinct '//
     *   'parameters (',maxpar,') is exceeded.  Contact Technical '//
     *   'Support at <support@applied-ml.com> for a complimentary '//
     *   'larger version.')
      stop
  999 return
      end
c=========================================================================
      subroutine writetable(width,ncol,numpar,maxpar,beta,std,head,df,
     *   basis,parname,lnl,comma,strict,inparens)
      implicit none
      integer width,ncol,numpar,maxpar,df(ncol),basis(ncol),inparens,
     *   nf,i,j,k,n,n1,n2,dlen,trm,countint
      double precision beta(maxpar,0:ncol),std(maxpar,0:ncol),
     *   lnl(ncol),t,p,dpvecmin,pvalue
      character head(ncol)*12,parname(numpar)*8,cparens(2:4)*18,
     *   fmt*24,i2c*12
      logical comma,strict,isnan
      data cparens
     *   /'standard errors','t-statistics','two-sided p-values'/
 
      if(comma)then
c        Output should be comma-separated.
c        Write header and a blank line, which is better filled with commas.
         fmt='('//i2c(ncol,0)
         fmt=fmt(1:trm(fmt))//'a,/,'
         fmt=fmt(1:trm(fmt))//i2c(ncol,0)
         fmt=fmt(1:trm(fmt))//'a)'
         write(6,fmt)(','//head(i)(1:trm(head(i))),i=1,ncol),
     *      (',',i=1,ncol)
 
         do 100 i=1,numpar
c           Write parameter name:
            write(6,'(a,$)')parname(i)(1:trm(parname(i)))
            do 110 j=1,ncol
               if(isnan(beta(i,j)))then
                  write(6,'(a,$)')','
               else
                  n=dlen(beta(i,j))+1+width
                  fmt='(a,f'//i2c(n,0)
                  fmt=fmt(1:trm(fmt))//'.'//i2c(width,0)
                  fmt=fmt(1:trm(fmt))//',$)'
                  write(6,fmt)',',beta(i,j)
                  if(inparens.eq.0)goto 110
                  if(isnan(std(i,j)).or.std(i,j).eq.0.d0)goto 110
                  t=abs(beta(i,j)/std(i,j))
c                 p=pvalue(t,df(j))
c                 if(p.le..001d0)then
                  if(t.gt.3.291)then
                     if(strict)write(6,'(a,$)')' ***'
                     if(.not.strict)write(6,'(a,$)')' ***'
c                 elseif(p.le..01d0)then
                  elseif(t.gt.2.576)then
                     if(strict)write(6,'(a,$)')' **'
                     if(.not.strict)write(6,'(a,$)')' ***'
c                 elseif(p.le..05d0)then
                  elseif(t.gt.1.960)then
                     if(strict)write(6,'(a,$)')' *'
                     if(.not.strict)write(6,'(a,$)')' **'
c                 elseif(p.le..1d0)then
                  elseif(t.gt.1.645)then
                     if(.not.strict)write(6,'(a,$)')' *'
                  endif
               endif
  110       continue
            write(6,'(1x)')
 
c           Write standard deviations, t-statistics, or p-values:
c           inparens = 0 = suppress standard errors and significance asterisks
c                      1 = suppress standard errors
c                      2 = standard errors in parentheses (default)
c                      3 = t-statistics in parentheses
c                      4 = p-values in parentheses
            if(inparens.eq.0.or.inparens.eq.1)goto 100
            do 120 j=1,ncol
               if(isnan(std(i,j)))then
                  write(6,'(a,$)')','
               elseif(inparens.eq.2)then
                  n=dlen(std(i,j))+1+width
                  fmt='(a,f'//i2c(n,0)
                  fmt=fmt(1:trm(fmt))//'.'//i2c(width,0)
                  fmt=fmt(1:trm(fmt))//',a1,$)'
                  write(6,fmt)',(',std(i,j),')'
               elseif(std(i,j).eq.0.d0)then
                  fmt='('//i2c(4+width,0)
                  fmt=fmt(1:trm(fmt))//'a,$)'
                  write(6,'(100a,$)')',(',('-',k=1,2+width),')'
               else
                  t=abs(beta(i,j)/std(i,j))
                  if(inparens.eq.3)then
                     n=dlen(t)+1+width
                     fmt='(a,f'//i2c(n,0)
                     fmt=fmt(1:trm(fmt))//'.'//i2c(width,0)
                     fmt=fmt(1:trm(fmt))//',a1,$)'
                     write(6,fmt)',(',t,')'
                  elseif(inparens.eq.4)then
                     p=pvalue(t,df(j))
                     n=dlen(p)+1+width
                     fmt='(a,f'//i2c(n,0)
                     fmt=fmt(1:trm(fmt))//'.'//i2c(width,0)
                     fmt=fmt(1:trm(fmt))//',a1,$)'
                     write(6,fmt)',(',p,')'
                  endif
               endif
  120       continue
            write(6,'(1x)')
  100    continue
c        Write out the log-likelihoods:
         fmt='('//i2c(ncol,0)
         fmt=fmt(1:trm(fmt))//'a,/,a,$)'
         write(6,fmt)(',',i=1,ncol),'ln-L'
         do 130 j=1,ncol
            n=dlen(lnl(j))+3
            fmt='(a,f'//i2c(n,0)
            fmt=fmt(1:trm(fmt))//'.2,$)'
            write(6,fmt)',',lnl(j)
  130    continue
         write(6,'(1x)')
      else
c        Output should be lined up as-if using tabs.
c        Write header:
         fmt='(9x,'//i2c(ncol,0)
         if(inparens.eq.0)then
            fmt=fmt(1:trm(fmt))//'a'//i2c(max(width,2)+6,0)
         else
            fmt=fmt(1:trm(fmt))//'a'//i2c(max(width,2)+10,0)
         endif
         fmt=fmt(1:trm(fmt))//',/)'
         write(6,fmt)head
         do 200 i=1,numpar
c           Write parameter name:
            write(6,'(a8,$)')parname(i)
            nf=0
            do 210 j=1,ncol
               if(isnan(beta(i,j)))then
                  nf=nf+6+width
                  if(inparens.gt.0)nf=nf+4
               else
                  if(j.gt.1)nf=nf+max(0,2-width)
                  fmt='('//i2c(nf,0)
                  fmt=fmt(1:trm(fmt))//'x,f'//i2c(width+6,0)
                  fmt=fmt(1:trm(fmt))//'.'//i2c(width,0)
                  fmt=fmt(1:trm(fmt))//',$)'
                  write(6,fmt)beta(i,j)
                  if(inparens.eq.0)then
                     nf=0
                  elseif(isnan(std(i,j)).or.std(i,j).eq.0.d0)then
                     nf=4
                  else
                     t=abs(beta(i,j)/std(i,j))
c                    p=pvalue(t,df(j))
c                    if(p.le..001d0)then
                     if(t.gt.3.291)then
                        if(strict)write(6,'(a4,$)')' ***'
                        if(.not.strict)write(6,'(a4,$)')' ***'
c                    elseif(p.le..01d0)then
                     elseif(t.gt.2.576)then
                        if(strict)write(6,'(a4,$)')' ** '
                        if(.not.strict)write(6,'(a4,$)')' ***'
c                    elseif(p.le..05d0)then
                     elseif(t.gt.1.960)then
                        if(strict)write(6,'(a4,$)')' *  '
                        if(.not.strict)write(6,'(a4,$)')' ** '
c                    elseif(p.le..1d0)then
                     elseif(t.gt.1.645)then
                        if(.not.strict)write(6,'(a4,$)')' *  '
                     else
                        write(6,'(a4,$)')'    '
                     endif
                     nf=0
                  endif
               endif
  210       continue
            write(6,'(1x)')
 
c           Write standard deviations, t-statistics, or p-values:
c           inparens = 0 = suppress standard errors and significance asterisks
c                      1 = suppress standard errors
c                      2 = standard errors in parentheses (default)
c                      3 = t-statistics in parentheses
c                      4 = p-values in parentheses
            if(inparens.eq.0.or.inparens.eq.1)goto 200
            nf=5
            do 220 j=1,ncol
               if(isnan(std(i,j)))then
                  nf=nf+10+width
               elseif(inparens.eq.2)then
                  n2=dlen(std(i,j))+1+width
                  n1=nf+8+width-n2
                  if(j.gt.1)n1=n1+max(0,2-width)
                  fmt='('//i2c(n1,0)
                  fmt=fmt(1:trm(fmt))//'x,a,f'//i2c(n2,0)
                  fmt=fmt(1:trm(fmt))//'.'//i2c(width,0)
                  fmt=fmt(1:trm(fmt))//',a1,$)'
                  write(6,fmt)'(',std(i,j),')'
                  nf=0
               elseif(std(i,j).eq.0.d0)then
                  n2=2+width
                  n1=nf+8+width-n2
                  if(j.gt.1)n1=n1+max(0,2-width)
                  fmt='('//i2c(n1,0)
                  fmt=fmt(1:trm(fmt))//'x,'//i2c(n2+2,0)
                  fmt=fmt(1:trm(fmt))//'a,$)'
                  write(6,fmt)'(',('-',k=1,2+width),')'
                  nf=0
               else
                  t=abs(beta(i,j)/std(i,j))
                  if(inparens.eq.3)then
                     n2=dlen(t)+1+width
                     n1=nf+8+width-n2
                     if(j.gt.1)n1=n1+max(0,2-width)
                     fmt='('//i2c(n1,0)
                     fmt=fmt(1:trm(fmt))//'x,a,f'//i2c(n2,0)
                     fmt=fmt(1:trm(fmt))//'.'//i2c(width,0)
                     fmt=fmt(1:trm(fmt))//',a1,$)'
                     write(6,fmt)'(',t,')'
                     nf=0
                  elseif(inparens.eq.4)then
                     p=pvalue(t,df(j))
                     n2=dlen(p)+1+width
                     n1=nf+8+width-n2
                     if(j.gt.1)n1=n1+max(0,2-width)
                     fmt='('//i2c(n1,0)
                     fmt=fmt(1:trm(fmt))//'x,a,f'//i2c(n2,0)
                     fmt=fmt(1:trm(fmt))//'.'//i2c(width,0)
                     fmt=fmt(1:trm(fmt))//',a1,$)'
                     write(6,fmt)'(',p,')'
                     nf=0
                  endif
               endif
  220       continue
            write(6,'(1x)')
  200    continue
 
c        Write out the log-likelihoods:
         if(inparens.eq.0)then
            n1=max(width,2)+6
            n1=max(n1,dlen(dpvecmin(lnl,ncol))+4)
            fmt='(/,a4,4x,'//i2c(ncol,0)
            fmt=fmt(1:trm(fmt))//'f'//i2c(n1,0)
            fmt=fmt(1:trm(fmt))//'.2)'
         else
            n1=max(width,2)+10
            n1=max(n1,dlen(dpvecmin(lnl,ncol))+4)
            fmt='(/,a4,1x,'//i2c(ncol,0)
            fmt=fmt(1:trm(fmt))//'f'//i2c(n1,0)
            fmt=fmt(1:trm(fmt))//'.2)'
         endif
         write(6,fmt)'ln-L',lnl
      endif
 
      if(inparens.eq.0)then
         goto 999
      elseif(inparens.eq.1)then
         write(6,'(/,a)')'NOTE:  Standard errors suppressed;'
      else
         write(6,301)cparens(inparens)(1:trm(cparens(inparens)))
      endif
      if(strict)then
         write(6,302)
      else
         write(6,303)
      endif
  301 format(/,'NOTE:  Asymptotic ',a,' in parentheses;')
  302 format(7x,'Significance: ''*''=5%;  ''**''=1%;  ''***''=0.1%.')
  303 format(7x,'Significance: ''*''=10%;  ''**''=5%;  ''***''=1%.')
 
      if(countint(3,basis,ncol).eq.ncol)then
c        All BHHH-based; don't mention it.
         goto 999
      elseif(countint(1,basis,ncol).eq.ncol)then
         write(6,304)cparens(inparens)(1:trm(cparens(inparens)))
  304    format(/,'NOTE:  All ',a,' are Huber-corrected.')
      elseif(countint(2,basis,ncol).eq.ncol)then
         write(6,305)cparens(inparens)(1:trm(cparens(inparens)))
  305    format(/,'NOTE:  All ',a,' are based on numerical Hessians.')
      else
         write(6,306)cparens(inparens)(1:trm(cparens(inparens)))
  306    format(/,'NOTE:  The ',a,' are not all consistently computed:')
         do 310 i=1,ncol
            if(basis(i).eq.1)then
               write(6,307)i,head(i)(1:trm(head(i)))
  307          format(7x,'Column',i2,' (',a,') is Huber-corrected')
            elseif(basis(i).eq.2)then
               write(6,308)i,head(i)(1:trm(head(i)))
  308          format(7x,'Column',i2,' (',a,') is based on a ',
     *            'numerical Hessian')
            elseif(basis(i).eq.3)then
               write(6,309)i,head(i)(1:trm(head(i)))
  309          format(7x,'Column',i2,' (',a,') is BHHH-based')
            endif
  310    continue
      endif
 
  999 return
      end
c=========================================================================
      double precision function pvalue(t,df)
 
c     Double precision function to compute a two-sided p-value.
c
c     BUG: only implemented on Digital platform.
c
c     Constantijn Panis, 19 November 2001
 
c     USE numerical_libraries
      implicit none
      double precision t
      integer df
 
c     pvalue=2.d0*(1.d0-dtdf(abs(t),dble(df)))
      call bug(0,1)
 
      return
      end
