      program update
 
c     Program to update control cards with estimated values.
c     Usage:  "update file"
c     This will update file.aml with the values in file.out.
c     Multiple control files may be specified at once.
c
c     Constantijn Panis, 11 April 2002
c     Last change: Guard against read-only files.
 
      implicit none
      integer maxpar,narg,npar,round
      parameter(maxpar=500)
      double precision beta(maxpar)
c     integer(2) i
      integer i
      character stem*128,parname(maxpar)*8
      logical tf(maxpar),force
      data round,force/1,.false./
 
      call getforce(narg,force)
c     Check that files exist and are not read-only.
      do 100 i=1,narg
         if(force.and.i.eq.1)goto 100
         call checkfile(i,stem)
  100 continue
c     Update files
      do 200 i=1,narg
         if(force.and.i.eq.1)goto 200
         call openfiles(i,stem)
         call readout(stem,maxpar,npar,round,parname,beta,tf,force)
         call writeout(npar,round,beta,stem,parname,tf)
  200 continue
 
  999 continue
      end
c=========================================================================
      subroutine getforce(narg,force)
      implicit none
      integer narg,iargc
c     integer(2) i
      integer i
      character argument*128
      logical force
 
      narg=iargc()
      if(narg.eq.0)goto 800
      i=1
      call getarg(i,argument)
      force=(argument.eq.'-f')
      goto 999
 
  800 write(6,801)
  801 format(/,'Usage: "update [-f] file [...]"',/,
     *   'to update ''file.aml'' with the values found in ',
     *   '''file.out''.',/,'Option -f forces an update, even if ',
     *   'the estimation did not converge.')
      stop
  999 return
      end
c=========================================================================
      subroutine checkfile(i,stem)
      implicit none
      integer nt,trm,readonly
c     integer(2) i
      integer i
      character stem*128
      logical exist
 
      call getarg(i,stem)
      nt=trm(stem)
 
      if(nt.gt.4.and.stem(nt-3:nt).eq.'.aml')then
         if(.not.exist(stem(1:nt-4)//'.out'))goto 810
         if(.not.exist(stem(1:nt)))goto 820
         open(13,file=stem(1:nt))
         if(readonly(13).eq.1)goto 830
         close(13)
      else
         if(.not.exist(stem(1:nt)//'.out'))goto 840
         if(.not.exist(stem(1:nt)//'.aml'))goto 850
         open(13,file=stem(1:nt)//'.aml')
         if(readonly(13).eq.1)goto 860
         close(13)
      endif
      goto 999
 
  810 call not_found(6,stem(1:nt)//'.out')
      stop
  820 call not_found(6,stem(1:nt))
      stop
  830 call wrapstring(6,0,72,0,1,'ERROR:  the existing '''//
     *   stem(1:nt)//''' is read-only.  Please delete or rename it.')
      stop
  840 call not_found(6,stem(1:nt)//'.out')
      stop
  850 call not_found(6,stem(1:nt)//'.aml')
      stop
  860 call wrapstring(6,0,72,0,1,'ERROR:  the existing '''//
     *   stem(1:nt)//'.aml'' is read-only.  Please delete or '//
     *   'rename it.')
      stop
  999 return
      end
c=========================================================================
      subroutine openfiles(i,stem)
      implicit none
      integer nt,trm
c     integer(2) i
      integer i
      character stem*128
      logical opened
 
      call getarg(i,stem)
      nt=trm(stem)
 
      if(opened(12))close(12)
      if(opened(13))close(13)
 
c     Checkfile guarantees that the .aml and .out files exist and that the
c     .aml file is not read-only.
      if(nt.gt.4.and.stem(nt-3:nt).eq.'.aml')then
         open(12,file=stem(1:nt-4)//'.out')
c        Make a backup copy of the old control file into stem.bak:
         call backup(1,stem(1:nt))
         open(13,file=stem(1:nt))
      else
         open(12,file=stem(1:nt)//'.out')
         call backup(1,stem(1:nt)//'.aml')
         open(13,file=stem(1:nt)//'.aml')
      endif
 
      return
      end
c=======================================================================
      subroutine readout(stem,maxpar,npar,round,parname,beta,tf,force)
      implicit none
      integer maxpar,npar,round,i,ii,i1,i2,il,status,trm
      double precision beta(maxpar)
      character stem*128,line*128,parname(maxpar)*8
      logical converged,tf(maxpar),force,ask_yes
      data converged/.false./
 
c     First read until  "Number of parameters in model:"
  100 read(12,'(a80)',err=800,end=800)line
      if(line(1:30).ne.'Number of parameters in model:')goto 100
      backspace 12
      read(12,'(35x,i4)',err=800,end=800)npar
      if(npar.eq.0)goto 800
      if(npar.gt.maxpar)goto 810
 
c     There may be multiple rounds of optimization.  Report them as they
c     appear.  Ultimately, scan for "Log Likelihood", and check for
c     convergence.
      round=0
  200 read(12,'(a80)',err=800,end=800)line
 
c...+....1....+....2....+....3....+....4....+....5....+....6....+....7....+....8
c                   ROUND 1 CONVERGED SUCCESSFULLY                   =
c                 ESTIMATION CONVERGED SUCCESSFULLY                  =
c                   ESTIMATION FAILED TO CONVERGE                    =
 
      if(line(21:25).eq.'ROUND'.and.
     *   line(29:50).eq.'CONVERGED SUCCESSFULLY')then
         round=round+1
         converged=.true.
         write(6,201)round
  201    format('...estimation round',i2,' converged...')
      elseif(line(21:49).eq.'ESTIMATION FAILED TO CONVERGE')then
         round=round+1
         converged=.false.
      elseif(line(19:51).eq.'ESTIMATION CONVERGED SUCCESSFULLY')then
         if(round.eq.0)round=1
         converged=.true.
      endif
      if(line(1:15).ne.'Log Likelihood:')goto 200
 
c     If not converged, update anyway??
      if(.not.converged)then
         if(force)then
            if(round.eq.1)write(6,101)stem(1:trm(stem))
            if(round.gt.1)write(6,103)stem(1:trm(stem)),round
         else
            if(round.eq.1)write(6,102)stem(1:trm(stem))
            if(round.gt.1)write(6,104)stem(1:trm(stem)),round
            if(.not.ask_yes('update anyway?'))stop
         endif
      endif
  101 format('WARNING: ''',a,''' did not converge; update is forced.')
  102 format('WARNING: ''',a,''' did not converge; ',$)
  103 format('WARNING: ''',a,''', round',i2,' did not converge; ',
     *   'update is forced.')
  104 format('WARNING: ''',a,''', round',i2,' did not converge; ',$)
 
c     Read in parameter values
      read(12,'(a80)',err=800,end=800)(line,i=1,3)
      do 220 i=1,npar
         read(12,'(a80)',err=800,end=800)line
         il=1
         call getint(12,ii,line,il,status)
         if(status.gt.0)goto 800
         if(i.ne.ii)goto 800
         call getstring(12,line,il,i1,i2)
         if(i2.eq.-1)goto 800
         parname(i)=line(i1:i2)
         call getstring(12,line,il,i1,i2)
         if(i2.eq.-1)goto 800
         tf(i)=(line(i1:i2).eq.'T')
         call getdp(12,beta(i),line,il,status)
         if(status.gt.0)goto 800
  220 continue
      read(12,'(a80)',err=800,end=800)line
      if(line.ne.' ')goto 800
      goto 999
 
  800 write(6,801)stem(1:trm(stem))
  801 format('ERROR or END-OF-FILE encountered while reading ''',
     *   a,'.out'';',/,6x,'the control file was not updated.')
      stop
  810 call writeaia(6,72,1,'ERROR: maximum number of parameters (',
     *   maxpar,') is exceeded.  Remedy: increase parameter maxpar '//
     *   'and recompile.')
      stop
  999 return
      end
c=========================================================================
      subroutine writeout(npar,round,beta,stem,parname,tf)
      implicit none
      integer npar,round,i,il,trm
      double precision beta(npar)
      character stem*128,line*128,parname(npar)*8,varname*8
      logical tf(npar),smtxt
 
c     Find begin position of old starting values in the output file:
  100 read(13,'(a80)',end=800,err=800)line
      il=1
      if(.not.smtxt(13,line,il,'STARTING VALUES;'))goto 100
c     check that following line contains correct variable name:
      read(13,*,end=800,err=800)varname
      if(parname(1).ne.varname)goto 820
      backspace 13
 
c     Write new values into control file:
      do 110 i=1,npar
         write(13,'(a8,l5,$)',err=810)parname(i),tf(i)
         call writedble(13,beta(i),-4,13,0,1)
  110 continue
      write(13,'(a)')';'
      if(round.eq.1)call writeaia(6,0,1,''''//stem(1:trm(stem))//
     *   '.aml'' has been updated (',npar,' parameters)')
      if(round.gt.1)call writea2ia(6,0,1,''''//stem(1:trm(stem))//
     *   '.aml'' has been updated with results from round ',round,' (',
     *   npar,' parameters)')
      goto 999
 
  800 write(6,801)stem(1:trm(stem))
  801 format('ERROR or END-OF-FILE encountered while reading ''',
     *   a,'.aml'';',/,'the control file was not updated.')
      stop
  810 write(6,811)stem(1:trm(stem))//'.aml'
  811 format(/,'ERROR: unable to update ''',a,'''.  ',
     *   'Is the file read-only?')
      stop
  820 write(6,821)stem(1:trm(stem))//'.aml',stem(1:trm(stem))//'.out'
  821 format('ERROR encountered while comparing the first variable ',
     *   'names of ''',a,'''',/,'and ''',a,'''; the control file ',
     *   'was not updated.')
      stop
  999 return
      end
