      subroutine outputfile(datfile,warn,r2afile)
 
c     Subroutine to determine the names of the output data file (.dat)
c     and the summary statistics file (.sum).  Connects the data
c     file to unit 13 and the summary file to unit 8.  If the summary
c     file already exists, a backup is made.
c
c     There are three ways to specify the output data file.
c     (1) Command line option "-o filename"
c     (2) Control file statement "OPTION OUTPUT DATA FILE = filename;"
c         Note: for backward compatibility, keyword OPTION may be omitted.
c     (3) Filename has same stem as control file name (default).
c
c     The data file has extension .dat unless the user specifies another
c     extension.  The corresponding summary statistics file has extension
c     .sum.  This extension replaces .dat unless the user specified
c     another data file name extension, in which case the .sum will be
c     appended.
c
c     If the data file already exists, a question to the user through
c     stdout asks for permission to overwrite.  The data file will also
c     be overwritten (without question) if command line option -r is
c     specified or "replace" appears after the data file statement
c     in the control file.  No backup is made.
c
c     This routine also opens the summary statistics file, filename.sum.
c     If it already exists, a backup is made.
c
c     Constantijn Panis, 24 January 2000
c     Last change: add syntax help.  Change replace syntax.
 
      implicit none
      integer warn,il,i1,i2,nt,match,dot,status,trm,readonly
      logical exist,smtxt
      character datfile*128,r2afile*128,line*128,sumfile*128
 
c     (1) Command line option "-o filename"
      if(datfile.ne.' ')goto 300
 
c     (2) Control file statement "[OPTION] OUTPUT DATA FILE = filename;"
c     Scan the control file.
      read(4,'(a80)',end=800,err=800)line
      il=1
  100 if(smtxt(4,line,il,'OUTPUT DATA FILE=').or.
     *   smtxt(4,line,il,'OPTION OUTPUT DATA FILE='))then
         call getstring(4,line,il,i1,i2)
         if(i2.eq.-1)goto 810
c        If dsn(replace=..) was specified, without space between dsn and (,
c        entire string may have been read as dsn.  Let's not allow parentheses
c        in data set names.
         call char1match('(',line(i1:i2),i2-i1+1,match)
         if(match.gt.0)then
            if(match.eq.1)goto 810
            i2=i1+match-2
            il=i2+1
         endif
         datfile=line(i1:i2)
         if(smtxt(4,line,il,'(REPLACE'))then
            if(smtxt(4,line,il,')').or.smtxt(4,line,il,'=YES)'))then
               warn=0
            elseif(smtxt(4,line,il,'=NO)'))then
               warn=1
            else
               goto 810
            endif
         endif
         if(.not.smtxt(4,line,il,';'))goto 810
      else
         call gotochar(4,';',line,il,status)
         if(status.gt.0)goto 200
         goto 100
      endif
      goto 300
 
c     (3) Filename has same stem as control file name (default).
  200 nt=trm(r2afile)
      if(r2afile(nt-3:nt).eq.'.r2a')then
         datfile=r2afile(1:nt-3)//'dat'
      elseif(r2afile(nt-3:nt).eq.'.R2A')then
         datfile=r2afile(1:nt-3)//'DAT'
      else
         datfile=r2afile(1:nt)//'.dat'
      endif
      goto 300
 
c     Name (or stem of name) of data output file is in datfile.
  300 rewind 4
      nt=trm(datfile)
c     We would like extension .dat; add it if the user did not specify
c     any extension.
      do 310 dot=nt,1,-1
         if(datfile(dot:dot).eq.'.')goto 330
         if(datfile(dot:dot).eq.'/'.or.datfile(dot:dot).eq.'\\')goto 320
c        if(datfile(dot:dot).eq.'/'.or.datfile(dot:dot).eq.'\')goto 320
  310 continue
  320 dot=0
  330 if(dot.eq.0)then
         datfile=datfile(1:nt)//'.dat'
         sumfile=datfile(1:nt)//'.sum'
      else
c        Try replacing the data file's extension by .sum for the
c        summary file, but only if the extension is .dat.
         if(dot.eq.nt)then
c           Respect express wish to have no extension:
            sumfile=datfile(1:dot)//'sum'
            datfile(nt:nt)=' '
            nt=nt-1
         elseif(datfile(dot:nt).eq.'.dat'.or.
     *          datfile(dot:nt).eq.'.DAT')then
            sumfile=datfile(1:dot)//'sum'
         else
            sumfile=datfile(1:nt)//'.sum'
         endif
      endif
 
c     Open the data file, perhaps with a warning:
      if(exist(datfile).and.warn.eq.1)call are_you_sure(datfile)
      if(exist(sumfile))call backup(2,sumfile)
 
      open(13,file=datfile,form='unformatted',err=830)
      open(8,file=sumfile,err=830)
      if(readonly(13).eq.1)goto 840
      if(readonly(8).eq.1)goto 850
      goto 999
 
  800 call caretline(6,line,il)
      write(6,801)
  801 format('ERROR or END-OF-FILE encountered in the control file.')
      stop
  810 call caretline(6,line,il)
      write(6,811)
  811 format('ERROR in syntax of output data file specification.  ',
     *   'The syntax is:',/,3x,'OPTION OUTPUT DATA FILE = ',
     *   'filename.dat [(REPLACE)];')
      stop
  830 call wrapstring(6,0,72,0,1,'ERROR opening "'//
     *   datfile(1:trm(datfile))//
     *   '"; you probably do not have write permission.')
      stop
  840 call wrapstring(6,0,72,0,1,'ERROR:  the existing '''//
     *   datfile(1:trm(datfile))//''' is read-only.  '//
     *   'Please delete or rename it.')
      stop
  850 call wrapstring(6,0,72,0,1,'ERROR:  the existing '''//
     *   sumfile(1:trm(sumfile))//''' is read-only.  '//
     *   'Please delete or rename it.')
      stop
  999 return
      end
