      subroutine varlist(numvar,nvar,nx,deduct,name,line,il)
 
c     Subroutine to read in a variable list.  Integer nx is incremented
c     by one for each variable name.  It is initialized to zero in this
c     routine.
c
c     Variable names must start with a letter (names starting with
c     an underscore are reserved) and may contain only letters,
c     numbers, and underscores.  Names 'and' 'or' 'uniform' are reserved.
c     Names may at most be 8 characters long.
c
c     Names may be followed by something in parentheses:
c        name(a)  variable must remain in absolute value, even if
c                 option deviations=yes;  is turned on.
c        name(d)  variable must be converted into deviations from its mean.
c        name(d=x) variable must be converted in deviations from x, where
c                 x is a real number.
c
c     Constantijn Panis, 20 July 1999
 
      implicit none
      integer numvar,nvar,nx,il,i1,i2,status
      real deduct(numvar),deductflag
      logical smtxt
      character name(numvar)*8,line*128
 
      nx=0
  100 if(smtxt(4,line,il,';'))goto 999
      call getname(4,line,il,i1,i2)
c     Variable syntax and naming rule checks were performed in countvars
c     (called by scanr2a)
      nx=nx+1
      nvar=nvar+1
      if(nvar.gt.numvar)call bug(1,1)
      name(nvar)=line(i1:i2)
      if(.not.smtxt(4,line,il,'('))goto 100
      if(smtxt(4,line,il,'A)'))then
c        This variable in absolute value
         deduct(nvar) = 0.
      elseif(smtxt(4,line,il,'D)'))then
c        This variable in deviations from its mean -- flag this
         deduct(nvar) = deductflag()
      elseif(smtxt(4,line,il,'D='))then
c        This variable in deviations from user-specified constant
         call getreal(4,deduct(nvar),line,il,status)
         if(status.gt.0)goto 800
         if(.not.smtxt(4,line,il,')'))goto 800
      else
         goto 800
      endif
      goto 100
 
  800 call caretline(8,line,il)
      write(8,801)
      call caretline(6,line,il)
      write(6,801)
  801 format(/,'ERROR encountered while reading a variable list.  ',
     *   'The syntax for',/,'transforming variables into deviations ',
     *   'is:',//,3x,'varname(a)   variable remains in absolute ',
     *   'value, even if',/,16x,'option "deviations=yes;" is ',
     *   'turned on;',/,3x,'varname(d)   variable is converted into ',
     *   'deviations from its mean;',/,3x,'varname(d=x) variable is ',
     *   'converted into deviations from x, where',/,
     *   16x,'x is a real number.')
      stop
  999 return
      end
