      subroutine amlfunc(task,maxcc,ccmax,nfull,nfree,fullbeta,fulltype,
     *   iffull,lnlall,nchol,chol,dfreebeta,fp,fp2,fpt,fpt2,sp,spt,
     *   locdim,dd,rr,ii,ll,cc,ipt,jpt,warn)
 
c     Subroutine to compute log likelihood, first and second derivatives
c     of model aML for applied Maximum Likelihood.
c
c     Argument task indicates what will be computed:
c        100 = function value
c         20 = first derivatives  (untransformed parameters only)
c         10 = first derivatives  (transformed and untransformed parameters)
c          4 = numeric second derivatives (untransformed parameters only)
c          3 = numeric second derivatives (transformed and untransformed paramet
c          2 = BHHH second derivatives (untransformed parameters only)
c          1 = BHHH second derivatives (transformed and untransformed parameters
c
c     If task<0, abs(task) is applied and no attempt is made to salvage a
c     function evaluation.  As of version 3.01 (August 2002), this only applies
c     to grid searches.  task<0 overrules "option ensure positive definite".
c
c     For example, task=110 means function and first derivatives only.
c
c     Note:  if a certain derivative is not computed, no space need be
c     allocated.  There is one exception:  if numerical second derivatives
c     are required, fp2 and/or fpt2 are used as scratch array.
c
c     Issue: numerical Hessian is often not negative definite.
c
c     Constantijn Panis,  2 December 2002
c     Last change: version
 
      implicit none
      integer maxcc,ccmax
      double precision dd(maxcc/8)
      real rr(maxcc/4)
      integer ii(maxcc/4),locdim(120)
      logical ll(maxcc/4)
      character cc(maxcc)*1
      integer task,nfull,nfree,fulltype(nfree),iffull(nfull),
     *   nchol,ipt,jpt,warn
      double precision fullbeta(nfull),lnlall,chol(nchol),
     *   dfreebeta(nfree),fp(nfree),fp2(nfree),fpt(nfree),fpt2(nfree),
     *   sp(nfree,nfree),spt(nfree,nfree)
c     Integer values and pointers to arrays:
      integer nfullplus,scrnobs,maxpnts,numspl,maxnest,numdist,
     *   numarma,sumdim,sumintdim,sumdim2,sumdim4,vdimmax,nihmax,nemax,
     *   nehmax,nelmax,nepmax,necmax,nebmax,nenmax,nepomax,nemlmax,
     *   nnemlmax,maxxnode,maxnmapmoel,maxds,nngammax,rsh3max,rsh4max,
     *   rsh5max,rsnamax,rsnpmax,scrdim,rspmax,rscmax,rsmlmax,allint6
      integer z,wt,weight,scaleweight,xnode,v,intpnts,intpntr,
     *   dimused,mapmoel,nested,distpos,distdim,ndraw,distyp,lamtyp,
     *   distrestr,distrestrpos,numpar,pardim,parpos,parname,fullname,
     *   refspl,beginspl,nodepos,numnodes,spltype,iflam,ifint4,ifint6,
     *   iftau,iid4,posdef
      integer like,lnl,dfullbeta,ddfullbeta,
     *   lnla,lnlb,xbh3,xbh4,xbh5,d1xbh3,d2xbh3,gamma,d1gamma,
     *   d2gamma,s,ds,ssg,dlnla,dlnlb,ui,ddui,sigdd,
     *   dsigdd,stda4,cdm,ddcdm,dcdmdsigii,dxbh,ddxbh,xblp,
     *   dxblp,ddxblp,xbc,dxbc,ddxbc,a5,da5,dda5,p5,dp5,ddp5,
     *   xbml,dxbml,ddxbml,indx,lampos,bdc,thisdraw,intdist,draw,cdraw
      integer abstask,newtask,i,j,k
      double precision save,delta,jacobeps
 
c     First get all locations and dimensions:
      call mklocdim(2,maxcc,ccmax,locdim,nfullplus,ipt,jpt,
     *   scrnobs,maxpnts,numspl,numdist,maxnest,numarma,sumdim,
     *   sumintdim,sumdim2,sumdim4,vdimmax,nihmax,nemax,nehmax,nelmax,
     *   nepmax,necmax,nebmax,nenmax,nepomax,nemlmax,nnemlmax,
     *   maxxnode,maxnmapmoel,maxds,nngammax,rsh3max,rsh4max,rsh5max,
     *   rsnamax,rsnpmax,scrdim,rspmax,rscmax,rsmlmax,
     *   allint6,z,wt,weight,scaleweight,xnode,v,intpnts,intpntr,
     *   dimused,mapmoel,nested,distpos,distdim,ndraw,distyp,lamtyp,
     *   distrestr,distrestrpos,numpar,pardim,parpos,parname,fullname,
     *   refspl,beginspl,nodepos,numnodes,spltype,iflam,ifint4,ifint6,
     *   iftau,iid4,posdef,like,lnl,dfullbeta,ddfullbeta,lnla,lnlb,xbh3,
     *   xbh4,xbh5,d1xbh3,d2xbh3,gamma,d1gamma,d2gamma,s,ds,ssg,dlnla,
     *   dlnlb,ui,ddui,sigdd,dsigdd,stda4,cdm,ddcdm,dcdmdsigii,dxbh,
     *   ddxbh,xblp,dxblp,ddxblp,xbc,dxbc,ddxbc,a5,da5,dda5,p5,dp5,ddp5,
     *   xbml,dxbml,ddxbml,indx,lampos,bdc,thisdraw,intdist,draw,cdraw)
 
c     If posdef=0, non-positive definite covariance matrices are left alone
c     by subroutine ensposdef.
      if(task.lt.0)posdef=0
      abstask=abs(task)
 
c     Allow for numerical second derivatives.
      if(mod(abstask,10).le.2)then
c        Nothing special; no numerical second derivatives:
         newtask=abstask
      else
c        Numerical derivatives are required.
         if(mod(abstask,100).ne.24.and.mod(abstask,100).ne.13)goto 800
c        First compute function at input parameter value, asking only for
c        analytical first derivatives.  Then loop over free parameters to
c        compute Hessian as numerical Jacobian of the analytical Jacobian.
         newtask=abstask-mod(abstask,10)
      endif
 
      call amlfunc_(newtask,maxcc,ccmax,dd,nfull,nfullplus,nfree,
     *   fullbeta,fulltype,iffull,lnlall,nchol,chol,dfreebeta,fp,
     *   fpt,sp,spt,ipt,jpt,scrnobs,maxpnts,numspl,maxnest,numdist,
     *   numarma,sumintdim,sumdim2,sumdim4,numpar,vdimmax,nihmax,nemax,
     *   nehmax,nelmax,nepmax,necmax,nebmax,nenmax,nepomax,nemlmax,
     *   nnemlmax,maxxnode,maxnmapmoel,maxds,nngammax,rsh3max,rsh4max,
     *   rsh5max,rsnamax,rsnpmax,rspmax,rscmax,rsmlmax,allint6,dd(z),
     *   dd(wt),dd(weight),rr(scaleweight),rr(xnode),rr(v),ii(intpnts),
     *   ii(intpntr),ii(dimused),ii(mapmoel),ii(nested),ii(distpos),
     *   ii(distdim),ii(ndraw),ii(distyp),ii(lamtyp),ii(distrestr),
     *   ii(distrestrpos),ii(pardim),ii(parpos),ii(refspl),ii(beginspl),
     *   ii(nodepos),ii(numnodes),ii(spltype),ii(ifint4),ii(ifint6),
     *   ll(iflam),ll(iftau),ll(iid4),posdef,dd(like),dd(lnl),
     *   dd(dfullbeta),dd(ddfullbeta),dd(lnla),dd(lnlb),dd(xbh3),
     *   dd(xbh4),dd(xbh5),dd(d1xbh3),dd(d2xbh3),dd(gamma),dd(d1gamma),
     *   dd(d2gamma),dd(s),dd(ds),dd(ssg),dd(dlnla),dd(dlnlb),dd(ui),
     *   dd(ddui),dd(sigdd),dd(dsigdd),dd(stda4),dd(cdm),dd(ddcdm),
     *   dd(dcdmdsigii),dd(dxbh),dd(ddxbh),dd(xblp),dd(dxblp),
     *   dd(ddxblp),dd(xbc),dd(dxbc),dd(ddxbc),dd(a5),dd(da5),dd(dda5),
     *   dd(p5),dd(dp5),dd(ddp5),dd(xbml),dd(dxbml),dd(ddxbml),
     *   cc(parname),cc(fullname),ii(indx),ii(lampos),ii(bdc),
     *   ii(thisdraw),ii(intdist),ii(draw),ii(cdraw),warn)
 
      if(mod(abstask,10).le.2)goto 999
c     No more need for function value, just analytical derivatives:
      newtask=mod(newtask,100)
      j=0
      do 100 i=1,nfull
         if(iffull(i).eq.0)goto 100
         j=j+1
         save=fullbeta(i)
         delta = jacobeps(fullbeta(i))
         fullbeta(i)=fullbeta(i)+delta
      call amlfunc_(newtask,maxcc,ccmax,dd,nfull,nfullplus,nfree,
     *   fullbeta,fulltype,iffull,lnlall,nchol,chol,dfreebeta,fp2,fpt2,
     *   0.d0,0.d0,ipt,jpt,scrnobs,maxpnts,numspl,maxnest,numdist,
     *   numarma,sumintdim,sumdim2,sumdim4,numpar,vdimmax,nihmax,nemax,
     *   nehmax,nelmax,nepmax,necmax,nebmax,nenmax,nepomax,nemlmax,
     *   nnemlmax,maxxnode,maxnmapmoel,maxds,nngammax,rsh3max,rsh4max,
     *   rsh5max,rsnamax,rsnpmax,rspmax,rscmax,rsmlmax,allint6,dd(z),
     *   dd(wt),dd(weight),rr(scaleweight),rr(xnode),rr(v),ii(intpnts),
     *   ii(intpntr),ii(dimused),ii(mapmoel),ii(nested),ii(distpos),
     *   ii(distdim),ii(ndraw),ii(distyp),ii(lamtyp),ii(distrestr),
     *   ii(distrestrpos),ii(pardim),ii(parpos),ii(refspl),ii(beginspl),
     *   ii(nodepos),ii(numnodes),ii(spltype),ii(ifint4),ii(ifint6),
     *   ll(iflam),ll(iftau),ll(iid4),posdef,dd(like),dd(lnl),
     *   dd(dfullbeta),dd(ddfullbeta),dd(lnla),dd(lnlb),dd(xbh3),
     *   dd(xbh4),dd(xbh5),dd(d1xbh3),dd(d2xbh3),dd(gamma),dd(d1gamma),
     *   dd(d2gamma),dd(s),dd(ds),dd(ssg),dd(dlnla),dd(dlnlb),dd(ui),
     *   dd(ddui),dd(sigdd),dd(dsigdd),dd(stda4),dd(cdm),dd(ddcdm),
     *   dd(dcdmdsigii),dd(dxbh),dd(ddxbh),dd(xblp),dd(dxblp),
     *   dd(ddxblp),dd(xbc),dd(dxbc),dd(ddxbc),dd(a5),dd(da5),dd(dda5),
     *   dd(p5),dd(dp5),dd(ddp5),dd(xbml),dd(dxbml),dd(ddxbml),
     *   cc(parname),cc(fullname),ii(indx),ii(lampos),ii(bdc),
     *   ii(thisdraw),ii(intdist),ii(draw),ii(cdraw),0)
         do 110 k=1,nfree
            sp(j,k)=(fp2(k)-fp(k))/delta
            if(mod(abstask,10).eq.3)spt(j,k)=(fpt2(k)-fpt(k))/delta
  110    continue
         fullbeta(i)=save
  100 continue
      call makesym(nfree,sp)
      if(mod(abstask,10).eq.3)call makesym(nfree,spt)
      goto 999
 
  800 call bug(ipt,jpt)
  999 return
      end
c=====================================================================
      subroutine amlfunc_(task,maxcc,ccmax,dd,nfull,nfullplus,nfree,
     *   fullbeta,fulltype,iffull,lnlall,nchol,chol,dfreebeta,fp,fpt,sp,
     *   spt,ipt,jpt,scrnobs,maxpnts,numspl,maxnest,numdist,numarma,
     *   sumintdim,sumdim2,sumdim4,numpar,vdimmax,nihmax,nemax,nehmax,
     *   nelmax,nepmax,necmax,nebmax,nenmax,nepomax,nemlmax,nnemlmax,
     *   maxxnode,maxnmapmoel,maxds,nngammax,rsh3max,rsh4max,rsh5max,
     *   rsnamax,rsnpmax,rspmax,rscmax,rsmlmax,allint6,z,wt,weight,
     *   scaleweight,xnode,v,intpnts,intpntr,dimused,mapmoel,nested,
     *   distpos,distdim,ndraw,distyp,lamtyp,distrestr,distrestrpos,
     *   pardim,parpos,refspl,beginspl,nodepos,numnodes,spltype,ifint4,
     *   ifint6,iflam,iftau,iid4,posdef,like,lnl,dfullbeta,ddfullbeta,
     *   lnla,lnlb,xbh3,xbh4,xbh5,d1xbh3,d2xbh3,gamma,d1gamma,d2gamma,s,
     *   ds,ssg,dlnla,dlnlb,ui,ddui,sigdd,dsigdd,stda4,cdm,ddcdm,
     *   dcdmdsigii,dxbh,ddxbh,xblp,dxblp,ddxblp,xbc,dxbc,ddxbc,a5,da5,
     *   dda5,p5,dp5,ddp5,xbml,dxbml,ddxbml,parname,fullname,indx,
     *   lampos,bdc,thisdraw,intdist,draw,cdraw,warn)
 
      implicit none
      integer task,maxcc,ccmax,nfull,nfullplus,nfree,fulltype(nfull),
     *   iffull(nfull),nchol
      double precision dd(maxcc/8)
      double precision fullbeta(nfullplus),lnlall,chol(nchol),
     *   dfreebeta(nfree),fp(nfree),fpt(nfree),sp(nfree,nfree),
     *   spt(nfree,nfree)
      integer ipt,jpt,warn
 
c     The following info came from locdim (subroutine mklocdim):
      integer scrnobs,maxpnts,numspl,maxnest,numdist,numarma,sumintdim,
     *   sumdim2,sumdim4,numpar,vdimmax,nihmax,nemax,nehmax,nelmax,
     *   nepmax,necmax,nebmax,nenmax,nepomax,nemlmax,nnemlmax,maxxnode,
     *   maxnmapmoel,maxds,nngammax,rsh3max,rsh4max,rsh5max,rsnamax,
     *   rsnpmax,rspmax,rscmax,rsmlmax,allint6
      double precision z(maxpnts),wt(maxpnts),weight(maxnest)
      real scaleweight,xnode(maxxnode),v(vdimmax)
      integer intpnts(sumintdim),intpntr(sumintdim),dimused(sumintdim),
     *   mapmoel(maxnmapmoel),nested(numdist,2),distpos(numdist),
     *   distdim(numdist),ndraw(numdist),distyp(numdist),
     *   lamtyp(numdist,8),distrestr(0:sumdim2),distrestrpos(numdist),
     *   pardim(numpar),parpos(numpar),refspl(numspl),beginspl(numspl),
     *   nodepos(numspl),numnodes(numspl),spltype(numspl),
     *   ifint4(numdist),ifint6,posdef
      logical iflam(numdist),iftau,iid4
      double precision like(maxnest),lnl(maxnest),
     *   dfullbeta(nfullplus,0:maxnest),ddfullbeta(nfullplus,maxnest),
     *   lnla(nehmax),lnlb(nehmax),
     *   xbh3(nihmax,0:rsh3max),xbh4(rsh4max,nehmax),xbh5(rsh5max),
     *   d1xbh3(nihmax),d2xbh3(nihmax),gamma(nngammax),
     *   d1gamma(nngammax),d2gamma(nngammax),s(0:nihmax),ds(maxds),
     *   ssg(nihmax),dlnla(nehmax*nfull),dlnlb(nehmax*nfull),
     *   ui(sumintdim),ddui(sumintdim),sigdd(sumdim2),dsigdd(sumdim2),
     *   stda4(nepmax),cdm(sumdim2),ddcdm(sumdim2),dcdmdsigii(sumdim4),
     *   dxbh(nehmax,0:maxnest),ddxbh(nehmax,maxnest),
     *   xblp(nelmax+nepmax,0:rspmax),dxblp(nelmax+nepmax,0:maxnest),
     *   ddxblp(nelmax+nepmax,maxnest),xbc(necmax,0:rscmax),
     *   dxbc(necmax,0:maxnest),ddxbc(necmax,maxnest),
     *   a5(nenmax,0:rsnamax),da5(nenmax,0:maxnest),
     *   dda5(nenmax,maxnest),p5(nebmax+nenmax+nepomax,0:rsnpmax),
     *   dp5(nebmax+nenmax+nepomax,0:maxnest),
     *   ddp5(nebmax+nenmax+nepomax,maxnest),
     *   xbml(nnemlmax,0:rsmlmax),dxbml(nnemlmax,0:maxnest),
     *   ddxbml(nnemlmax,maxnest)
      character parname(numpar)*12,fullname(nfull)*8
      integer indx(sumintdim),lampos(numdist,8),bdc(nepmax),
     *   thisdraw(numdist),
     *   intdist(maxnest),draw(numdist,nemax),cdraw(nemax)
 
c     Locally required scalars:
      double precision lnlobs,dpnan
      real obsweight
      integer id,neh,nel,nep,nec,neb,nen,nepo,neml,nneml,ne,neint,
     *   numnest,iobs,sumbigintdim,vpntr1,vpntr4,vpntr6,vpntr5,vpntr7,
     *   dimv,mp1,mp4,mp6,mp5,mp7,nmapmoel,dimscr,plusmin,status,
     *   intvecmax
      integer dimlam,dimarar,dimii,maxnepsumintdim,siga4a4,
     *   dsiga4a4,ddsiga4a4,tau,dtau,ddtau,siga6a6,dsiga6a6,ddsiga6a6,
     *   sigil6,dsigil6,cmui,dcmui,ddcmui,sigarar,dsigarar,sigl4l6,
     *   dsigl4l6,ddsigl4l6,lam,dlam,ddlam,epsh,ddepsh,epsl,ddepsl,epsp,
     *   ddepsp,epsc,ddepsc,epsb,ddepsb,epsn,ddepsn,epspo,ddepspo,epsml,
     *   ddepsml,sigii,dsigii,ddsigii,a6,da6,siga6a6ia6,cmua4,
     *   sigl4l6a6a6i
      logical ifdrv,isnan,opened
 
      status=0
 
c     Initialize function and derivatives to zero and determine ifdrv.
      call setupfunc(task,nfree,ifdrv,lnlall,fp,fpt,sp,spt)
 
c     Create covariance matrices of all distributions, not expanded.
c     Check that all are positive definite; fix if needed and allowed.
      call prepcov(1,nfull,numdist,sumdim2,distdim,distyp,distpos,
     *   distrestrpos,distrestr,fullbeta,dfullbeta,sigdd,dsigdd,
     *   fullname,posdef,ipt,jpt,status)
      if(status.ne.0)goto 810
 
c     Compute inverse of matrices and store after fullbeta(nfull)
      if(nfullplus.gt.nfull)call prepinv(1,nfull,nfullplus,fullbeta,
     *   dfullbeta,numpar,pardim,parpos,parname,ipt,jpt)
 
c     Check that vectors are increasing, if so defined.  (May be violated
c     by gridsearch.)
 
c     Check domain restrictions:
      call chkrange(nfull,iffull,fulltype,1,fullbeta,
     *   fullname,ipt,jpt,status)
      if(status.ne.0)goto 810
 
      do 300 iobs=1,scrnobs
         read(13,err=800,end=800)id,neh,nel,nep,nec,neb,nen,nepo,neml,
     *      nneml,neint,vpntr1,vpntr4,vpntr6,vpntr5,vpntr7,dimv,dimscr,
     *      mp1,mp4,mp6,mp5,mp7,nmapmoel,plusmin,obsweight
         ne=neh+nel+nep+nec+neb+nen+nepo+neml
         call read_scr(ne,numdist,sumintdim,nmapmoel,mapmoel,dimv,
     *      nested,cdraw,ndraw,dimused,lamtyp,v,status)
         if(status.ne.0)goto 800
         call codelamtyp(2,numdist,lamtyp)
c        If there is no integration, numnest=0.  However, we need the
c        space in dxblp and dxbc to unravel derivatives in prepall.
         numnest=max(1,intvecmax(nested,numdist))
 
c        Determine beginning positions in scratch arrays
         call figure_pos(maxcc,ccmax,numnest,numdist,sumbigintdim,
     *      nested,distdim,distyp,lamtyp,ndraw,lampos,ne,neh,nel,nep,
     *      nec,neb,nen,nepo,neml,nneml,dimscr,dimarar,dimii,dimlam,
     *      siga4a4,dsiga4a4,ddsiga4a4,
     *      tau,dtau,ddtau,siga6a6,dsiga6a6,ddsiga6a6,sigil6,
     *      dsigil6,sigarar,dsigarar,sigl4l6,dsigl4l6,ddsigl4l6,lam,
     *      dlam,ddlam,epsh,ddepsh,epsl,ddepsl,epsp,ddepsp,epsc,ddepsc,
     *      epsb,ddepsb,epsn,ddepsn,epspo,ddepspo,epsml,ddepsml,sigii,
     *      dsigii,ddsigii,a6,da6,siga6a6ia6,cmui,dcmui,ddcmui,cmua4,
     *      sigl4l6a6a6i,iid4,ipt,jpt)
         maxnepsumintdim=max(nep,sumintdim)
 
         call ithobs(nfull,nfullplus,ne,neh,nel,nep,nec,neb,nen,nepo,
     *      neml,nneml,nihmax,neint,numnest,numdist,sumintdim,sumdim2,
     *      sumdim4,sumbigintdim,dimlam,dimarar,dimii,maxnepsumintdim,
     *      maxpnts,intpnts,intpntr,numarma,vpntr1,vpntr4,vpntr6,vpntr5,
     *      vpntr7,dimv,mp1,mp4,mp6,mp5,mp7,nmapmoel,rsh3max,rsh4max,
     *      rsh5max,rspmax,rsnamax,rsnpmax,rscmax,rsmlmax,lnlobs,like,
     *      lnl,fullbeta,dfullbeta,ddfullbeta,z,wt,weight,ui,ddui,
     *      dd(cmui),dd(dcmui),dd(ddcmui),sigdd,dsigdd,
     *      cdm,ddcdm,dcdmdsigii,s,ds,ssg,xbh3,xbh4,xbh5,dxbh,ddxbh,xbc,
     *      dxbc,ddxbc,gamma,d1gamma,d2gamma,lnla,lnlb,d1xbh3,d2xbh3,
     *      dlnla,dlnlb,xblp,dxblp,ddxblp,stda4,a5,da5,dda5,p5,dp5,ddp5,
     *      xbml,dxbml,ddxbml,dd(lam),dd(dlam),dd(ddlam),dd(siga4a4),
     *      dd(dsiga4a4),dd(ddsiga4a4),dd(tau),dd(dtau),dd(ddtau),
     *      dd(siga6a6),dd(dsiga6a6),dd(ddsiga6a6),dd(sigil6),
     *      dd(dsigil6),dd(sigarar),dd(dsigarar),dd(sigl4l6),
     *      dd(dsigl4l6),dd(ddsigl4l6),dd(siga6a6ia6),dd(epsh),dd(epsl),
     *      dd(epsp),dd(epsc),dd(epsb),dd(epsn),dd(epspo),dd(epsml),
     *      dd(ddepsh),dd(ddepsl),dd(ddepsp),dd(ddepsc),dd(ddepsb),
     *      dd(ddepsn),dd(ddepspo),dd(ddepsml),dd(sigii),dd(dsigii),
     *      dd(ddsigii),dd(a6),dd(da6),dd(cmua4),dd(sigl4l6a6a6i),v,
     *      xnode,nested,distdim,distyp,lamtyp,distpos,lampos,bdc,
     *      thisdraw,draw,cdraw,ndraw,intdist,mapmoel,indx,dimused,
     *      refspl,beginspl,nodepos,numnodes,spltype,ifdrv,iflam,iftau,
     *      ifint4,ifint6,allint6,iid4,ipt,jpt)
 
         if(ifdrv)then
c           Unravel wrt inverse of matrices into wrt fullbeta
            if(nfullplus.gt.nfull)call prepinv(2,nfull,nfullplus,
     *         fullbeta,dfullbeta,numpar,pardim,parpos,parname,ipt,jpt)
c           Unravel derivatives wrt covariance matrices into wrt fullbeta
c           (Only the variance of the innovation term of ARMA processes
c           is dealt with here.)
            call prepcov(2,nfull,numdist,sumdim2,distdim,distyp,distpos,
     *         distrestrpos,distrestr,fullbeta,dfullbeta,sigdd,dsigdd,
     *         fullname,posdef,ipt,jpt,status)
         endif
 
         call accumfunc(task,iobs,id,scaleweight,obsweight,nfull,nfree,
     *      plusmin,fulltype,iffull,fullbeta,dfullbeta,dfreebeta,
     *      lnlall,lnlobs,fp,fpt,sp,spt,nchol,chol,ifdrv,ipt,jpt,warn)
         if(isnan(lnlall))goto 400
 
  300 continue
  400 rewind(13)
 
c     Accumfunc (accumfpsp) only accumulated the lower triangle; fill out here:
      if(mod(task,10).eq.1.or.mod(task,10).eq.2)call reflect(sp,nfree)
      if(mod(task,10).eq.1)call reflect(spt,nfree)
      goto 999
 
  800 if(ipt.gt.0.and.opened(8))write(8,801)
      if(jpt.gt.0)write(6,801)
  801 format(/,'ERROR or END-OF-FILE encountered while reading the ',
     *   'scratch data set.')
      stop
c     Control is passed here if covariance matrix is not positive definite
  810 lnlall=dpnan()
      goto 999
 
  999 return
      end
