      subroutine m2mult003(task,numrowa,numrowb,numblockc,dimblockc,
     *   a,b,cblock,factor)
 
c     Subroutine to multiply matrices A*B = C.  This routine
c     is similar to m2mult, but exploits special matrix properties.
c     Matrix types can be:
c        0 = generic
c        1 = identity (Only its dimension is stored.)
c        2 = symmetric
c        3 = symmetric and block-diagonal (Only its first block, its
c            dimension, and number of blocks are stored.)
c        4 = sparse in the sense that there is at most one nonzero
c            element in any one row.  (Only its nonzero elements
c            and their column position are stored.)
c
c     In this routine, A and B are generic, C block-diagonal.
c     C consists of numblockc blocks of dimension (dimblockc,dimblockc).
c     Only the first block is stored, as bblock(dimblockc,dimblockc).
c
c     Conceptual dimensions:
c        A(numrowa,numrowb) -- generic
c        B(numrowb,dimc)    -- generic
c        C(dimc,dimc)       -- symmetric and block-diagonal
c
c     where dimc = numblockc*dimblockc.  It must be the case that
c     numrowa=dimc, otherwise things would not be consistent.
c
c     Note: this routine multiplies A and B and collapses the results into
c     the top-left diagonal block of C.  It does not ensure that C is symmetric.
c     In real-life applications, C will often come out symmetric, but this
c     is not the case with generic A and B.  If symmetry is required,
c     follow this routine with "call makesym(dimblockc,cblock)".
c
c     Integer factor determines:
c        factor = 0     C = A*B
c              else     C = C + factor*A*B
c
c     Constantijn Panis, 3 December 2001
 
      implicit none
      integer task,numrowa,numrowb,numblockc,dimblockc,factor,
     *   i,j,k,ii,jj,ia,ib,ja,jb,sa,sb,jsa,jsb
      double precision a(numrowa*numrowb),b(numrowb*numrowa),
     *   cblock(dimblockc,dimblockc),abij
 
c     Check that numrowa=dimc:
      if(numrowa.ne.numblockc*dimblockc)call bug(1,1)
 
      if(factor.eq.0)call zerodp(dimblockc*dimblockc,cblock)
 
      if(mod(task/100,10).eq.0)then
         ja=numrowa
         jsa=1
      else
         ja=1
         jsa=numrowb
      endif
      if(mod(task/10,10).eq.0)then
         jb=1
         jsb=numrowb
      else
         jb=numrowa
         jsb=1
      endif
c     C is symmetric
 
      sa=1
      do 100 i=1,numrowa
         ii=mod(i-1,dimblockc)+1
         sb=1
         do 200 j=1,numrowa
            ib=sb
            sb=sb+jsb
            if((i-1)/dimblockc.ne.(j-1)/dimblockc)goto 200
            jj=mod(j-1,dimblockc)+1
            ia=sa
            abij=0.d0
            do 300 k=1,numrowb
               abij = abij + a(ia)*b(ib)
               ia=ia+ja
               ib=ib+jb
  300       continue
            if(factor.ne.0.and.factor.ne.1)abij=dble(factor)*abij
            cblock(ii,jj) = cblock(ii,jj) + abij
  200    continue
         sa=sa+jsa
  100 continue
 
      return
      end
 
