      subroutine m2mult340(numblocka,dimblocka,ncolc,ablock,bsparse,c,
     *   factor)
 
c     Subroutine to multiply two matrices A*B' = C.  This routine
c     is similar to m2mult, but exploits special matrix properties.
c     Matrix types can be:
c        0 = generic
c        1 = identity (Only its dimension is stored.)
c        2 = symmetric
c        3 = symmetric and block-diagonal (Only its first block, its
c            dimension, and number of blocks are stored.)
c        4 = sparse in the sense that there is at most one nonzero
c            element in any one row.  (Only its nonzero elements
c            and their column position are stored.)
c
c     In this routine, A is block-diagonal, B sparse, and C generic.
c     A consists of numblocka blocks of dimension (dimblocka,dimblocka).
c     Only the first block is stored, as ablock(dimblocka,dimblocka).
c     B is stored as two column vectors.  The first contains the column
c     number, the second the actual element.  In other words, B(i,.)=0
c     except B(i,bsparse(i,1)) = bsparse(i,2).  If bsparse(i,1)<=0, all
c     B(i,.)=0.
c
c     Conceptual dimensions:
c
c            A(dima,dima)   B(ncolc,dima)   C(dima,ncolc)
c
c     Integer factor determines:
c        factor = 0     C = A*B
c              else     C = C + factor*A*B
c
c     Constantijn Panis, 2 December 2001
 
      implicit none
      integer numblocka,dimblocka,ncolc,factor,i,j,k,ii,kk,dima
      double precision ablock(dimblocka,dimblocka),bsparse(ncolc,2),
     *   c(numblocka*dimblocka,ncolc),abij
 
      dima=numblocka*dimblocka
      if(factor.eq.0)call zerodp(dima*ncolc,c)
 
      do 100 i=1,dima
         ii=mod(i-1,dimblocka)+1
         do 200 j=1,ncolc
            k = int(bsparse(j,1))
            if((i-1)/dimblocka.ne.(k-1)/dimblocka)goto 200
            kk=mod(k-1,dimblocka)+1
            abij = ablock(ii,kk)*bsparse(j,2)
            if(factor.ne.0.and.factor.ne.1)abij=dble(factor)*abij
            c(i,j)=c(i,j)+abij
  200    continue
  100 continue
 
      return
      end
