      subroutine m3mult0003(task,numrowa,numrowb,numrowc,numblockd,
     *   dimblockd,a,b,c,dblock,factor)
 
c     Subroutine to multiply three matrices A*B*C = D.  This routine
c     is similar to m3mult, but exploits special matrix properties.
c     Matrix types can be:
c        0 = generic
c        1 = identity (Only its dimension is stored.)
c        2 = symmetric
c        3 = symmetric and block-diagonal (Only its first block, its
c            dimension, and number of blocks are stored.)
c        4 = sparse in the sense that there is at most one nonzero
c            element in any one row.  (Only its nonzero elements
c            and their column position are stored.)
c
c     In this routine, A, B, and C are generic, while D is block-diagonal.
c     D consists of numblockd blocks of dimension (dimblockd,dimblockd).
c     Only the first block is stored, as dblock(dimblockd,dimblockd).
c
c     Conceptual dimensions:
c        A(numrowa,numrowb) -- generic
c        B(numrowb,numrowc) -- generic
c        C(numrowc,dimd)    -- generic
c        D(dimd,dimd)       -- symmetric and block-diagonal
c
c     where dimd = numblockd*dimblockd.  It must be the case that
c     numrowa=dimd, otherwise things would not be consistent.
c
c     The four matrices may be in transposed format.  Integer task
c     may be 1000,0100,0010, et cetera, depending on which matrices
c     are transposed.  The first digit refers to A; the second to B;
c     the third to C; the fourth to D.  For example,
c        task = 1100 means that A and B are transposed: A'*B'*C = D
c        task =   10 means that C is transposed: A*B*C' = D
c
c     Dimensions as given above assume no transpositions; if a matrix
c     is transposed, its dimensions really are reversed.
c
c     Note: this routine multiplies A*B*C and collapses the results into
c     the top-left diagonal block of D.  It does not ensure that D is symmetric.
c     In real-life applications, D will often come out symmetric, but this
c     is not the case with generic A, B, C.  If symmetry is required,
c     follow this routine with "call makesym(dimblockd,dblock)".
c
c     Integer factor determines:
c        factor = 0     D = A*B*C
c              else     D = D + factor*A*B*C
c
c     Constantijn Panis, 3 March 2002
 
      implicit none
      integer task,numrowa,numrowb,numrowc,numblockd,dimblockd,
     *   factor,i,j,k,ia,ib,ja,jb,jc,sa,sb,jsa,jsb,jsc,ii,jj,cold,posc
      double precision a(numrowa*numrowb),b(numrowb*numrowc),
     *   c(numrowc*numrowa),dblock(dimblockd,dimblockd),abij
 
c     Check that numrowa=dimd:
      if(numrowa.ne.numblockd*dimblockd)call bug(1,1)
 
      if(factor.eq.0)call zerodp(dimblockd*dimblockd,dblock)
 
      if(mod(task/1000,10).eq.0)then
         ja=numrowa
         jsa=1
      else
         ja=1
         jsa=numrowb
      endif
      if(mod(task/100,10).eq.0)then
         jb=1
         jsb=numrowb
      else
         jb=numrowc
         jsb=1
      endif
      if(mod(task/10,10).eq.0)then
         jc=numrowc
         jsc=1
      else
         jc=1
         jsc=numrowa
      endif
c     D is symmetric
 
      sa=1
      do 210 i=1,numrowa
         ii=mod(i-1,dimblockd)+1
         sb=1
         do 220 j=1,numrowc
            abij = 0.d0
            ia=sa
            ib=sb
            do 230 k=1,numrowb
               abij = abij + a(ia)*b(ib)
               ia=ia+ja
               ib=ib+jb
  230       continue
            if(factor.ne.0.and.factor.ne.1)abij=dble(factor)*abij
c           D(i,.) =+ AB(i,j)*C(j,.)
c           Only innerblock of D is relevant
            do 240 jj=1,dimblockd
               cold=i-ii+jj
               posc=1+(j-1)*jsc+(cold-1)*jc
               dblock(ii,jj) = dblock(ii,jj) + abij*c(posc)
  240       continue
            sb=sb+jsb
  220    continue
         sa=sa+jsa
  210 continue
 
      return
      end
 
