      subroutine m3mult0430(task,nrowa,nrowb,numblockc,dimblockc,
     *   a,bsparse,cblock,d,factor)
 
c     Subroutine to multiply three matrices A*B*C = D.  This routine
c     is similar to m3mult, but exploits special matrix properties.
c     Matrix types can be:
c        0 = generic
c        1 = identity (Only its dimension is stored.)
c        2 = symmetric
c        3 = symmetric and block-diagonal (Only its first block, its
c            dimension, and number of blocks are stored.)
c        4 = sparse in the sense that there is at most one nonzero
c            element in any one row.  (Only its nonzero elements
c            and their column position are stored.)
c
c     In this routine, A is generic, B is sparse, C is block-diagonal,
c     and D is generic.
c     B is stored as two column vectors.  The first contains the column
c     number, the second the actual element.  In other words, B(i,.)=0
c     except B(i,bs(i,1)) = bs(i,2).  If bs(i,1)<=0, all B(i,.)=0.
c
c     C consists of numblockc blocks of dimension (dimblockc,dimblockc).
c     Only the first block is stored, as cblock(dimblockc,dimblockc).
c
c     Conceptual dimensions:
c        A(nrowa,nrowb)   -- generic
c        B(nrowb,dimc)    -- sparse
c        C(dimc,dimc)     -- symmetric and block-diagonal
c        D(nrowa,dimc)    -- sparse
c
c     where dimc = numblockc*dimblockc
c
c     Only matrix A may be in transposed format.  (C is symmetric and
c     transformations are irrelevant.)  Integer task may be 1000 or
c     0000, where
c        task = 1000 means that A is transposed: A'*B*C = D
c        task =    0 means that none is transposed: A*B*C = D
c
c     Integer factor determines:
c        factor = 0     D = A*B*C
c              else     D = D + factor*A*B*C
c
c     Constantijn Panis, 6 February 2002
 
      implicit none
      integer task,nrowa,nrowb,numblockc,dimblockc,factor,
     *   rowa,cold,cola,colb,dimc,ia,sa,ja,jsa,i,j
      double precision a(nrowa*nrowb),bsparse(nrowb,2),
     *   cblock(dimblockc,dimblockc),d(nrowa,numblockc*dimblockc)
 
      dimc=numblockc*dimblockc
      if(factor.eq.0)call zerodp(nrowa*dimc,d)
 
      if(mod(task/1000,10).eq.0)then
         ja=nrowa
         jsa=1
      else
         ja=1
         jsa=nrowb
      endif
c     B is sparse and must not be transposed.
      if(mod(task/100,10).ne.0)call bug(1,1)
c     C is symmetric
c     D is sparse and must not be transposed.
      if(mod(task,10).eq.1)call bug(1,1)
 
      sa=1
      do 100 rowa=1,nrowa
         do 200 cold=1,dimc
c           Compute D(rowa,cold), equal to the innerproduct of A(rowa,.) and BC(
c           B is sparse, so BC(cola,cold) is easy.
            ia=sa-ja
            do 300 cola=1,nrowb
               ia=ia+ja
               colb=int(bsparse(cola,1))
               if(colb.eq.0)goto 300
               if(int((colb-1)/dimblockc).ne.int((cold-1)/dimblockc))
     *            goto 300
               i=mod(colb-1,dimblockc)+1
               j=mod(cold-1,dimblockc)+1
               if(factor.eq.0.or.factor.eq.1)then
                  d(rowa,cold)=d(rowa,cold) +
     *               a(ia)*bsparse(cola,2)*cblock(i,j)
               else
                  d(rowa,cold)=d(rowa,cold) + dble(factor)*
     *               a(ia)*bsparse(cola,2)*cblock(i,j)
               endif
  300       continue
  200    continue
         sa=sa+jsa
  100 continue
 
      return
      end
 
