      subroutine m3mult0434(task,numrowa,numrowb,numblockc,dimblockc,
     *   a,bsparse,cblock,dsparse,factor)
 
c     Subroutine to multiply three matrices A*B*C = D.  This routine
c     is similar to m3mult, but exploits special matrix properties.
c     Matrix types can be:
c        0 = generic
c        1 = identity (Only its dimension is stored.)
c        2 = symmetric
c        3 = symmetric and block-diagonal (Only its first block, its
c            dimension, and number of blocks are stored.)
c        4 = sparse in the sense that there is at most one nonzero
c            element in any one row.  (Only its nonzero elements
c            and their column position are stored.)
c
c     In this routine, A is generic, B is sparse, C is block-diagonal,
c     and D is sparse.
c     B is stored as two column vectors.  The first contains the column
c     number, the second the actual element.  In other words, B(i,.)=0
c     except B(i,bs(i,1)) = bs(i,2).  If bs(i,1)<=0, all B(i,.)=0.
c     D is stored analogously.
c
c     D's sparsity structure must be input to this routine, i.e.,
c     dsparse(.,1) must contain column numbers.
c
c     C consists of numblockc blocks of dimension (dimblockc,dimblockc).
c     Only the first block is stored, as cblock(dimblockc,dimblockc).
c
c     Conceptual dimensions:
c        A(numrowa,numrowb) -- generic
c        B(numrowb,dimc)    -- sparse
c        C(dimc,dimc)       -- symmetric and block-diagonal
c        D(numrowa,dimc)    -- sparse
c
c     where dimc = numblockc*dimblockc
c
c     Only matrix A may be in transposed format.  (C is symmetric and
c     transformations are irrelevant.)  Integer task may be 1000/1010 or
c     0000/0010, where
c        task = 1000 means that A is transposed: A'*B*C = D
c        task =    0 means that none is transposed: A*B*C = D
c
c     Integer factor determines:
c        factor = 0     D = A*B*C'
c              else     D = D + factor*A*B*C'
c
c     Constantijn Panis, 3 March 2002
 
      implicit none
      integer task,numrowa,numrowb,numblockc,dimblockc,factor,
     *   rowd,cold,cola,colb,i,j,ja,jsa,posa
      double precision a(numrowa*numrowb),bsparse(numrowb,2),
     *   cblock(dimblockc,dimblockc),dsparse(numrowa,2)
 
      if(factor.eq.0)call zerodp(numrowa,dsparse(1,2))
 
      if(mod(task/1000,10).eq.0)then
         ja=numrowa
         jsa=1
      else
         ja=1
         jsa=numrowb
      endif
c     B is sparse and must not be transposed.
      if(mod(task/100,10).ne.0)call bug(1,1)
c     C is symmetric
c     D is sparse and must not be transposed.
      if(mod(task,10).eq.1)call bug(1,1)
 
      do 100 rowd=1,numrowa
         cold=int(dsparse(rowd,1))
         if(cold.eq.0)goto 100
c        D(rowd,cold) is the only relevant element in D(rowd,.).
c        It is the innerproduct of A(rowd,.) and BC(.,cold).
c        BC(i,cold) is the innerproduct of B(i,.) and C(.,cold).
c        Only one element B(i,.) is nonzero.
         do 200 cola=1,numrowb
            colb = int(bsparse(cola,1))
            if(colb.eq.0)goto 200
            if(int((colb-1)/dimblockc).ne.int((cold-1)/dimblockc))
     *         goto 200
            i=mod(colb-1,dimblockc)+1
            j=mod(cold-1,dimblockc)+1
c           location of A(rowd,cola):
            posa=1+(rowd-1)*jsa+(cola-1)*ja
            if(factor.eq.0.or.factor.eq.1)then
               dsparse(rowd,2)=dsparse(rowd,2) +
     *            a(posa)*bsparse(cola,2)*cblock(i,j)
            else
               dsparse(rowd,2)=dsparse(rowd,2) +
     *            dble(factor)*a(posa)*bsparse(cola,2)*cblock(i,j)
            endif
  200    continue
  100 continue
 
      return
      end
 
